package com.basic.eyflutter_uikit;

import android.app.Activity;

import com.basic.eyflutter_core.ChannelConstants;
import com.basic.eyflutter_core.channel.ChannelPlugin;
import com.basic.eyflutter_core.channel.OnDistributionSubscribe;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.PickersManager;
import com.basic.eyflutter_uikit.pickers.beans.PickerArgs;
import com.basic.eyflutter_uikit.pickers.beans.PickerModel;
import com.basic.eyflutter_uikit.pickers.enums.PickBehavior;
import com.basic.eyflutter_uikit.subscribe.Mp4ImageSubscribe;
import com.basic.eyflutter_uikit.subscribe.PermissionApplySubscribe;
import com.basic.eyflutter_uikit.subscribe.ToasterSubscribe;
import com.cloud.eyutils.ebus.EBus;
import com.cloud.eyutils.ebus.SubscribeEBus;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.mapper.MapperEntry;
import com.cloud.eyutils.utils.JsonUtils;
import com.cloud.eyutils.utils.ObjectJudge;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodChannel;

/**
 * EyflutterUikitPlugin
 */
public class EyflutterUikitPlugin implements FlutterPlugin {

    private List<String> verifyList = new LinkedList<>();
    private MethodChannel.Result pickerResult;
    private PickersManager pickersManager = new PickersManager();

    @Override
    public void onAttachedToEngine(FlutterPluginBinding binding) {
        EBus.getInstance().registered(this);
        ChannelPlugin.getInstance().putSubscribe(UiKitConstants.toastMethodName, new ToasterSubscribe());
        ChannelPlugin.getInstance().putSubscribe(UiKitConstants.mp4ImageProviderMethodName, new Mp4ImageSubscribe(verifyList));
        ChannelPlugin.getInstance().putSubscribe(PickersConstants.pickersKey, new PickersSubscribe());
        ChannelPlugin.getInstance().putSubscribe(ChannelConstants.permissionApply, new PermissionApplySubscribe());
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        EBus.getInstance().unregister(this);
        ChannelPlugin.getInstance().removeSubScribe(UiKitConstants.toastMethodName);
        ChannelPlugin.getInstance().removeSubScribe(UiKitConstants.mp4ImageProviderMethodName);
        ChannelPlugin.getInstance().removeSubScribe(PickersConstants.pickersKey);
        ChannelPlugin.getInstance().removeSubScribe(ChannelConstants.permissionApply);
    }

    private class PickersSubscribe extends OnDistributionSubscribe {

        @Override
        public void onSubscribe(MethodChannel.Result result, HashMap<String, Object> arguments) {
            if (ObjectJudge.isNullOrEmpty(arguments)) {
                result.success(null);
                return;
            }
            PickerArgs pickerArgs = MapperEntry.toEntity(arguments, PickerArgs.class);
            Activity recentlyCreateActivity = LauncherState.getInstance().getRecentlyCreateActivity();
            if (recentlyCreateActivity != null) {
                pickersManager.showPicker(recentlyCreateActivity, pickerArgs, result);
            }
            pickerResult = result;
        }
    }

    @SubscribeEBus(receiveKey = PickersConstants.pickerResultBusKey)
    public void onEventPickerResult(List<File> files, PickBehavior behavior) {
        if (pickerResult == null) {
            return;
        }
        List<PickerModel> models = new LinkedList<>();
        if (!ObjectJudge.isNullOrEmpty(files)) {
            for (File file : files) {
                PickerModel model = new PickerModel();
                model.setFilePath(file.getAbsolutePath());
                models.add(model);
            }
        }
        String json = JsonUtils.toJson(models);
        pickerResult.success(json);
        pickerResult = null;
        EBus.getInstance().post(PickersConstants.finishPickerBusKey);
    }
}
