package com.basic.eyflutter_uikit.dialogs;

import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.FrameLayout;

import androidx.annotation.IdRes;

import com.basic.eyflutter_uikit.R;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.logs.Logger;
import com.cloud.eyutils.manager.ObjectManager;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-06-19
 * Description:浮动视图
 * Modifier:
 * ModifyContent:
 */
public class BaseFloatViewDialog {

    protected void onMatchView(View view) {
        //匹配视图
    }

    protected void addViewForActivity(Activity activity, View view, int statusBarHeight, CharSequence defText) {
        if (activity == null) {
            return;
        }
        Window window = activity.getWindow();
        View decorView = window.getDecorView();
        if (!(decorView instanceof FrameLayout)) {
            return;
        }
        int viewId = view.getId();
        FrameLayout frameLayout = (FrameLayout) decorView;
        try {
            //根据视图id查找是否存在
            View oldView = frameLayout.findViewById(viewId);
            if (oldView != null) {
                onMatchView(oldView);
                return;
            }
            FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT);
            layoutParams.topMargin = statusBarHeight;
            layoutParams.leftMargin = 0;
            onMatchView(view);
            frameLayout.addView(view, layoutParams);
        } catch (Exception e) {
            Logger.error(e);
        }
    }

    public static void removeFloatViewForActivity(Activity activity, int viewId) {
        if (activity == null) {
            return;
        }
        Window window = activity.getWindow();
        View decorView = window.getDecorView();
        if (!(decorView instanceof FrameLayout)) {
            return;
        }
        FrameLayout frameLayout = (FrameLayout) decorView;
        View view = frameLayout.findViewById(viewId);
        if (view == null) {
            return;
        }
        frameLayout.removeView(view);
    }

    /**
     * 添加浮动视图
     *
     * @param context 上下文
     * @param defText 默认文本
     */
    protected void addFloatView(Context context, CharSequence defText) {
        if (context == null) {
            return;
        }
        View view = View.inflate(context, R.layout.cl_top_tip_view, null);
        if (view.getId() == View.NO_ID) {
            return;
        }
        Activity createActivity = LauncherState.getInstance().getRecentlyCreateActivity();
        if (createActivity == null) {
            return;
        }
        int statusBarHeight = ObjectManager.getStatusBarHeight(context);
        addViewForActivity(createActivity, view, statusBarHeight, defText);
    }

    /**
     * 移除浮动视图
     *
     * @param layoutViewId 浮动视图id
     */
    protected void removeFloatView(@IdRes int layoutViewId) {
        if (layoutViewId == 0 || layoutViewId == View.NO_ID) {
            return;
        }
        Activity createActivity = LauncherState.getInstance().getRecentlyCreateActivity();
        if (createActivity == null) {
            return;
        }
        removeFloatViewForActivity(createActivity, layoutViewId);
    }

    /**
     * 根据布局视图id匹配view
     *
     * @param layoutViewId 布局视图id
     */
    protected void matchVeiws(int layoutViewId) {
        Activity createActivity = LauncherState.getInstance().getRecentlyCreateActivity();
        if (createActivity == null) {
            return;
        }
        Window window = createActivity.getWindow();
        View decorView = window.getDecorView();
        if (!(decorView instanceof FrameLayout)) {
            return;
        }
        FrameLayout frameLayout = (FrameLayout) decorView;
        View view = frameLayout.findViewById(layoutViewId);
        if (view == null) {
            return;
        }
        onMatchView(view);
    }
}
