package com.basic.eyflutter_uikit.dialogs;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.TextView;

import com.basic.eyflutter_uikit.R;
import com.cloud.eyutils.HandlerManager;
import com.cloud.eyutils.events.RunnableParamsN;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.launchs.events.OnActivityLifeCycleCreated;
import com.cloud.eyutils.launchs.events.OnActivityLifeCycleDestroyed;
import com.cloud.eyutils.manager.ObjectManager;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.SharedPrefUtils;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-06-19
 * Description:页面顶部半透明提示视图
 * Modifier:
 * ModifyContent:
 */
public class FloatTipBarDialog extends BaseFloatViewDialog implements OnActivityLifeCycleCreated, OnActivityLifeCycleDestroyed {

    private CharSequence currText;
    private Context context;

    /**
     * 显示浮动视图
     *
     * @param context 上下文
     * @param defText 默认文本
     */
    public void show(Context context, CharSequence defText) {
        if (context == null) {
            return;
        }
        this.context = context;
        this.currText = defText;
        HandlerManager.getInstance().post(new RunnableParamsN<Object>() {
            @Override
            public void run(Object... objects) {
                SharedPrefUtils.setPrefBoolean(FloatTipBarDialog.this.context, "$_global_float_dialog_destoryed", false);
                LauncherState launcherState = LauncherState.getInstance();
                launcherState.setOnActivityLifeCycleCreated(FloatTipBarDialog.this);
                launcherState.setOnActivityLifeCycleDestroyed(FloatTipBarDialog.this);
                addFloatView(FloatTipBarDialog.this.context, FloatTipBarDialog.this.currText);
            }
        });
    }

    /**
     * 更新文本
     *
     * @param text 将要更新的文本
     */
    public void notifyTextView(CharSequence text) {
        if (TextUtils.isEmpty(text)) {
            return;
        }
        this.currText = text;
        if (ObjectJudge.isMainThread()) {
            super.matchVeiws(R.id.cl_tip_view_rl);
        } else {
            HandlerManager.getInstance().post(new RunnableParamsN<Object>() {
                @Override
                public void run(Object... objects) {
                    matchVeiws(R.id.cl_tip_view_rl);
                }
            });
        }
    }

    @Override
    protected void onMatchView(View view) {
        View mview = view.findViewById(R.id.cl_tip_tv);
        if (!(mview instanceof TextView)) {
            return;
        }
        TextView textView = (TextView) mview;
        textView.setText(currText);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        View view = View.inflate(activity, R.layout.cl_top_tip_view, null);
        int statusBarHeight = ObjectManager.getStatusBarHeight(activity);
        super.addViewForActivity(activity, view, statusBarHeight, currText);
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        BaseFloatViewDialog.removeFloatViewForActivity(activity, R.id.cl_tip_view_rl);
    }

    /**
     * 销毁视图
     */
    public void dismiss() {
        //设置状态为关闭，以便之后打开的页面如果存在view则移除
        if (context != null) {
            SharedPrefUtils.setPrefBoolean(context, "$_global_float_dialog_destoryed", true);
        }
        //移除视图
        if (ObjectJudge.isMainThread()) {
            super.removeFloatView(R.id.cl_tip_view_rl);
        } else {
            HandlerManager.getInstance().post(new RunnableParamsN<Object>() {
                @Override
                public void run(Object... objects) {
                    removeFloatView(R.id.cl_tip_view_rl);
                }
            });
        }
    }
}
