package com.basic.eyflutter_uikit.dialogs;

import android.content.Context;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.text.method.PasswordTransformationMethod;
import android.view.Gravity;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.beans.ActionItem;
import com.basic.eyflutter_uikit.enums.DialogButtonsEnum;
import com.basic.eyflutter_uikit.enums.InputType;
import com.basic.eyflutter_uikit.enums.MsgBoxClickButtonEnum;
import com.cloud.eyutils.utils.ValidUtils;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2018/9/26
 * Description:带输入的消息提示框
 * Modifier:
 * ModifyContent:
 */
public class InputMessageBox {

    //文本输入类型
    private InputType inputType = InputType.text;
    //输入正则
    private String inputRegular = "";

    public boolean onMessageCall(String text, MsgBoxClickButtonEnum mcbenum, Object extras) {
        return true;
    }

    public boolean onMessageCall(String text, String cmdid, Object extras) {
        return true;
    }

    class InputMessageBoxView {

        public TextView messageTv;
        public EditText messageEt;
        public View contentView;

        public InputMessageBoxView(Context context) {
            contentView = View.inflate(context, R.layout.dialog_message_input_view, null);
            messageTv = (TextView) contentView.findViewById(R.id.cl_message_tv);
            messageEt = (EditText) contentView.findViewById(R.id.cl_message_et);
        }
    }

    /**
     * 文本框输入类型
     *
     * @param inputType inputType
     */
    public void setInputType(InputType inputType) {
        if (inputType == null) {
            return;
        }
        this.inputType = inputType;
    }

    /**
     * 输入框输入规范
     *
     * @param inputRegular 正则
     */
    public void setInputRegular(String inputRegular) {
        this.inputRegular = inputRegular;
    }

    private void show(Context context,
                      String title,
                      String content,
                      DialogButtonsEnum btnsenum,
                      ActionItem[] cmds,
                      Object extras) {
        BaseMessageBox messageBox = new BaseMessageBox() {
            @Override
            public boolean onItemClickListener(View v, MsgBoxClickButtonEnum mcbenum, String target, Object extraData) {
                if (TextUtils.equals(target, "cl_input_message_dialog") && (extraData instanceof EditText)) {
                    EditText editText = (EditText) extraData;
                    if (editText == null) {
                        return true;
                    }
                    String text = editText.getText().toString().trim();
                    return onMessageCall(text, mcbenum, editText.getTag());
                }
                return super.onItemClickListener(v, mcbenum, target, extraData);
            }

            @Override
            public boolean onItemClickListener(View v, String cmdid, String target, Object extraData) {
                if (TextUtils.equals(target, "cl_input_message_dialog") && (extraData instanceof EditText)) {
                    EditText editText = (EditText) extraData;
                    if (editText == null) {
                        return true;
                    }
                    String text = editText.getText().toString().trim();
                    return onMessageCall(text, cmdid, editText.getTag());
                }
                return super.onItemClickListener(v, cmdid, target, extraData);
            }
        };
        if (TextUtils.isEmpty(title)) {
            messageBox.setShowTitle(false);
        } else {
            messageBox.setShowTitle(true);
            messageBox.setTitle(title);
        }
        messageBox.setShowClose(false);
        messageBox.setCancelable(false);
        messageBox.setTitleGravity(Gravity.CENTER);
        final InputMessageBoxView boxView = new InputMessageBoxView(context);
        boxView.messageTv.setText(content);
        messageBox.setContentView(boxView.contentView);
        messageBox.setButtons(cmds);
        //输入类型
        if (inputType == InputType.number) {
            boxView.messageEt.setInputType(android.text.InputType.TYPE_CLASS_NUMBER);
        } else if (inputType == InputType.password) {
            boxView.messageEt.setTransformationMethod(PasswordTransformationMethod.getInstance());
        } else {
            boxView.messageEt.setInputType(android.text.InputType.TYPE_CLASS_TEXT);
        }
        //输入限制
        if (!TextUtils.isEmpty(inputRegular)) {
            boxView.messageEt.addTextChangedListener(new TextWatcher() {
                @Override
                public void beforeTextChanged(CharSequence s, int start, int count, int after) {

                }

                @Override
                public void onTextChanged(CharSequence s, int start, int before, int count) {
                    String editable = boxView.messageEt.getText().toString();
                    String matche = ValidUtils.matche(inputRegular, editable);
                    if (!editable.equals(matche)) {
                        boxView.messageEt.setText(matche);
                        boxView.messageEt.setSelection(matche.length());
                    }
                }

                @Override
                public void afterTextChanged(Editable s) {

                }
            });
        }
        //扩展限制
        boxView.messageEt.setTag(extras);
        messageBox.setTarget("cl_input_message_dialog", boxView.messageEt);
        messageBox.show(context, btnsenum);
    }

    /**
     * 显示带输入消息提示框
     *
     * @param context
     * @param title    标题
     * @param content  内容
     * @param btnsenum 除DialogButtonsEnum.Custom外
     * @param extras   扩展参数
     */
    public void show(Context context, String title, String content, DialogButtonsEnum btnsenum, Object extras) {
        this.show(context, title, content, btnsenum, null, extras);
    }

    /**
     * 显示带输入消息提示框
     *
     * @param context
     * @param title    标题
     * @param content  内容
     * @param btnsenum 除DialogButtonsEnum.Custom外
     */
    public void show(Context context, String title, String content, DialogButtonsEnum btnsenum) {
        show(context, title, content, btnsenum, null);
    }

    /**
     * 显示带输入消息提示框
     *
     * @param context
     * @param title   标题
     * @param content 内容
     * @param cmds    自定义按钮
     * @param extras  扩展参数
     */
    public void show(Context context, String title, String content, ActionItem[] cmds, Object extras) {
        this.show(context, title, content, DialogButtonsEnum.Custom, cmds, extras);
    }

    /**
     * 显示带输入消息提示框
     *
     * @param context
     * @param title   标题
     * @param content 内容
     * @param cmds    自定义按钮
     */
    public void show(Context context, String title, String content, ActionItem[] cmds) {
        show(context, title, content, cmds, null);
    }
}
