package com.basic.eyflutter_uikit.dialogs;

import android.content.Context;
import android.view.View;

import com.basic.eyflutter_uikit.beans.ActionItem;
import com.basic.eyflutter_uikit.enums.DialogButtonsEnum;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.ViewUtils;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2020/7/27
 * Description:
 * Modifier:
 * ModifyContent:
 */
public class MessageBox extends BaseMessageBox {

    protected void onActionClick(View view, int id, ActionItem actionItem) {
        //action click
    }

    private ActionItem[] actionItems;

    //设置dialog布局id
    public void setLayoutId(Context context, int layoutId) {
        if (context == null || layoutId == 0) {
            return;
        }
        View view = View.inflate(context, layoutId, null);
        super.setContentView(view);
    }

    /**
     * 设置视图操作事件
     *
     * @param actionItems 其中{@link ActionItem#getActionId()}为视图id
     */
    public void setActionItems(ActionItem... actionItems) {
        this.actionItems = actionItems;
    }

    public void show(Context context, int contentWidth) {
        if (context == null) {
            return;
        }
        bindViewEvents();
        super.show(context, DialogButtonsEnum.Custom, contentWidth);
    }

    public void show(Context context) {
        this.show(context, 0);
    }

    private void bindViewEvents() {
        if (ObjectJudge.isNullOrEmpty(actionItems)) {
            return;
        }
        View contentView = super.getContentView();
        if (contentView == null) {
            return;
        }
        for (ActionItem actionItem : actionItems) {
            Object actionId = actionItem.getActionId();
            if (!(actionId instanceof Integer)) {
                continue;
            }
            int id = ConvertUtils.toInt(actionId);
            View view = ViewUtils.bindView(contentView, id);
            if (view == null) {
                continue;
            }
            view.setTag(actionItem);
            view.setOnClickListener(this::onClickEvent);
        }
    }

    private void onClickEvent(View view) {
        Object tag = view.getTag();
        if (!(tag instanceof ActionItem)) {
            return;
        }
        int id = view.getId();
        ActionItem actionItem = (ActionItem) tag;
        onActionClick(view, id, actionItem);
    }
}
