package com.basic.eyflutter_uikit.permission;

import android.text.TextUtils;

import com.basic.eyflutter_uikit.enums.PermissionGrantedType;
import com.cloud.eyutils.utils.ConvertUtils;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-05-07
 * Description:授予消费者
 * Modifier:
 * ModifyContent:
 */
public abstract class OnGrantedConsumer {

    private Object[] extras;
    private Map<CheckDataEnums, List<Set<String>>> permissionGroup = new HashMap<CheckDataEnums, List<Set<String>>>();
    //需要强提醒权限项
    private Set<String> strongReminderPermissions = new HashSet<String>();
    public OnGrantedConsumer() {

    }

    public OnGrantedConsumer(Object... extras) {
        this.extras = extras;
    }

    //get extras
    public Object[] getExtras() {
        return extras;
    }

    public Map<CheckDataEnums, List<Set<String>>> getPermissionGroup() {
        return permissionGroup;
    }

    /**
     * 获取强提醒权限项(以、号分隔)
     *
     * @return 权限描述
     */
    public String getSRPItems() {
        return ConvertUtils.toJoin(strongReminderPermissions, "、");
    }

    public void addSRPItems(String srpitem) {
        if (TextUtils.isEmpty(srpitem)) {
            return;
        }
        strongReminderPermissions.add(srpitem);
    }

    /**
     * 检测权限(申请之前回调)
     *
     * @param detectionStates permission-state
     */
    public void detection(Map<String, PermissionGrantedType> detectionStates) {
        //可用于判断对应权限是否弹出系统申请框
    }

    /**
     * Consume the given value.
     *
     * @param grantedType 检测授予类型
     */
    public void accept(PermissionGrantedType grantedType) {

    }

    /**
     * 各权限回调时授权状态
     *
     * @param authStates     各权限授权状态
     * @param isApplySuccess 是否申请成功
     */
    public void acceptEach(Map<String, PermissionState> authStates, boolean isApplySuccess) {

    }

    /**
     * Consume the given value.
     *
     * @param grantedType 检测授予类型
     * @param extras      扩展数据
     */
    public void accept(PermissionGrantedType grantedType, Object[] extras) {

    }
}
