package com.basic.eyflutter_uikit.permission;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.hardware.Camera;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Build;
import android.text.TextUtils;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;

import com.basic.eyflutter_uikit.enums.PermissionGrantedType;
import com.basic.eyflutter_uikit.permission.detections.FloatWindowPermissionDetection;
import com.basic.eyflutter_uikit.permission.detections.OnPermissionCheckPromptConsumer;
import com.cloud.eyutils.HandlerManager;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.events.RunnableParamsN;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.logs.Logger;
import com.cloud.eyutils.observable.BaseObservable;
import com.cloud.eyutils.observable.call.OnSubscribeConsumer;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.GlobalUtils;
import com.cloud.eyutils.utils.JsonUtils;
import com.cloud.eyutils.utils.NetworkUtils;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.SharedPrefUtils;
import com.tbruyelle.rxpermissions3.Permission;
import com.tbruyelle.rxpermissions3.RxPermissions;
import com.yanzhenjie.permission.Action;
import com.yanzhenjie.permission.AndPermission;
import com.yanzhenjie.permission.option.Option;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.functions.Consumer;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-05-07
 * Description:权限管理
 * Modifier:
 * ModifyContent:
 */
public class PermissionManager extends BaseObservable {

    private static PermissionManager permissionManager;
    private static HashMap<String, DisposableItem> mCompositeDisposable = new HashMap<>();
    //适配权限列表
    private Set<String> adapterPermissions = new HashSet<>();
    //权限映射列表
    private Map<String, String> permissionMapperMap = new HashMap<String, String>();
    private LinkedList<Set<String>> permissions = new LinkedList<>();

    private PermissionManager() {
        //init
    }

    private static class DisposableItem {
        Disposable disposable;
        Object object;
        String key;

        private DisposableItem(Disposable disposable, String key, Object object) {
            this.disposable = disposable;
            this.key = key;
            this.object = object;
        }
    }

    //获取实例
    public static PermissionManager getInstance() {
        if (permissionManager == null) {
            synchronized (PermissionManager.class) {
                if (permissionManager == null) {
                    permissionManager = new PermissionManager();
                }
            }
        }
        return permissionManager;
    }

    private Map<String, String> getPermissionMapperMap() {
        if (!ObjectJudge.isNullOrEmpty(permissionMapperMap)) {
            return permissionMapperMap;
        }
        permissionMapperMap.put(Manifest.permission.SEND_SMS, "短信");
        permissionMapperMap.put(Manifest.permission.RECEIVE_SMS, "短信");
        permissionMapperMap.put(Manifest.permission.READ_SMS, "短信");
        permissionMapperMap.put(Manifest.permission.RECEIVE_WAP_PUSH, "短信");
        permissionMapperMap.put(Manifest.permission.RECEIVE_MMS, "短信");
        if (Build.VERSION.SDK_INT >= 16) {
            permissionMapperMap.put(Manifest.permission.READ_EXTERNAL_STORAGE, "存储");
        }
        permissionMapperMap.put(Manifest.permission.WRITE_EXTERNAL_STORAGE, "存储");
        permissionMapperMap.put(Manifest.permission.READ_CONTACTS, "通讯录");
        permissionMapperMap.put(Manifest.permission.WRITE_CONTACTS, "通讯录");
        permissionMapperMap.put(Manifest.permission.GET_ACCOUNTS, "获取联系人帐户信息");
        permissionMapperMap.put(Manifest.permission.READ_PHONE_STATE, "读取手机状态信息");
        permissionMapperMap.put(Manifest.permission.CALL_PHONE, "拨打电话");
        if (Build.VERSION.SDK_INT >= 16) {
            permissionMapperMap.put(Manifest.permission.READ_CALL_LOG, "手机通话日志");
            permissionMapperMap.put(Manifest.permission.WRITE_CALL_LOG, "手机通话日志");
        }
        permissionMapperMap.put(Manifest.permission.ADD_VOICEMAIL, "语音信箱");
        permissionMapperMap.put(Manifest.permission.CAMERA, "摄像头");
        permissionMapperMap.put(Manifest.permission.RECORD_AUDIO, "麦克风");
        permissionMapperMap.put(Manifest.permission.ACCESS_FINE_LOCATION, "定位");
        permissionMapperMap.put(Manifest.permission.ACCESS_COARSE_LOCATION, "定位");
        return permissionMapperMap;
    }

    /**
     * 适配权限列表
     *
     * @param permissions 需要做兼容检测的权限
     */
    public void setAdapterPermissions(String... permissions) {
        if (ObjectJudge.isNullOrEmpty(permissions)) {
            return;
        }
        super.buildSubscribe(permissions, new OnSubscribeConsumer<String[], Object>() {
            @Override
            public void onSubscribe(String[] permissions, Object o) {
                for (String permission : permissions) {
                    adapterPermissions.add(permission);
                }
                String json = JsonUtils.toJson(permissions);
                Context applicationContext = LauncherState.getApplicationContext();
                SharedPrefUtils.setPrefString(applicationContext, "$_adapter_permission_key", json);
            }
        }, null, null);
    }

    /**
     * 获取适配权限列表
     *
     * @return permissions
     */
    public Set<String> getAdapterPermissions() {
        if (!ObjectJudge.isNullOrEmpty(adapterPermissions)) {
            return adapterPermissions;
        }
        Context applicationContext = LauncherState.getApplicationContext();
        String permissionsJson = SharedPrefUtils.getPrefString(applicationContext, "$_adapter_permission_key");
        Set<String> sets = JsonUtils.parseT(permissionsJson, HashSet.class);
        if (sets == null) {
            return adapterPermissions;
        } else {
            adapterPermissions.addAll(sets);
        }
        return adapterPermissions;
    }

    private void applyRxPermission(Object object, List<Set<String>> permissionArray, OnGrantedConsumer consumer, CheckDataEnums checkDataEnums, Map<String, PermissionState> authStates) {
        permissions.addAll(permissionArray);
        if (permissions.size() > 0) {
            Set<String> remove = permissions.remove(0);
            requestPermission(object, remove, consumer, checkDataEnums, authStates);
        }
    }

    private void requestPermission(Object object, Set<String> permissionArray, OnGrantedConsumer consumer, CheckDataEnums checkDataEnums, Map<String, PermissionState> authStates) {
        RxPermissions rxPermissions = getRxPermissions(object);
        if (rxPermissions == null) {
            return;
        }
        String guid = GlobalUtils.getNewGuid();
        TargetConsumer targetConsumer = new TargetConsumer(object, guid, consumer, checkDataEnums, authStates);
        Disposable disposable = rxPermissions
                .requestEachCombined(ConvertUtils.toArray(permissionArray))
                .subscribe(targetConsumer, new TargetErrorConsumer(consumer, authStates));
        mCompositeDisposable.put(guid, new DisposableItem(disposable, guid, object));
    }

    private void preparePermissions(OnGrantedConsumer consumer, HashMap<String, Set<String>> permissions) {
        Map<CheckDataEnums, List<Set<String>>> permissionGroup = consumer.getPermissionGroup();
        permissionGroup.clear();
        if (!permissionGroup.containsKey(CheckDataEnums.adapter)) {
            permissionGroup.put(CheckDataEnums.adapter, new LinkedList<Set<String>>());
        }
        if (!permissionGroup.containsKey(CheckDataEnums.normal)) {
            permissionGroup.put(CheckDataEnums.normal, new LinkedList<Set<String>>());
        }
        //如果权限在适配列表中则先用AndPermission检测
        Set<String> adapterPermissions = getAdapterPermissions();
        for (Map.Entry<String, Set<String>> entry : permissions.entrySet()) {
            Set<String> value = entry.getValue();
            boolean flag = false;
            if (value.size() > 0) {
                for (String s : value) {
                    flag = adapterPermissions.contains(s);
                    if (flag) {
                        break;
                    }
                }
            }
            List<Set<String>> sets = permissionGroup.get(flag ? CheckDataEnums.adapter : CheckDataEnums.normal);
            sets.add(value);
        }
    }

    /**
     * 申请权限
     *
     * @param fragmentActivity fragmentActivity
     * @param consumer         授予权限监听
     * @param permissions      权限{@link Manifest.permission}
     */
    public void apply(FragmentActivity fragmentActivity, OnGrantedConsumer consumer, HashMap<String, Set<String>> permissions) {
        if (fragmentActivity == null || ObjectJudge.isNullOrEmpty(permissions)) {
            //要检测的权限为空或页面为空则不检测
            return;
        }
        if (fragmentActivity.isFinishing() || (Build.VERSION.SDK_INT >= 17 && fragmentActivity.isDestroyed())) {
            //如果页面已销毁则不检测
            return;
        }
        Map<String, PermissionGrantedType> detectionMap = applyBeforeDetection(fragmentActivity, permissions);
        Map<String, PermissionState> authStates = toAuthStates(detectionMap);
        //先回调检测结果
        if (consumer != null) {
            consumer.detection(detectionMap);
        }
        preparePermissions(consumer, permissions);
        check(fragmentActivity, consumer, CheckDataEnums.adapter, authStates);
    }

    private Map<String, PermissionState> toAuthStates(Map<String, PermissionGrantedType> detectionMap) {
        Map<String, PermissionState> authStates = new HashMap<>();
        for (Map.Entry<String, PermissionGrantedType> entry : detectionMap.entrySet()) {
            PermissionState permissionState = new PermissionState();
            permissionState.setPreviousGrantedType(entry.getValue());
            permissionState.setGrantedType(PermissionGrantedType.none);
            authStates.put(entry.getKey(), permissionState);
        }
        return authStates;
    }

    private Map<String, PermissionGrantedType> applyBeforeDetection(Object object, HashMap<String, Set<String>> permissions) {
        Map<String, PermissionGrantedType> detection = new HashMap<>();
        for (Map.Entry<String, Set<String>> entry : permissions.entrySet()) {
            Set<String> value = entry.getValue();
            for (String s : value) {
                boolean granted = isAndGranted(object, s);
                if (granted) {
                    if (TextUtils.equals(s.trim(), Manifest.permission.ACCESS_FINE_LOCATION)) {
                        //如果是gps权限则调用service
                        if (NetworkUtils.isEnableLocation()) {
                            detection.put(s, PermissionGrantedType.granted);
                        } else {
                            detection.put(s, PermissionGrantedType.previousNotGranted);
                        }
                    } else {
                        detection.put(s, PermissionGrantedType.granted);
                    }
                } else {
                    detection.put(s, PermissionGrantedType.previousNotGranted);
                }
            }
        }
        return detection;
    }

    /**
     * 检测权限
     *
     * @param fragmentActivity fragmentActivity
     * @param permissions      permissions
     * @return permission-state
     */
    public Map<String, PermissionGrantedType> detectionPermissions(FragmentActivity fragmentActivity, HashMap<String, Set<String>> permissions) {
        return applyBeforeDetection(fragmentActivity, permissions);
    }

    /**
     * 检测权限
     *
     * @param fragment    fragment
     * @param permissions permissions
     * @return permission-state
     */
    public Map<String, PermissionGrantedType> detectionPermissions(Fragment fragment, HashMap<String, Set<String>> permissions) {
        return applyBeforeDetection(fragment, permissions);
    }

    //and permission检测
    private boolean isAndGranted(Object object, String permission) {
        boolean flag = false;
        if (object instanceof FragmentActivity) {
            flag = AndPermission.hasPermissions((FragmentActivity) object, permission);
        } else if (object instanceof Fragment) {
            flag = AndPermission.hasPermissions((Fragment) object, permission);
        }
        return flag;
    }

    private boolean isAndAlwaysDenied(Object object, String permission) {
        boolean flag = false;
        if (object instanceof FragmentActivity) {
            flag = AndPermission.hasAlwaysDeniedPermission((FragmentActivity) object, permission);
        } else if (object instanceof Fragment) {
            flag = AndPermission.hasAlwaysDeniedPermission((Fragment) object, permission);
        }
        return flag;
    }

    //rx permission检测
    private boolean isRxGranted(Object object, String permission) {
        boolean flag = false;
        RxPermissions rxPermissions = null;
        if (object instanceof FragmentActivity) {
            rxPermissions = new RxPermissions((FragmentActivity) object);
            flag = rxPermissions.isGranted(permission);
        } else if (object instanceof Fragment) {
            rxPermissions = new RxPermissions((Fragment) object);
            flag = rxPermissions.isGranted(permission);
        }
        return flag;
    }

    //rx permission检测
    private boolean isRxRevoked(Object object, String permission) {
        boolean flag = false;
        RxPermissions rxPermissions = null;
        if (object instanceof FragmentActivity) {
            rxPermissions = new RxPermissions((FragmentActivity) object);
            flag = rxPermissions.isRevoked(permission);
        } else if (object instanceof Fragment) {
            rxPermissions = new RxPermissions((Fragment) object);
            flag = rxPermissions.isRevoked(permission);
        }
        return flag;
    }

    /**
     * 仅通过RxPermission来处理
     *
     * @param fragmentActivity fragmentActivity
     * @param consumer         授予权限监听
     * @param permissions      权限{@link Manifest.permission}
     */
    public void weakApply(FragmentActivity fragmentActivity, OnGrantedConsumer consumer, HashMap<String, Set<String>> permissions) {
        if (fragmentActivity == null || ObjectJudge.isNullOrEmpty(permissions)) {
            //要检测的权限为空或页面为空则不检测
            return;
        }
        if (fragmentActivity.isFinishing() || (Build.VERSION.SDK_INT >= 17 && fragmentActivity.isDestroyed())) {
            //如果页面已销毁则不检测
            return;
        }
        RxPermissions rxPermissions = getRxPermissions(fragmentActivity);
        if (rxPermissions == null) {
            return;
        }
        Map<String, PermissionGrantedType> detectionMap = applyBeforeDetection(fragmentActivity, permissions);
        Map<String, PermissionState> authStates = toAuthStates(detectionMap);
        //先回调检测结果
        if (consumer != null) {
            consumer.detection(detectionMap);
        }
        List<Set<String>> sets = new LinkedList<>();
        for (Map.Entry<String, Set<String>> entry : permissions.entrySet()) {
            sets.add(entry.getValue());
        }
        applyRxPermission(fragmentActivity, sets, consumer, CheckDataEnums.normal, authStates);
    }

    private RxPermissions getRxPermissions(Object object) {
        RxPermissions rxPermissions = null;
        if (object instanceof FragmentActivity) {
            rxPermissions = new RxPermissions((FragmentActivity) object);
        } else if (object instanceof Fragment) {
            rxPermissions = new RxPermissions((Fragment) object);
        }
        return rxPermissions;
    }

    private Option getAndPermissionOption(Object object) {
        Option option = null;
        if (object instanceof FragmentActivity) {
            option = AndPermission.with((FragmentActivity) object);
        } else if (object instanceof Fragment) {
            option = AndPermission.with((Fragment) object);
        }
        return option;
    }

    private void check(final Object object, final OnGrantedConsumer consumer, final CheckDataEnums checkDataEnums, final Map<String, PermissionState> authStates) {
        List<Set<String>> plist = null;
        Map<CheckDataEnums, List<Set<String>>> permissionGroup = consumer.getPermissionGroup();
        if (checkDataEnums == CheckDataEnums.adapter) {
            if (!permissionGroup.containsKey(checkDataEnums)) {
                //继续检测下一组权限
                check(object, consumer, CheckDataEnums.weeks, authStates);
                return;
            }
            plist = permissionGroup.get(checkDataEnums);
            if (ObjectJudge.isNullOrEmpty(plist)) {
                //继续检测下一组权限
                check(object, consumer, CheckDataEnums.weeks, authStates);
                return;
            }
        } else if (checkDataEnums == CheckDataEnums.weeks) {
            if (!permissionGroup.containsKey(checkDataEnums)) {
                //继续检测下一组权限
                check(object, consumer, CheckDataEnums.normal, authStates);
                return;
            }
            plist = permissionGroup.get(checkDataEnums);
            if (ObjectJudge.isNullOrEmpty(plist)) {
                //继续检测下一组权限
                check(object, consumer, CheckDataEnums.normal, authStates);
                return;
            }
        } else if (checkDataEnums == CheckDataEnums.normal) {
            if (!permissionGroup.containsKey(checkDataEnums)) {
                consumer.acceptEach(authStates, true);
                checkResults(authStates, consumer);
                return;
            }
            plist = permissionGroup.get(checkDataEnums);
            if (ObjectJudge.isNullOrEmpty(plist)) {
                consumer.acceptEach(authStates, true);
                checkResults(authStates, consumer);
                return;
            }
        }
        if (checkDataEnums == CheckDataEnums.weeks || (Build.VERSION.SDK_INT >= 23 && !hasAdaperPermissions(plist))) {
            applyRxPermission(object, plist, consumer, checkDataEnums, authStates);
        } else {
            if (Build.VERSION.SDK_INT >= 23) {
                applyRxPermission(object, plist, consumer, checkDataEnums, authStates);
            } else {
                try {
                    HandlerManager.getInstance().postDelayed(new RunnableParamsN<List<Set<String>>>() {
                        @Override
                        public void run(List<Set<String>>... params) {
                            Set<String> lst = new HashSet<>();
                            for (Set<String> sets : params[0]) {
                                lst.addAll(sets);
                            }
                            //<23且用户禁止权限时无法弹出权限申请需要用户手动设置
                            Option option = getAndPermissionOption(object);
                            option.runtime()
                                    .permission(ConvertUtils.toArray(lst))
                                    .onGranted(new AndTargetConsumer(object, consumer, checkDataEnums, CheckType.AndPermission, true, authStates))
                                    .onDenied(new AndErrorConsumer(consumer, true, authStates)).start();
                        }
                    }, 50, plist);
                } catch (Exception e) {
                    applyRxPermission(object, plist, consumer, checkDataEnums, authStates);
                }
            }
        }
    }

    private boolean hasAdaperPermissions(Iterable<String> permissions) {
        boolean flag = false;
        Set<String> adapterPermissions = getAdapterPermissions();
        for (String set : permissions) {
            if (adapterPermissions.contains(set)) {
                flag = true;
                break;
            }
        }
        return flag;
    }

    private boolean hasAdaperPermissions(List<Set<String>> permissions) {
        boolean flag = false;
        Set<String> adapterPermissions = getAdapterPermissions();
        for (Set<String> permission : permissions) {
            for (String set : permission) {
                if (adapterPermissions.contains(set)) {
                    flag = true;
                    break;
                }
            }
        }
        return flag;
    }

    private class AndTargetConsumer implements Action<List<String>> {

        private Object object;
        private OnGrantedConsumer consumer;
        //<23且禁止权限是否需要强提醒
        private boolean isNeedStrongReminder;
        private CheckDataEnums checkDataEnums;
        private CheckType checkType;
        private Map<String, PermissionState> authStates;

        public AndTargetConsumer(Object object, OnGrantedConsumer consumer, CheckDataEnums checkDataEnums, CheckType checkType, boolean isNeedStrongReminder, Map<String, PermissionState> authStates) {
            this.object = object;
            this.consumer = consumer;
            this.checkDataEnums = checkDataEnums;
            this.checkType = checkType;
            this.isNeedStrongReminder = isNeedStrongReminder;
            this.authStates = authStates;
        }

        private void bindPermStates(List<String> data, PermissionGrantedType grantedType) {
            for (String perm : data) {
                if (TextUtils.isEmpty(perm)) {
                    continue;
                }
                if (!authStates.containsKey(perm.trim())) {
                    continue;
                }
                PermissionState permissionState = authStates.get(perm.trim());
                if (permissionState == null) {
                    continue;
                }
                if (TextUtils.equals(perm.trim(), Manifest.permission.ACCESS_FINE_LOCATION)) {
                    //如果是gps权限则调用service
                    if (NetworkUtils.isEnableLocation()) {
                        permissionState.setGrantedType(PermissionGrantedType.granted);
                    } else {
                        permissionState.setGrantedType(PermissionGrantedType.askNeverAgain);
                    }
                } else {
                    permissionState.setGrantedType(grantedType);
                }
            }
        }

        @Override
        public void onAction(List<String> data) {
            if (consumer == null) {
                return;
            }
            final Action1<List<String>> action1 = new Action1<List<String>>() {
                @Override
                public void call(List<String> data) {
                    //把当前已经授予的权限状态改过来
                    bindPermStates(data, PermissionGrantedType.granted);
                    if (checkDataEnums == CheckDataEnums.normal) {
                        consumer.acceptEach(authStates, true);
                        //所有权限检测结束
                        checkResults(authStates, consumer);
                        return;
                    }
                    if (checkDataEnums == CheckDataEnums.adapter) {
                        Map<CheckDataEnums, List<Set<String>>> permissionGroup = consumer.getPermissionGroup();
                        if (!permissionGroup.containsKey(CheckDataEnums.weeks)) {
                            check(object, consumer, CheckDataEnums.normal, authStates);
                            return;
                        }
                        List<Set<String>> plist = permissionGroup.get(CheckDataEnums.weeks);
                        if (ObjectJudge.isNullOrEmpty(plist)) {
                            check(object, consumer, CheckDataEnums.normal, authStates);
                            return;
                        }
                        if (checkType == CheckType.RxPermission) {
                            applyRxPermission(object, plist, consumer, CheckDataEnums.weeks, authStates);
                        } else if (checkType == CheckType.AndPermission) {
                            HandlerManager.getInstance().postDelayed(new RunnableParamsN<List<Set<String>>>() {
                                @Override
                                public void run(List<Set<String>>... params) {
                                    Set<String> lst = new HashSet<>();
                                    for (Set<String> sets : params[0]) {
                                        lst.addAll(sets);
                                    }
                                    Option option = getAndPermissionOption(object);
                                    option.runtime()
                                            .permission(ConvertUtils.toArray(lst))
                                            .onGranted(new AndTargetConsumer(object, consumer, CheckDataEnums.weeks, checkType, isNeedStrongReminder, authStates))
                                            .onDenied(new AndErrorConsumer(consumer, isNeedStrongReminder, authStates)).start();
                                }
                            }, 20, plist);
                        }
                    } else if (checkDataEnums == CheckDataEnums.weeks) {
                        Map<CheckDataEnums, List<Set<String>>> permissionGroup = consumer.getPermissionGroup();
                        if (!permissionGroup.containsKey(CheckDataEnums.normal)) {
                            consumer.acceptEach(authStates, true);
                            checkResults(authStates, consumer);
                            return;
                        }
                        List<Set<String>> plist = permissionGroup.get(CheckDataEnums.normal);
                        if (ObjectJudge.isNullOrEmpty(plist)) {
                            consumer.acceptEach(authStates, true);
                            checkResults(authStates, consumer);
                            return;
                        }
                        if (checkType == CheckType.RxPermission) {
                            applyRxPermission(object, plist, consumer, CheckDataEnums.normal, authStates);
                        } else if (checkType == CheckType.AndPermission) {
                            HandlerManager.getInstance().postDelayed(new RunnableParamsN<List<Set<String>>>() {
                                @Override
                                public void run(List<Set<String>>... params) {
                                    Set<String> lst = new HashSet<>();
                                    for (Set<String> sets : params[0]) {
                                        lst.addAll(sets);
                                    }
                                    Option option = getAndPermissionOption(object);
                                    option.runtime()
                                            .permission(ConvertUtils.toArray(lst))
                                            .onGranted(new AndTargetConsumer(object, consumer, CheckDataEnums.normal, checkType, isNeedStrongReminder, authStates))
                                            .onDenied(new AndErrorConsumer(consumer, isNeedStrongReminder, authStates)).start();
                                }
                            }, 20, plist);
                        }
                    }
                }
            };
            if (Build.VERSION.SDK_INT >= 23 && !hasAdaperPermissions(data)) {
                action1.call(data);
            } else {
                if ((data.contains(Manifest.permission.CAMERA) && !checkCameraEnable(object)) ||
                        (data.contains(Manifest.permission.RECORD_AUDIO) && !checkAudio(object))) {
                    if (Build.VERSION.SDK_INT >= 23) {
                        bindPermStates(data, PermissionGrantedType.askNeverAgain);
                        consumer.acceptEach(authStates, true);
                        checkResults(authStates, consumer);
                    } else {
                        Map<String, String> map = getPermissionMapperMap();
                        for (String checkItem : data) {
                            if (!TextUtils.isEmpty(checkItem)) {
                                if (authStates.containsKey(checkItem.trim())) {
                                    PermissionState permissionState = authStates.get(checkItem.trim());
                                    if (permissionState != null) {
                                        permissionState.setGrantedType(PermissionGrantedType.needStrongReminder);
                                    }
                                }
                            }
                            if (map.containsKey(checkItem)) {
                                consumer.addSRPItems(map.get(checkItem));
                            }
                        }
                        consumer.acceptEach(authStates, true);
                        checkResults(authStates, consumer);
                    }
                } else {
                    action1.call(data);
                }
            }
        }
    }

    private boolean checkAudio(Object object) {
        boolean checkAudioPermission = false;
        if (object == null) {
            return false;
        }
        AudioPermissionCheckUtils checkUtils = new AudioPermissionCheckUtils();
        if (object instanceof FragmentActivity) {
            checkAudioPermission = checkUtils.checkAudioPermission((FragmentActivity) object);
        } else if (object instanceof Fragment) {
            Fragment fragment = (Fragment) object;
            checkAudioPermission = checkUtils.checkAudioPermission(fragment.getContext());
        }
        return checkAudioPermission;
    }

    private class AndErrorConsumer implements Action<List<String>> {

        private OnGrantedConsumer consumer;
        //<23且禁止权限是否需要强提醒
        private boolean isNeedStrongReminder;
        private Map<String, PermissionState> authStates;

        public AndErrorConsumer(OnGrantedConsumer consumer, boolean isNeedStrongReminder, Map<String, PermissionState> authStates) {
            this.consumer = consumer;
            this.isNeedStrongReminder = isNeedStrongReminder;
            this.authStates = authStates;
        }

        @Override
        public void onAction(List<String> data) {
            if (isNeedStrongReminder && Build.VERSION.SDK_INT < 23) {
                if (data != null) {
                    Map<String, String> map = getPermissionMapperMap();
                    for (String checkItem : data) {
                        if (!TextUtils.isEmpty(checkItem)) {
                            if (authStates.containsKey(checkItem.trim())) {
                                PermissionState permissionState = authStates.get(checkItem.trim());
                                if (permissionState != null) {
                                    permissionState.setGrantedType(PermissionGrantedType.needStrongReminder);
                                }
                            }
                        }
                        if (map.containsKey(checkItem)) {
                            consumer.addSRPItems(map.get(checkItem));
                        }
                    }
                }
                consumer.acceptEach(authStates, false);
                consumer.accept(PermissionGrantedType.needStrongReminder);
                consumer.accept(PermissionGrantedType.needStrongReminder, consumer.getExtras());
            } else {
                for (String perm : data) {
                    if (TextUtils.isEmpty(perm)) {
                        continue;
                    }
                    if (!authStates.containsKey(perm.trim())) {
                        continue;
                    }
                    PermissionState permissionState = authStates.get(perm.trim());
                    if (permissionState == null) {
                        continue;
                    }
                    permissionState.setGrantedType(PermissionGrantedType.askNeverAgain);
                }
                consumer.acceptEach(authStates, false);
                consumer.accept(PermissionGrantedType.askNeverAgain);
                consumer.accept(PermissionGrantedType.askNeverAgain, consumer.getExtras());
            }
        }
    }

    private class TargetErrorConsumer implements Consumer<Throwable> {

        private OnGrantedConsumer consumer;
        private Map<String, PermissionState> authStates;

        public TargetErrorConsumer(OnGrantedConsumer consumer, Map<String, PermissionState> authStates) {
            this.consumer = consumer;
            this.authStates = authStates;
        }

        @Override
        public void accept(Throwable throwable) {
            if (consumer == null) {
                return;
            }
            //清除队列中的权限
            permissions.clear();
            consumer.acceptEach(authStates, false);
            consumer.accept(PermissionGrantedType.askNeverAgain);
            consumer.accept(PermissionGrantedType.askNeverAgain, consumer.getExtras());
        }
    }

    /**
     * 申请权限
     *
     * @param fragment    fragment
     * @param consumer    授予权限监听
     * @param permissions 权限{@link Manifest.permission}
     */
    public void apply(Fragment fragment, OnGrantedConsumer consumer, HashMap<String, Set<String>> permissions) {
        if (fragment == null || ObjectJudge.isNullOrEmpty(permissions)) {
            //要检测的权限为空或页面为空则不检测
            return;
        }
        Map<String, PermissionGrantedType> detectionMap = applyBeforeDetection(fragment, permissions);
        Map<String, PermissionState> authStates = toAuthStates(detectionMap);
        //先回调检测结果
        if (consumer != null) {
            consumer.detection(detectionMap);
        }
        preparePermissions(consumer, permissions);
        check(fragment, consumer, CheckDataEnums.adapter, authStates);
    }

    private class TargetConsumer implements Consumer<Permission> {

        private OnGrantedConsumer consumer;
        private String guid;
        private CheckDataEnums checkDataEnums;
        //大多数情况下object是activity，这里的TargetConsumer使用之后产生的
        //disposable没有及时的dispose掉，经常发生内存泄漏
        //所以这里改成弱应用，保证activity不会泄漏，Disposable的泄漏问题再看吧
        //2020-05-21修复disposable内存泄漏的问题
        private WeakReference<Object> objectRef;
        private Map<String, PermissionState> authStates;

        public TargetConsumer(Object object, String guid, OnGrantedConsumer consumer, CheckDataEnums checkDataEnums, Map<String, PermissionState> authStates) {
            this.guid = guid;
            this.consumer = consumer;
            this.checkDataEnums = checkDataEnums;
            this.objectRef = new WeakReference<>(object);
            this.authStates = authStates;
        }

        private void bindGrantedStates(Permission permission) {
            String name = permission.name;
            if (name == null) {
                return;
            }
            String[] plist = name.split(",");
            for (String permis : plist) {
                if (TextUtils.isEmpty(permis)) {
                    continue;
                }
                if (!authStates.containsKey(permis.trim())) {
                    continue;
                }
                PermissionState permissionState = authStates.get(permis.trim());
                if (permissionState == null) {
                    continue;
                }
                if (TextUtils.equals(permis.trim(), Manifest.permission.ACCESS_FINE_LOCATION)) {
                    //如果是gps权限则调用service
                    if (NetworkUtils.isEnableLocation()) {
                        permissionState.setGrantedType(PermissionGrantedType.granted);
                    } else {
                        permissionState.setGrantedType(PermissionGrantedType.askNeverAgain);
                    }
                } else {
                    if (permission.granted) {
                        permissionState.setGrantedType(PermissionGrantedType.granted);
                    } else if (permission.shouldShowRequestPermissionRationale) {
                        permissionState.setGrantedType(PermissionGrantedType.shouldShowRequestPermissionRationale);
                    } else {
                        permissionState.setGrantedType(PermissionGrantedType.askNeverAgain);
                    }
                }
            }
        }

        @Override
        public void accept(Permission permission) {
            if (consumer == null) {
                return;
            }
            Object object = objectRef.get();
            if (object == null) {
                return;
            }
            bindGrantedStates(permission);
            disposable(guid);
            //回调下一个权限
            if (permissions.size() > 0) {
                Set<String> remove = permissions.remove(0);
                requestPermission(object, remove, consumer, checkDataEnums, authStates);
                return;
            }
            //执行下一组回调
            if (checkDataEnums == CheckDataEnums.adapter) {
                check(object, consumer, CheckDataEnums.weeks, authStates);
            } else if (checkDataEnums == CheckDataEnums.weeks) {
                check(object, consumer, CheckDataEnums.normal, authStates);
            } else {
                consumer.acceptEach(authStates, true);
                checkResults(authStates, consumer);
            }
        }
    }

    private void checkResults(Map<String, PermissionState> authStates, OnGrantedConsumer consumer) {
        int grantedCount = 0, rationalCount = 0, neverCount = 0;
        for (Map.Entry<String, PermissionState> entry : authStates.entrySet()) {
            PermissionState permissionState = entry.getValue();
            PermissionGrantedType permissionGrantedType = permissionState.getGrantedType();
            if (permissionGrantedType == PermissionGrantedType.askNeverAgain) {
                neverCount++;
            } else if (permissionGrantedType == PermissionGrantedType.shouldShowRequestPermissionRationale) {
                rationalCount++;
            } else if (permissionGrantedType == PermissionGrantedType.granted) {
                grantedCount++;
            }
        }
        if (grantedCount > 0 && grantedCount == authStates.size()) {
            consumer.accept(PermissionGrantedType.granted);
            consumer.accept(PermissionGrantedType.granted, consumer.getExtras());
        } else if (neverCount > 0) {
            consumer.accept(PermissionGrantedType.askNeverAgain);
            consumer.accept(PermissionGrantedType.askNeverAgain, consumer.getExtras());
        } else if (rationalCount > 0) {
            consumer.accept(PermissionGrantedType.shouldShowRequestPermissionRationale);
            consumer.accept(PermissionGrantedType.shouldShowRequestPermissionRationale, consumer.getExtras());
        }
    }

    private void disposable(String guid) {
        try {
            if (TextUtils.isEmpty(guid)) {
                return;
            }
            DisposableItem item = mCompositeDisposable.remove(guid);
            if (item == null) {
                return;
            }
            Disposable remove = item.disposable;
            if (remove == null) {
                return;
            }
            if (!remove.isDisposed()) {
                remove.dispose();
            }
        } catch (Exception e) {
            Logger.error(e);
        }
    }

    /**
     * 目前的代码实现上，Disposable没有被加入到map之前就会触发consumer的accept方法
     * 然后直接开始移除与之对应的Disposable，既然没有加进去，那么移除肯定也无效
     * 等到移除代码执行结束（即consumer的accept方法执行结束了）才会把Disposable加进去
     * 然后它没有机会被移除了，又因为该类是单例设计，所以Disposable会一直存在在内存当中
     * 而这里的disposable持有了consumer，consumer里的object又是activity，所以
     * 这个场景下的activity存在内存泄漏，因此封装这个方法在activity的onDestroy方法中
     * 调用避免内存泄漏
     * <p>
     * <p>
     * 当前的activity被销毁掉之后，依赖当前activity的所有disposable都应该被dispose
     */
    public void disposeAllForActivityDestroy(Activity activity) {
        List<DisposableItem> items = new ArrayList<>();
        for (Map.Entry<String, DisposableItem> entry : mCompositeDisposable.entrySet()) {
            DisposableItem item = entry.getValue();
            if (item.object == activity) {
                if (item.disposable != null && !item.disposable.isDisposed()) {
                    item.disposable.dispose();
                }
                items.add(item);
            }
        }
        for (DisposableItem item : items) {
            mCompositeDisposable.remove(item.key);
        }
    }

    /**
     * 权限是否被注销
     *
     * @param fragmentActivity fragmentActivity
     * @param permission       检测权限{@link Manifest.permission}
     * @return true-被注销;false-未被注销;
     */
    public boolean isRevoked(FragmentActivity fragmentActivity, String permission) {
        if (fragmentActivity == null || TextUtils.isEmpty(permission)) {
            return false;
        }
        return isRxRevoked(fragmentActivity, permission);
    }

    /**
     * 权限是否被注销
     *
     * @param fragment   fragment
     * @param permission 检测权限{@link Manifest.permission}
     * @return true-被注销;false-未被注销;
     */
    public boolean isRevoked(Fragment fragment, String permission) {
        if (fragment == null || TextUtils.isEmpty(permission)) {
            return false;
        }
        return isRxRevoked(fragment, permission);
    }

    /**
     * 权限是否开启
     *
     * @param fragmentActivity fragmentActivity
     * @param permission       检测权限{@link Manifest.permission}
     * @return true-已开启;false-未开启;
     */
    public boolean isGranted(FragmentActivity fragmentActivity, String permission) {
        if (fragmentActivity == null || TextUtils.isEmpty(permission)) {
            return false;
        }
        return isRxGranted(fragmentActivity, permission);
    }

    /**
     * 权限是否开启
     *
     * @param fragment   fragment
     * @param permission 检测权限{@link Manifest.permission}
     * @return true-已开启;false-未开启;
     */
    public boolean isGranted(Fragment fragment, String permission) {
        if (fragment == null || TextUtils.isEmpty(permission)) {
            return false;
        }
        return isRxGranted(fragment, permission);
    }

    /**
     * 申请浮动窗口权限
     * (在申请的过程若权限未开启将引导用户打开权限)
     *
     * @param context  上下文
     * @param consumer 检测成功和引导提示回调
     */
    public void applyFloatWindowPermission(Context context, OnPermissionCheckPromptConsumer consumer) {
        FloatWindowPermissionDetection floatWindowPermissionDetection = new FloatWindowPermissionDetection(consumer);
        floatWindowPermissionDetection.checkFloatWindowPermission(context);
    }

    private static class AudioPermissionCheckUtils {

        // 音频获取源
        public int audioSource = MediaRecorder.AudioSource.MIC;
        // 设置音频采样率，44100是目前的标准，但是某些设备仍然支持22050，16000，11025
        public int sampleRateInHz = 44100;
        // 设置音频的录制的声道CHANNEL_IN_STEREO为双声道，CHANNEL_CONFIGURATION_MONO为单声道
        public int channelConfig = AudioFormat.CHANNEL_IN_STEREO;
        // 音频数据格式:PCM 16位每个样本。保证设备支持。PCM 8位每个样本。不一定能得到设备支持。
        public int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
        // 缓冲区字节大小
        public int bufferSizeInBytes = 0;

        /**
         * 判断是是否有录音权限
         */
        public boolean checkAudioPermission(final Context context) {
            bufferSizeInBytes = 0;
            bufferSizeInBytes = AudioRecord.getMinBufferSize(sampleRateInHz,
                    channelConfig, audioFormat);
            AudioRecord audioRecord = new AudioRecord(audioSource, sampleRateInHz,
                    channelConfig, audioFormat, bufferSizeInBytes);
            //开始录制音频
            try {
                // 防止某些手机崩溃，例如联想
                audioRecord.startRecording();
            } catch (IllegalStateException e) {
                Logger.error(e);
                return false;
            }
            //根据开始录音判断是否有录音权限
            if (audioRecord.getRecordingState() != AudioRecord.RECORDSTATE_RECORDING) {
                if (audioRecord.getRecordingState() != AudioRecord.RECORDSTATE_STOPPED) {
                    return false;
                }
            }
            audioRecord.stop();
            audioRecord.release();
            audioRecord = null;
            return true;
        }
    }

    @SuppressLint("UnsupportedChromeOsCameraSystemFeature")
    private boolean checkCameraEnable(Object object) {
        if (object == null) {
            return false;
        }
        PackageManager packageManager = null;
        if (object instanceof FragmentActivity) {
            FragmentActivity fragmentActivity = (FragmentActivity) object;
            packageManager = fragmentActivity.getPackageManager();
        } else if (object instanceof Fragment) {
            Fragment fragment = (Fragment) object;
            Context context = fragment.getContext();
            packageManager = context.getPackageManager();
        }
        if (packageManager == null) {
            return false;
        }
        boolean hasSystemFeature;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            hasSystemFeature = packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA_ANY);
        } else {
            hasSystemFeature = packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA);
        }
        if (!hasSystemFeature) {
            //不支持相机
            return false;
        }
        boolean canUse = false;
        Camera mCamera = null;
        try {
            mCamera = Camera.open();
            Camera.Parameters mParameters = mCamera.getParameters();
            mCamera.setParameters(mParameters);
            canUse = true;
        } catch (Exception e) {
            canUse = false;
        }
        if (mCamera != null) {
            mCamera.release();
        }
        return canUse;
    }
}
