package com.basic.eyflutter_uikit.permission.detections;

import android.Manifest;
import android.content.Context;
import android.os.Build;
import android.provider.Settings;

import androidx.fragment.app.FragmentActivity;

import com.basic.eyflutter_uikit.enums.PermissionGrantedType;
import com.basic.eyflutter_uikit.permission.OnGrantedConsumer;
import com.basic.eyflutter_uikit.permission.PermissionManager;
import com.cloud.eyutils.logs.Logger;
import com.cloud.eyutils.utils.CollectionUtils;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Set;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-06-18
 * Description:悬浮窗权限检测
 * Modifier:
 * ModifyContent:
 */
public class FloatWindowPermissionDetection {

    private OnPermissionCheckPromptConsumer consumer;
    private Context context;

    public FloatWindowPermissionDetection(OnPermissionCheckPromptConsumer consumer) {
        this.consumer = consumer;
    }

    /**
     * 获取context
     *
     * @return context
     */
    public Context getContext() {
        return this.context;
    }

    /**
     * 检测悬浮窗权限
     *
     * @param context 上下文
     */
    public void checkFloatWindowPermission(Context context) {
        this.context = context;
        try {
            //6.0 版本之后由于 google 增加了对悬浮窗权限的管理，所以方式就统一了
            if (Build.VERSION.SDK_INT < 23) {
                if (RomUtils.checkIsMiuiRom() ||
                        RomUtils.checkIsMeizuRom() ||
                        RomUtils.checkIsHuaweiRom() ||
                        RomUtils.checkIs360Rom() ||
                        RomUtils.checkIsOppoRom()) {
                    //api小于23部分系统对于悬浮窗仍需要权限申请
                    if (VendorUtils.checkFloatWindowPermission(context)) {
                        if (consumer != null) {
                            consumer.onCheckSuccess();
                        }
                        return;
                    }
                }
            }
            if (commonROMPermissionCheck(context)) {
                if (consumer != null) {
                    consumer.onCheckSuccess();
                }
            } else {
                //如果granted==true表示权限已启用,否则由PermissionManager再一次检测
                //如果context不为FragmentActivity或Fragment则回调false
                if (context instanceof FragmentActivity) {
                    FragmentActivity fragmentActivity = (FragmentActivity) context;
                    HashMap<String, Set<String>> map = new HashMap<>();
                    CollectionUtils.put(map, Manifest.permission.SYSTEM_ALERT_WINDOW, Manifest.permission.SYSTEM_ALERT_WINDOW);
                    PermissionManager.getInstance().apply(fragmentActivity, floatWindowPermissionConsumer, map);
                } else {
                    if (consumer != null) {
                        consumer.onGuidePrompt(this);
                    }
                }
            }
        } catch (Exception e) {
            Logger.error(e);
        }
    }

    private OnGrantedConsumer floatWindowPermissionConsumer = new OnGrantedConsumer() {
        @Override
        public void accept(PermissionGrantedType grantedType) {
            if (consumer != null) {
                if (grantedType == PermissionGrantedType.granted) {
                    consumer.onCheckSuccess();
                } else {
                    consumer.onGuidePrompt(FloatWindowPermissionDetection.this);
                }
            }
        }
    };

    private boolean commonROMPermissionCheck(Context context) {
        if (RomUtils.checkIsMeizuRom()) {
            return VendorUtils.checkFloatWindowPermission(context);
        } else {
            Boolean result = true;
            if (Build.VERSION.SDK_INT >= 23) {
                try {
                    Class clazz = Settings.class;
                    Method canDrawOverlays = clazz.getDeclaredMethod("canDrawOverlays", Context.class);
                    result = (Boolean) canDrawOverlays.invoke(null, context);
                } catch (Exception e) {
                    result = false;
                }
            }
            return result;
        }
    }

    /**
     * 引导启用浮动窗口权限
     *
     * @param context 上下文
     */
    public void guideEnableFloatWindowPermission(Context context) {
        if (context == null) {
            return;
        }
        if (Build.VERSION.SDK_INT < 23) {
            if (RomUtils.checkIsMiuiRom()) {
                TurnUtils.applyMiuiPermission(context);
            } else if (RomUtils.checkIsMeizuRom()) {
                TurnUtils.applyMeizuPermission(context);
            } else if (RomUtils.checkIsHuaweiRom()) {
                TurnUtils.applyHuaweiPermission(context);
            } else if (RomUtils.checkIs360Rom()) {
                TurnUtils.apply360Permission(context);
            } else if (RomUtils.checkIsOppoRom()) {
                TurnUtils.applyOppoPermission(context);
            }
        } else {
            TurnUtils.commonROMPermissionApply(context);
        }
    }
}
