package com.basic.eyflutter_uikit.pickers;

import android.app.Activity;
import android.content.Intent;


import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.beans.PickConfigResponse;
import com.basic.eyflutter_uikit.pickers.beans.PickerArgs;
import com.basic.eyflutter_uikit.pickers.beans.PickersArguments;
import com.basic.eyflutter_uikit.pickers.beans.PickersLangs;
import com.basic.eyflutter_uikit.pickers.beans.PickersPermission;
import com.basic.eyflutter_uikit.pickers.enums.SelectMode;
import com.basic.eyflutter_uikit.pickers.pick.PhotoPickConfig;
import com.basic.eyflutter_uikit.pickers.pick.PickLauncher;
import com.basic.eyflutter_uikit.pickers.ui.CameraActivity;
import com.basic.eyflutter_uikit.pickers.utils.PickUtils;
import com.cloud.eyutils.utils.JsonUtils;

import io.flutter.plugin.common.MethodChannel;

public class PickersManager {

    public void showPicker(Activity activity, PickerArgs pickerArgs, MethodChannel.Result result) {
        try {
            PhotoPickConfig.Builder builder = getPickConfig(activity, pickerArgs.getPickerArguments(), pickerArgs.getMode());
            PickConfigResponse response = builder.build();
            PickBean pickBean = response.getPickBean();
            PickersLangs langArguments = JsonUtils.parseT(pickerArgs.getLanguage(), PickersLangs.class);
            if (pickBean == null || langArguments == null) {
                finishPick(result);
            } else {
                String permissionJson = pickerArgs.getPermission();
                pickBean.setPermission(JsonUtils.parseT(permissionJson, PickersPermission.class));
                PickUtils.getInstance().setPickBean(pickBean);
                PickUtils.getInstance().setLangs(langArguments);
                //根据选择模式决定启动行为
                SelectMode mode = pickBean.getMode();
                if (mode == SelectMode.all || mode == SelectMode.image || mode == SelectMode.video) {
                    PickLauncher.startPick(response.getActivity());
                } else if (mode == SelectMode.talkingAll) {
                    Intent intent = new Intent();
                    intent.setClass(activity, CameraActivity.class);
                    activity.startActivityForResult(intent, 100);
                } else if (mode == SelectMode.talkingImage) {
                    PickUtils.getInstance().startSystemTaking(activity, false);
                } else if (mode == SelectMode.talkingVideo) {
                    PickUtils.getInstance().startSystemTaking(activity, true);
                } else {
                    finishPick(result);
                }
            }
        } catch (Exception e) {
            finishPick(result);
        }
    }

    private void finishPick(MethodChannel.Result result) {
        PickUtils.getInstance().clean();
        if (result != null) {
            result.success("");
            result = null;
        }
    }

    private PhotoPickConfig.Builder getPickConfig(Activity activity, PickersArguments arguments, String mode) {
        PhotoPickConfig.Builder builder = new PhotoPickConfig.Builder(activity);
        builder.setMaxCount(arguments.getMaxCount());
        //maxCount>1为多选
        if (arguments.getMaxCount() > 1) {
            builder.setPickMode(PhotoPickConfig.MODE_PICK_MORE);
            builder.setSpanCount(arguments.getSpanCount());
            //最大允许选择数量
            builder.setMaxPickSize(arguments.getMaxCount());
        } else {
            builder.setPickMode(PhotoPickConfig.MODE_PICK_SINGLE);
            builder.setSpanCount(PhotoPickConfig.GRID_SPAN_COUNT);
            builder.setMaxPickSize(PhotoPickConfig.DEFAULT_CHOOSE_SIZE);
        }
        //选择模式
        builder.setMode(SelectMode.getMode(mode));
        //是否开启裁剪照片功能，默认关闭
        if (arguments.isClipPhoto()) {
            builder.setClipPhoto(arguments.isClipPhoto());
        }
        //是否裁剪方式为圆形，默认为矩形
        if (arguments.isClipCircle()) {
            builder.setClipCircle(arguments.isClipCircle());
        }
        //是否显示原图按钮，默认显示
        if (arguments.isShowOriginal()) {
            builder.setShowOriginal(arguments.isShowOriginal());
        }
        //是否开启压缩，默认true
        if (arguments.isStartCompression()) {
            builder.setStartCompression(arguments.isStartCompression());
        }
        //是否可以多选视频,默认为false;该属性仅在maxCount>1时有效;
        if (arguments.isCanMultipleVideo()) {
            builder.setCanMultipleVideo(arguments.isCanMultipleVideo());
        }
        //视频最大时长
        builder.setVideoMaxDuration(arguments.getVideoMaxDuration() * 1000);
        return builder;
    }
}
