package com.basic.eyflutter_uikit.pickers.adapters;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RelativeLayout;

import androidx.databinding.DataBindingUtil;
import androidx.recyclerview.widget.RecyclerView;

import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.beans.ActionItem;
import com.basic.eyflutter_uikit.databinding.ItemCameraLayoutBinding;
import com.basic.eyflutter_uikit.databinding.ItemPickGalleryBinding;
import com.basic.eyflutter_uikit.dialogs.MessageBox;
import com.basic.eyflutter_uikit.enums.MsgBoxClickButtonEnum;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.beans.ChooseLimit;
import com.basic.eyflutter_uikit.pickers.beans.MediaData;
import com.basic.eyflutter_uikit.pickers.beans.PermissionItem;
import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.beans.PickersLangs;
import com.basic.eyflutter_uikit.pickers.beans.PickersPermission;
import com.basic.eyflutter_uikit.pickers.pick.loader.GlideImageLoader;
import com.basic.eyflutter_uikit.pickers.pick.loader.ImageLoader;
import com.basic.eyflutter_uikit.pickers.utils.PickUtils;
import com.cloud.eyutils.ebus.EBus;
import com.cloud.eyutils.manager.ObjectManager;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.PixelUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

public class PickAdapter extends RecyclerView.Adapter {

    private Context context;
    private ImageLoader imageLoader = new GlideImageLoader();
    private ArrayList<MediaData> mediaDatas;
    private LinkedList<MediaData> selectItems = new LinkedList<>();
    private HashMap<Integer, Integer> posMap = new HashMap<>();
    private int imageSize = 0;
    private PickBean pickBean;
    private PickersLangs langs;
    private OnPickSelectItemsListener selectItemsListener;

    public PickAdapter(Context context, ArrayList<MediaData> mediaDatas, PickBean pickBean, PickersLangs langs) {
        this.context = context;
        this.mediaDatas = mediaDatas;
        this.pickBean = pickBean;
        this.langs = langs;
        int screenWidth = ObjectManager.getScreenWidth(context);
        int spec = PickUtils.getInstance().getSpec();
        imageSize = (screenWidth - (pickBean.getSpanCount() - 1) * PixelUtils.dip2px(context, spec) - PixelUtils.dip2px(context, 6) * 2 - PixelUtils.dip2px(context, 0.8f) * pickBean.getSpanCount() * 2) / pickBean.getSpanCount();
    }

    public interface OnPickSelectItemsListener {
        void onPickSelectItems(LinkedList<MediaData> selectItems);
    }

    public void setSelectItemsListener(OnPickSelectItemsListener listener) {
        this.selectItemsListener = listener;
    }

    @Override
    public int getItemViewType(int position) {
        MediaData mediaData = mediaDatas.get(position);
        if (mediaData.isCamera()) {
            return 1;
        } else {
            return 2;
        }
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        if (viewType == 1) {
            View view = LayoutInflater.from(context).inflate(R.layout.item_camera_layout, null);
            return new CameraViewHolder(view);
        } else {
            View view = LayoutInflater.from(context).inflate(R.layout.item_pick_gallery, null);
            return new PickGalleryViewHolder(view);
        }
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder viewHolder, int position) {
        MediaData mediaData = mediaDatas.get(position);
        mediaData.setPosition(position);
        if (!mediaData.isCamera()) {
            PickGalleryViewHolder holder = (PickGalleryViewHolder) viewHolder;
            RecyclerView.LayoutParams vparams = new RecyclerView.LayoutParams(imageSize, imageSize);
            holder.binding.itemRl.setLayoutParams(vparams);
            RelativeLayout.LayoutParams markParams = new RelativeLayout.LayoutParams(imageSize, imageSize);
            holder.binding.markFl.setLayoutParams(markParams);
            RelativeLayout.LayoutParams ivparams = new RelativeLayout.LayoutParams(imageSize, imageSize);
            ivparams.addRule(RelativeLayout.CENTER_IN_PARENT);
            holder.binding.galleryIv.setLayoutParams(ivparams);
            imageLoader.displayImage(context, mediaData.getOriginalPath(), holder.binding.galleryIv, false, 4, imageSize);
            holder.binding.checkLl.setTag(mediaData);
            //选择状态
            if (mediaData.isCheck()) {
                holder.binding.checkNumberTv.setBackgroundResource(R.drawable.cl_item_select_state);
                holder.binding.checkNumberTv.setText(String.valueOf(mediaData.getCount()));
            } else {
                holder.binding.checkNumberTv.setBackgroundResource(R.drawable.cl_item_unselect_state);
                holder.binding.checkNumberTv.setText("");
            }
            //媒体类型
            String imageType = mediaData.getImageType();
            if (TextUtils.equals(imageType, PickersConstants.mp4Type)) {
                holder.binding.playIv.setVisibility(View.VISIBLE);
            } else {
                holder.binding.playIv.setVisibility(View.GONE);
            }
        } else {
            CameraViewHolder holder = (CameraViewHolder) viewHolder;
            RecyclerView.LayoutParams vparams = new RecyclerView.LayoutParams(imageSize, imageSize);
            holder.binding.itemCameraRl.setLayoutParams(vparams);
        }
    }

    @Override
    public int getItemCount() {
        return ObjectJudge.isNullOrEmpty(mediaDatas) ? 0 : mediaDatas.size();
    }

    private class CameraViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ItemCameraLayoutBinding binding;

        public CameraViewHolder(View itemView) {
            super(itemView);
            itemView.setOnClickListener(this::onClick);
            binding = DataBindingUtil.bind(itemView);
            binding.cameraTv.setText(langs.getCamera());
        }

        @Override
        public void onClick(View v) {
            PickersPermission permission = pickBean.getPermission();
            PermissionItem camera = permission.getCamera();
            EBus.getInstance().post(PickersConstants.startCameraBusKey, camera.getPermissions(), camera.getPrompt());
        }
    }

    private class PickGalleryViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ItemPickGalleryBinding binding;

        public PickGalleryViewHolder(View itemView) {
            super(itemView);
            binding = DataBindingUtil.bind(itemView);
            binding.checkLl.setOnClickListener(this::onClick);
        }

        @Override
        public void onClick(View v) {
            int id = v.getId();
            if (id == R.id.check_ll) {
                checkHandler(v);
            }
        }

        private void checkHandler(View view) {
            Object tag = view.getTag();
            if (!(tag instanceof MediaData)) {
                return;
            }
            int maxCount = pickBean.getMaxCount();
            MediaData mediaData = (MediaData) tag;
            if (!mediaData.isCheck() && maxCount > 1) {
                int count = selectItems.size() + 1;
                if (count > maxCount) {
                    notifyMaxSelectCount();
                    return;
                }
            }
            //如果是单选则清除其它项状态
            if (maxCount == 1) {
                resetOtherItems(mediaData);
            } else {
                //单选视频时清空其它视频状态
                if (!pickBean.isCanMultipleVideo()) {
                    singleVideoState(mediaData);
                }
            }
            mediaData.setCheck(!mediaData.isCheck());
            if (mediaData.isCheck()) {
                selectItems.add(mediaData);
                posMap.put(mediaData.getMediaId(), mediaData.getPosition());
                resetCount();
                if (selectItemsListener != null) {
                    selectItemsListener.onPickSelectItems(selectItems);
                }
            } else {
                removeSelectedItem(mediaData);
            }
            //单选时需要清除未选中状态
            if (maxCount == 1) {
                cleanUnSelectState();
            }
            if (selectItemsListener != null) {
                selectItemsListener.onPickSelectItems(selectItems);
            }
        }

        private void singleVideoState(MediaData mediaData) {
            if (!TextUtils.equals(mediaData.getImageType(), PickersConstants.mp4Type)) {
                return;
            }
            for (int i = 0; i < selectItems.size(); i++) {
                MediaData data = selectItems.get(i);
                if (!mediaData.isCheck() && data.getMediaId() == mediaData.getMediaId()) {
                    //将要被选中时不处理
                    continue;
                }
                if (!TextUtils.equals(data.getImageType(), PickersConstants.mp4Type)) {
                    continue;
                }
                Integer position = posMap.get(data.getMediaId());
                if (position != null) {
                    MediaData resultData = mediaDatas.get(position.intValue());
                    resultData.setCheck(false);
                    notifyItemChanged(position.intValue());
                }
                posMap.remove(data.getMediaId());
                selectItems.remove(i);
                i--;
            }
        }

        private void cleanUnSelectState() {
            for (int i = 0; i < selectItems.size(); i++) {
                MediaData mediaData = selectItems.get(i);
                if (!mediaData.isCheck()) {
                    selectItems.remove(i);
                    Integer position = posMap.get(mediaData.getMediaId());
                    if (position != null) {
                        MediaData data = mediaDatas.get(position.intValue());
                        data.setCheck(false);
                        notifyItemChanged(position.intValue());
                    }
                    posMap.remove(mediaData.getMediaId());
                    i--;
                }
            }
        }

        private void resetOtherItems(MediaData mediaData) {
            for (int i = 0; i < selectItems.size(); i++) {
                MediaData data = selectItems.get(i);
                if (data.getMediaId() != mediaData.getMediaId()) {
                    data.setCheck(false);
                    selectItems.remove(i);
                    Integer position = posMap.get(data.getMediaId());
                    if (position != null) {
                        MediaData origin = mediaDatas.get(position.intValue());
                        origin.setCheck(false);
                        notifyItemChanged(position.intValue());
                    }
                    posMap.remove(data.getMediaId());
                    i--;
                } else {
                    data.setCount(1);
                }
            }
        }

        //通知最大选择数
        private void notifyMaxSelectCount() {
            ChooseLimit chooseLimit = langs.getChooseLimit();
            String message = chooseLimit.getMessage();
            int maxCount = pickBean.getMaxCount();
            message = String.format(message, maxCount);
            messageBox.setContent(message);
            messageBox.setShowClose(false);
            ActionItem[] actions = {
                    new ActionItem("confirm", chooseLimit.getConfirm()).setTextColor(0xffFF6700).setTextSize(16)
            };
            messageBox.setButtons(actions);
            messageBox.show(context);
        }

        private MessageBox messageBox = new MessageBox() {
            @Override
            public boolean onItemClickListener(View v, MsgBoxClickButtonEnum mcbenum, String target, Object extraData) {
                return super.onItemClickListener(v, mcbenum, target, extraData);
            }
        };

        private void resetCount() {
            for (int i = 0; i < selectItems.size(); i++) {
                MediaData mediaData = selectItems.get(i);
                mediaData.setCount(i + 1);
                notifyItemChanged(mediaData.getPosition());
            }
        }

        private void removeSelectedItem(MediaData mediaData) {
            for (int i = 0; i < selectItems.size(); i++) {
                MediaData selectItem = selectItems.get(i);
                selectItem.setCount(i + 1);
                if (selectItem.getMediaId() == mediaData.getMediaId()) {
                    selectItems.remove(i);
                    posMap.remove(selectItem.getMediaId());
                    i--;
                }
                notifyItemChanged(selectItem.getPosition());
            }
        }
    }

    public LinkedList<MediaData> getSelectItems() {
        return selectItems;
    }
}
