package com.basic.eyflutter_uikit.pickers.adapters;

import android.content.Context;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;

import androidx.databinding.DataBindingUtil;
import androidx.recyclerview.widget.RecyclerView;

import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.databinding.ItemPreviewLayoutBinding;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.beans.ImageInfo;
import com.basic.eyflutter_uikit.pickers.pick.loader.GlideImageLoader;
import com.basic.eyflutter_uikit.pickers.pick.loader.ImageLoader;
import com.basic.eyflutter_uikit.pickers.viewer.ImagePreview;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.PixelUtils;

import java.util.LinkedList;
import java.util.List;

public class SelectedImageAdapter extends RecyclerView.Adapter<SelectedImageAdapter.SelectedImageViewHolder> {

    private Context context;
    private List<ImageInfo> imageInfos = new LinkedList<>();
    private ImageLoader imageLoader = new GlideImageLoader();
    private OnSelectImageItemListener selectImageItemListener;
    private int currentItem = 0;
    private int imageSize;

    public interface OnSelectImageItemListener {
        void onSelectImageItem(ImageInfo imageInfo, int position);
    }

    protected void onCurrentPositionCall(int position) {

    }

    public SelectedImageAdapter(Context context, List<ImageInfo> imageInfos, OnSelectImageItemListener listener) {
        if (ObjectJudge.isNullOrEmpty(imageInfos)) {
            return;
        }
        this.context = context;
        this.imageInfos.addAll(imageInfos);
        ImageInfo imageInfo = this.imageInfos.get(0);
        imageInfo.setCheck(true);
        this.selectImageItemListener = listener;
        this.currentItem = ImagePreview.getInstance().getIndex();
        this.imageSize = PixelUtils.dip2px(context, 70);
    }

    @Override
    public SelectedImageViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = View.inflate(context, R.layout.item_preview_layout, null);
        return new SelectedImageViewHolder(view);
    }

    @Override
    public void onBindViewHolder(SelectedImageViewHolder holder, int position) {
        ImageInfo imageInfo = imageInfos.get(position);
        ItemPreviewLayoutBinding binding = holder.binding;
        imageLoader.displayImage(context, imageInfo.getThumbnailUrl(), binding.previewImageRiv, false, 4, imageSize);
        binding.itemRl.setTag(position);
        if (imageInfo.isCheck()) {
            binding.checkIv.setVisibility(View.VISIBLE);
        } else {
            binding.checkIv.setVisibility(View.GONE);
        }
        //媒体类型
        String imageType = imageInfo.getImageType();
        if (TextUtils.equals(imageType, PickersConstants.mp4Type)) {
            binding.playIv.setVisibility(View.VISIBLE);
        } else {
            binding.playIv.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        return imageInfos.size();
    }

    public class SelectedImageViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ItemPreviewLayoutBinding binding;

        public SelectedImageViewHolder(View itemView) {
            super(itemView);
            binding = DataBindingUtil.bind(itemView);
            binding.itemRl.setOnClickListener(this::onClick);
        }

        @Override
        public void onClick(View v) {
            int position = ConvertUtils.toInt(v.getTag());
            ImageInfo imageInfo = imageInfos.get(position);
            if (selectImageItemListener != null) {
                selectImageItemListener.onSelectImageItem(imageInfo, position);
            }
            currentItem = position;
            setCurrentItemState(position);
        }
    }

    public void setCurrentItemState(int position) {
        int count = imageInfos.size();
        for (int i = 0; i < count; i++) {
            ImageInfo info = imageInfos.get(i);
            if (position == i) {
                info.setCheck(true);
            } else {
                info.setCheck(false);
            }
        }
        notifyDataSetChanged();
        onCurrentPositionCall(position);
    }
}
