package com.basic.eyflutter_uikit.pickers.dialogs;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.RelativeLayout;

import androidx.databinding.DataBindingUtil;

import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.beans.DropListItem;
import com.basic.eyflutter_uikit.databinding.ItemCategoryLayoutBinding;
import com.basic.eyflutter_uikit.dialogs.BaseDropListDialog;
import com.basic.eyflutter_uikit.enums.DropListShowType;
import com.basic.eyflutter_uikit.pickers.beans.MediaDirectory;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.PixelUtils;

import java.util.List;

/// 选择器分类选择dialog
public class PickerCategoryDialog extends BaseDropListDialog {

    private Context context;

    public PickerCategoryDialog(Context context, List<MediaDirectory> directories) {
        this.context = context;
        List<DropListItem> datalist = super.getDatalist();
        for (MediaDirectory directory : directories) {
            datalist.add(buildDropListItem(directory));
        }
    }

    private DropListItem buildDropListItem(MediaDirectory directory) {
        DropListItem dropListItem = new DropListItem();
        dropListItem.setId(directory.getId());
        dropListItem.setStringValue(directory.getId());
        dropListItem.setDisplayName(directory.getName());
        dropListItem.setObjValue(directory);
        return dropListItem;
    }

    @Override
    protected View buildItemView(DropListItemViewHolder holder, DropListItem listItem) {
        ViewGroup.LayoutParams vgparams = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, PixelUtils.dip2px(context, 50));
        View view = LayoutInflater.from(context).inflate(R.layout.item_category_layout, null);
        view.setLayoutParams(vgparams);
        ItemCategoryLayoutBinding binding = DataBindingUtil.bind(view);
        if (binding == null) {
            return null;
        }
        holder.displayNameTv = binding.nameTv;
        holder.stateIv = binding.checkIv;
        return binding.getRoot();
    }

    @Override
    protected void bindItemView(DropListItemViewHolder holder, DropListItem listItem) {
        holder.displayNameTv.setText(listItem.getDisplayName());
        if (listItem.isChk()) {
            holder.displayNameTv.setTextColor(0xffFF6700);
            holder.stateIv.setVisibility(View.VISIBLE);
        } else {
            holder.displayNameTv.setTextColor(0xff000000);
            holder.stateIv.setVisibility(View.INVISIBLE);
        }
    }

    @Override
    protected void onItemSelectedListener(DropListItem dlitem) {
        Object objValue = dlitem.getObjValue();
        if (!(objValue instanceof MediaDirectory)) {
            return;
        }
        List<DropListItem> datalist = getDatalist();
        MediaDirectory directory = (MediaDirectory) objValue;
        for (DropListItem listItem : datalist) {
            if (TextUtils.equals(listItem.getId(), directory.getId())) {
                listItem.setChk(true);
                directory.setCheck(true);
            } else {
                listItem.setChk(false);
            }
        }
        super.notifyDataSetChanged();
        super.dismiss();
    }

    public void show(Activity activity, View parent,String currentItemId) {
        List<DropListItem> datalist = getDatalist();
        if (ObjectJudge.isNullOrEmpty(datalist)) {
            return;
        }
        for (DropListItem listItem : datalist) {
            if (TextUtils.equals(listItem.getId(), currentItemId)) {
                listItem.setChk(true);
            } else {
                listItem.setChk(false);
            }
        }
        RelativeLayout contentView = super.buildContentView(activity, DropListShowType.EachRowOne);
        super.show(activity, parent, contentView, 0, 21, Color.parseColor("#30000000"));
        super.notifyDataSetChanged();
    }
}
