package com.basic.eyflutter_uikit.pickers.pick;

import android.content.Context;
import android.database.Cursor;
import android.os.Bundle;

import androidx.fragment.app.FragmentActivity;
import androidx.loader.app.LoaderManager;
import androidx.loader.content.Loader;

import com.basic.eyflutter_uikit.pickers.beans.MediaData;
import com.basic.eyflutter_uikit.pickers.beans.MediaDirectory;
import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.enums.SelectMode;
import com.basic.eyflutter_uikit.pickers.pick.loader.LocalMediaLoader;
import com.basic.eyflutter_uikit.pickers.utils.PickUtils;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;

public class MediaStoreManager implements LoaderManager.LoaderCallbacks<Cursor> {

    private LoaderManager mLoaderManager;
    private WeakReference<Context> mContext;
    private boolean showGif;
    private PickBean pickBean;
    private OnPickCallback pickCallback;

    public MediaStoreManager(FragmentActivity activity, int type, PickBean pickBean, OnPickCallback callback) {
        this.mContext = new WeakReference<>(activity);
        this.pickBean = pickBean;
        this.pickCallback = callback;
        this.mLoaderManager = activity.getSupportLoaderManager();
        this.mLoaderManager.initLoader(SelectMode.all.ordinal(), null, this);
    }

    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        Context context = mContext.get();
        if (context == null) {
            return null;
        }
        return new LocalMediaLoader(context, pickBean.getMode().ordinal(), showGif);
    }

    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor data) {
        Context context = mContext.get();
        if (context == null) return;
        if (data == null || data.isClosed()) return;
        List<MediaDirectory> directories = new ArrayList<>();
        MediaDirectory photoDirectoryAll = new MediaDirectory();
        while (data.moveToNext()) {
            int media_id = data.getInt(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[0]));
            int media_width = data.getInt(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[1]));
            int media_height = data.getInt(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[2]));
            // 使用DURATION获取的时长不准确
            // media_duration = data.getLong(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[3]));
            String media_path = data.getString(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[4]));
            String media_type = data.getString(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[5]));
            long media_size = data.getLong(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[6]));
            String media_dirId = data.getString(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[7]));
            String media_dirName = data.getString(data.getColumnIndexOrThrow(LocalMediaLoader.FILE_PROJECTION[8]));
            String media_directoryPath = media_path.substring(0, media_path.lastIndexOf(File.separator));
            long media_duration = PickUtils.getInstance().isVideo(media_type) ? PickUtils.getInstance().getVideoDuration(media_path) : 0;
            //判断文件是否损坏
            boolean isDamage = PickUtils.getInstance().checkImageIsDamage(media_width, media_path);
            if (isDamage) {
                continue;
            }
            MediaData mediaData = getMediaData(media_id, media_path, media_size, media_duration, pickBean.getMode().ordinal(), media_type, media_width, media_height);
            MediaDirectory mediaDirectory = new MediaDirectory();
            mediaDirectory.setId(media_dirId);
            mediaDirectory.setDirPath(media_directoryPath);
            mediaDirectory.setName(media_dirName);

            if (!directories.contains(mediaDirectory)) {
                mediaDirectory.setCoverPath(media_path);
                mediaDirectory.addMediaData(mediaData);
                directories.add(mediaDirectory);
            } else {
                directories.get(directories.indexOf(mediaDirectory)).addMediaData(mediaData);
            }
            photoDirectoryAll.addMediaData(mediaData);
        }
        MediaDirectory allPhotoDirectory = PickUtils.getInstance().getAllPhotoDirectory();
        photoDirectoryAll.setName(allPhotoDirectory.getName());
        photoDirectoryAll.setId(allPhotoDirectory.getId());
        if (photoDirectoryAll.getPhotoPaths().size() > 0) {
            photoDirectoryAll.setCoverPath(photoDirectoryAll.getPhotoPaths().get(0));
        }
        directories.add(0, photoDirectoryAll);
        data.close();

        if (pickCallback != null) {
            pickCallback.onLoadItems(directories);
        }
    }

    @Override
    public void onLoaderReset(Loader<Cursor> loader) {

    }

    public void onDestory() {
        if (mLoaderManager != null) {
            mLoaderManager.destroyLoader(pickBean.getMode().ordinal());
        }
        pickCallback = null;
    }

    private static MediaData getMediaData(int mediaId, String mediaPath, long mediaSize, long duration, int mimeType, String mediaType, int mediaWidth, int mediaHeight) {
        MediaData mediaData = new MediaData();
        mediaData.setMediaId(mediaId);
        mediaData.setOriginalPath(mediaPath);
        mediaData.setOriginalSize(mediaSize);
        mediaData.setDuration(duration);
        mediaData.setMimeType(mimeType);
        mediaData.setImageType(mediaType);
        mediaData.setImageWidth(mediaWidth);
        mediaData.setImageHeight(mediaHeight);
        return mediaData;
    }
}
