package com.basic.eyflutter_uikit.pickers.ui;

import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;

import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentActivity;

import com.basic.eyflutter_core.enums.DirectoryNames;
import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.pickers.PickerService;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.beans.ImageInfo;
import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.camera.JCameraView;
import com.basic.eyflutter_uikit.pickers.camera.listener.ClickListener;
import com.basic.eyflutter_uikit.pickers.camera.listener.ErrorListener;
import com.basic.eyflutter_uikit.pickers.camera.listener.JCameraListener;
import com.basic.eyflutter_uikit.pickers.enums.PickBehavior;
import com.basic.eyflutter_uikit.pickers.utils.PickUtils;
import com.basic.eyflutter_uikit.pickers.utils.SelectHandler;
import com.cloud.eyutils.ebus.EBus;
import com.cloud.eyutils.ebus.SubscribeEBus;
import com.cloud.eyutils.events.Action0;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.storage.files.DirectoryUtils;
import com.cloud.eyutils.storage.files.StorageUtils;
import com.cloud.eyutils.utils.GlobalUtils;
import com.gyf.immersionbar.ImmersionBar;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

public class CameraActivity extends FragmentActivity {
    private JCameraView jCameraView;
    private PickerService pickerService = new PickerService();
    private SelectHandler selectHandler = new SelectHandler();
    private File cropDestFile;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.camera_activity);
        ImmersionBar.with(this).init();
        EBus.getInstance().registered(this);
        jCameraView = (JCameraView) findViewById(R.id.jcameraview);
        //设置视频保存路径
        File directory = DirectoryUtils.getInstance().getDirectory(DirectoryNames.images.name());
        jCameraView.setSaveVideoPath(directory.getAbsolutePath());
        jCameraView.setFeatures(JCameraView.BUTTON_STATE_BOTH);
        jCameraView.setMediaQuality(JCameraView.MEDIA_QUALITY_MIDDLE);
        jCameraView.setErrorLisenter(new ErrorListener() {
            @Override
            public void onError() {
                errorHandle();
            }

            @Override
            public void AudioPermissionError() {
                errorHandle();
            }
        });
        //JCameraView监听
        jCameraView.setJCameraLisenter(new JCameraListener() {
            @Override
            public void captureSuccess(Bitmap bitmap) {
                try {
                    String fileName = String.format("%s.jpg", GlobalUtils.getGuidNoConnect());
                    File file = StorageUtils.saveBitmap(directory, fileName, bitmap);
                    List<File> files = new LinkedList<>();
                    files.add(file);
                    PickBean pickBean = PickUtils.getInstance().getPickBean();
                    List<ImageInfo> imageInfos = new LinkedList<>();
                    ImageInfo imageInfo = new ImageInfo();
                    imageInfo.setOriginUrl(file.getAbsolutePath());
                    imageInfo.setImageType(PickersConstants.jpgType);
                    imageInfos.add(imageInfo);
                    selectHandler.onPickSendClick(CameraActivity.this, imageInfos, new Action1<File>() {
                        @Override
                        public void call(File destFile) {
                            cropDestFile = destFile;
                        }
                    }, pickBean, PickUtils.getInstance().getLangs(), PickBehavior.takingRecord);
                    finish();
                } catch (Exception e) {
                    errorHandle();
                }
            }

            @Override
            public void recordSuccess(String url, Bitmap firstFrame) {
                try {
                    String fileName = String.format("%s.mp4", GlobalUtils.getGuidNoConnect());
                    File file = StorageUtils.saveBitmap(directory, fileName, firstFrame);
                    Intent intent = new Intent();
                    intent.putExtra("path", file.getAbsolutePath());
                    setResult(101, intent);
                    finish();
                } catch (Exception e) {
                    errorHandle();
                }
            }
        });

        jCameraView.setLeftClickListener(new ClickListener() {
            @Override
            public void onClick() {
                finish();
            }
        });
        jCameraView.setRightClickListener(new ClickListener() {
            @Override
            public void onClick() {

            }
        });
    }

    private void errorHandle() {
        Intent intent = new Intent();
        setResult(103, intent);
        finish();
    }

    @Override
    protected void onStart() {
        super.onStart();
    }

    @Override
    protected void onResume() {
        super.onResume();
        jCameraView.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        jCameraView.onPause();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        selectHandler.cropResultHandle(this, requestCode, resultCode, data, cropDestFile, PickBehavior.takingRecord, new Action0() {
            @Override
            public void call() {
                finish();
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EBus.getInstance().unregister(this);
    }

    @SubscribeEBus(receiveKey = PickersConstants.cameraCloseBusKey)
    public void onEventCameraClose() {
        finish();
    }
}
