package com.basic.eyflutter_uikit.pickers.ui;

import android.content.Intent;
import android.graphics.Rect;
import android.os.Build;
import android.os.Bundle;
import android.os.StrictMode;
import android.text.TextUtils;
import android.view.View;

import androidx.databinding.DataBindingUtil;
import androidx.fragment.app.FragmentActivity;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.basic.eyflutter_core.enums.DirectoryNames;
import com.basic.eyflutter_uikit.PermissionCheckAuxiliary;
import com.basic.eyflutter_uikit.R;
import com.basic.eyflutter_uikit.beans.DropListItem;
import com.basic.eyflutter_uikit.databinding.SelectPickActivityBinding;
import com.basic.eyflutter_uikit.dialogs.LoadingDialog;
import com.basic.eyflutter_uikit.enums.PermissionGrantedType;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.adapters.PickAdapter;
import com.basic.eyflutter_uikit.pickers.beans.ImageInfo;
import com.basic.eyflutter_uikit.pickers.beans.MediaData;
import com.basic.eyflutter_uikit.pickers.beans.MediaDirectory;
import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.beans.PickersLangs;
import com.basic.eyflutter_uikit.pickers.dialogs.PickerCategoryDialog;
import com.basic.eyflutter_uikit.pickers.enums.PickBehavior;
import com.basic.eyflutter_uikit.pickers.enums.SelectMode;
import com.basic.eyflutter_uikit.pickers.pick.MediaStoreManager;
import com.basic.eyflutter_uikit.pickers.pick.OnPickCallback;
import com.basic.eyflutter_uikit.pickers.utils.PickUtils;
import com.basic.eyflutter_uikit.pickers.utils.SelectHandler;
import com.basic.eyflutter_uikit.pickers.viewer.ImagePreview;
import com.cloud.eyutils.ClickEvent;
import com.cloud.eyutils.HandlerManager;
import com.cloud.eyutils.ebus.EBus;
import com.cloud.eyutils.ebus.SubscribeEBus;
import com.cloud.eyutils.events.Action0;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.events.RunnableParamsN;
import com.cloud.eyutils.storage.files.DirectoryUtils;
import com.cloud.eyutils.storage.files.StorageUtils;
import com.cloud.eyutils.utils.GlobalUtils;
import com.cloud.eyutils.utils.ObjectJudge;
import com.cloud.eyutils.utils.PixelUtils;
import com.gyf.immersionbar.ImmersionBar;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class SelectPickActivity extends FragmentActivity implements OnPickCallback, PickAdapter.OnPickSelectItemsListener {

    private SelectPickActivityBinding binding;
    private PickersLangs langs;
    private PickBean pickBean;
    private MediaStoreManager storeManager;
    private LoadingDialog loading = new LoadingDialog();
    private ArrayList<MediaData> mediaDatas = new ArrayList<>();
    private List<MediaDirectory> directories;
    private PickAdapter pickAdapter;
    private String currentItemId = "";
    private File cropDestFile;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = DataBindingUtil.setContentView(this, R.layout.select_pick_activity);
        binding.setHandler(this);
        ImmersionBar.with(this).init();
        EBus.getInstance().registered(this);
        initData();
        loading.showDialog(this, langs.getLoading(), null);
        initViews();
        //加载数据
        HandlerManager.getInstance().postDelayed(new RunnableParamsN<Object>() {
            @Override
            public void run(Object... objects) {
                delayedBind();
            }
        }, 50);

        //android 7.0系统解决拍照的问题
        //https://blog.csdn.net/guiping_ding/article/details/78502290
        if (Build.VERSION.SDK_INT >= 18) {
            StrictMode.VmPolicy.Builder builder = new StrictMode.VmPolicy.Builder();
            StrictMode.setVmPolicy(builder.build());
            builder.detectFileUriExposure();
        }
        int maxCount = pickBean.getMaxCount();
        binding.indicatorTv.setText(String.format("0/%s", maxCount));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EBus.getInstance().unregister(this);
    }

    private void delayedBind() {
        storeManager = new MediaStoreManager(this, SelectMode.all.ordinal(), pickBean, this::onLoadItems);
        binding.galleryRcl.setLayoutManager(new GridLayoutManager(this, pickBean.getSpanCount()));
        int spec = PickUtils.getInstance().getSpec();
        binding.galleryRcl.addItemDecoration(new SpaceItemDecoration(spec));
        pickAdapter = new PickAdapter(this, mediaDatas, pickBean, langs);
        pickAdapter.setSelectItemsListener(this::onPickSelectItems);
        binding.galleryRcl.setAdapter(pickAdapter);
    }

    private void initData() {
        langs = PickUtils.getInstance().getLangs();
        pickBean = PickUtils.getInstance().getPickBean();
        pickBean.setCallback(this::onLoadItems);
    }

    private void initViews() {
        binding.previewTv.setText(langs.getPreview());
        binding.sendBtn.setText(langs.getSend());
    }

    public void onBackClick(View view) {
        finish();
    }

    @Override
    public void onPickSelectItems(LinkedList<MediaData> selectItems) {
        if (ObjectJudge.isNullOrEmpty(selectItems)) {
            binding.previewTv.setTextColor(0xff999999);
            binding.previewTv.setEnabled(false);
            binding.sendBtn.setEnabled(false);
        } else {
            binding.previewTv.setTextColor(0xff518aff);
            binding.previewTv.setEnabled(true);
            binding.sendBtn.setEnabled(true);
            int count = selectItems.size();
            int total = pickBean.getMaxCount();
            binding.indicatorTv.setText(String.format("%s/%s", count, total));
        }
    }

    public void onEventSend(View view) {
        if (ClickEvent.isFastClick(view.getId())) {
            return;
        }
        File directory = DirectoryUtils.getInstance().getDirectory(DirectoryNames.images.name());
        String fileName = String.format("%s.png", GlobalUtils.getGuidNoConnect());
        File destFile = StorageUtils.getFile(directory, fileName);
        SelectHandler handler = new SelectHandler();
        LinkedList<MediaData> selectItems = pickAdapter.getSelectItems();
        List<ImageInfo> imageInfos = new LinkedList<>();
        for (MediaData selectItem : selectItems) {
            ImageInfo info = new ImageInfo();
            info.setImageType(selectItem.getImageType());
            info.setOriginUrl(selectItem.getOriginalPath());
            info.setThumbnailUrl(selectItem.getOriginalPath());
            imageInfos.add(info);
        }
        handler.onPickSendClick(this, imageInfos, new Action1<File>() {
            @Override
            public void call(File destFile) {
                cropDestFile = destFile;
            }
        }, pickBean, langs, PickBehavior.choose);
    }

    public class SpaceItemDecoration extends RecyclerView.ItemDecoration {

        private int space;
        private int bottomSpace;

        public SpaceItemDecoration(int space) {
            this.space = space;
            this.bottomSpace = PixelUtils.dip2px(getApplicationContext(), 8);
        }

        @Override
        public void getItemOffsets(Rect outRect, View view, RecyclerView parent, RecyclerView.State state) {
            int position = parent.getChildLayoutPosition(view);
            int spanCount = pickBean.getSpanCount();
            if ((position + 1) % spanCount != 0) {
                outRect.right = space;
            }
            outRect.bottom = bottomSpace;
        }
    }

    @Override
    public void onLoadItems(List<MediaDirectory> directories) {
        this.directories = directories;
        MediaDirectory allPhotoDirectory = PickUtils.getInstance().getAllPhotoDirectory();
        this.currentItemId = allPhotoDirectory.getId();
        for (MediaDirectory directory : directories) {
            if (TextUtils.equals(directory.getId(), allPhotoDirectory.getId())) {
                onEventChangeCate(directory);
                break;
            }
        }
        loading.dismiss();
    }

    public void onCategoryClick(View view) {
        PickerCategoryDialog dialog = new PickerCategoryDialog(this, directories) {
            @Override
            protected void onItemSelectedListener(DropListItem dlitem) {
                super.onItemSelectedListener(dlitem);
                Object objValue = dlitem.getObjValue();
                if ((objValue instanceof MediaDirectory)) {
                    MediaDirectory directory = (MediaDirectory) objValue;
                    onEventChangeCate(directory);
                    //更新标题文本
                    binding.menuTitleTv.setText(dlitem.getDisplayName());
                }
                currentItemId = dlitem.getId();
            }

            @Override
            protected void onDismissListener() {
                binding.menuTitleArrowIv.setImageResource(R.mipmap.menu_arrow_down);
            }
        };
        binding.menuTitleArrowIv.setImageResource(R.mipmap.menu_arrow_up);
        dialog.show(this, view, currentItemId);
    }

    public void onEventChangeCate(MediaDirectory directory) {
        mediaDatas.clear();
        SelectMode selectMode = pickBean.getMode();
        if (selectMode == SelectMode.all || selectMode == SelectMode.image || selectMode == SelectMode.video) {
            MediaData mediaData = new MediaData();
            mediaData.setCamera(true);
            mediaDatas.add(mediaData);
        }
        mediaDatas.addAll(directory.getMediaData());
        RecyclerView.Adapter adapter = binding.galleryRcl.getAdapter();
        if (adapter != null) {
            adapter.notifyDataSetChanged();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        PickUtils.getInstance().cameraResultHandle(this, requestCode, resultCode);
        SelectHandler selectHandler = new SelectHandler();
        selectHandler.cropResultHandle(this, requestCode, resultCode, data, cropDestFile, PickBehavior.choose, new Action0() {
            @Override
            public void call() {
                finish();
            }
        });
    }

    @SubscribeEBus(receiveKey = PickersConstants.startCameraBusKey)
    public void onEventStartCamera(List<String> permissions, String permissionPrompt) {
        PermissionCheckAuxiliary checkAuxiliary = new PermissionCheckAuxiliary();
        checkAuxiliary.apply(this, permissionPrompt, new Action1<PermissionGrantedType>() {
            @Override
            public void call(PermissionGrantedType grantedType) {
                if (grantedType == PermissionGrantedType.granted) {
                    //打开拍照和录制页面
                    startCameraActivity();
                }
            }
        }, permissions.toArray(new String[0]));
    }

    private void startCameraActivity() {
        SelectMode mode = pickBean.getMode();
        if (mode == SelectMode.all) {
            Intent intent = new Intent();
            intent.setClass(this, CameraActivity.class);
            startActivityForResult(intent, 100);
        } else if (mode == SelectMode.image) {
            PickUtils.getInstance().startSystemTaking(this, false);
        } else if (mode == SelectMode.video) {
            PickUtils.getInstance().startSystemTaking(this, true);
        }
    }

    public void onPreviewClick(View view) {
        if (pickAdapter == null) {
            return;
        }
        LinkedList<MediaData> selectItems = pickAdapter.getSelectItems();
        if (ObjectJudge.isNullOrEmpty(selectItems)) {
            return;
        }
        List<ImageInfo> images = new LinkedList<>();
        for (MediaData selectItem : selectItems) {
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.setThumbnailUrl(selectItem.getOriginalPath());
            imageInfo.setOriginUrl(selectItem.getOriginalPath());
            imageInfo.setImageType(selectItem.getImageType());
            images.add(imageInfo);
        }
        ImagePreview.getInstance()
                .setContext(this)
                .setIndex(0)
                .setDownIconResId(0)
                .setDownloadClickListener(null)
                .setImageInfoList(images)
                .setBigImageClickListener(null)
                .setBigImageLongClickListener(null)
                .setEnableClickClose(false)
                .setEnableDragClose(false)
                .setEnableDragCloseIgnoreScale(true)
                .setEnableUpDragClose(false)
                .start();
    }

    @SubscribeEBus(receiveKey = PickersConstants.finishPickerBusKey)
    public void onEventFinishPage() {
        finish();
    }
}
