package com.basic.eyflutter_uikit.pickers.utils;

import android.content.Context;

import com.basic.eyflutter_core.enums.DirectoryNames;
import com.basic.eyflutter_uikit.pickers.beans.ImageInfo;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.storage.files.DirectoryUtils;
import com.cloud.eyutils.storage.files.StorageUtils;
import com.cloud.eyutils.utils.ObjectJudge;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import top.zibin.luban.Luban;
import top.zibin.luban.OnCompressListener;

public class CompressionHandler {
    public void startCompression(Context context, List<ImageInfo> imageInfos, Action1<List<File>> resultAction) {
        List<String> paths = new ArrayList<>();
        for (ImageInfo imageInfo : imageInfos) {
            paths.add(imageInfo.getOriginUrl());
        }
        startCompressionByPaths(context, paths, resultAction);
    }

    public void startCompressionByPaths(Context context, List<String> paths, Action1<List<File>> resultAction) {
        File directory = DirectoryUtils.getInstance().getDirectory(DirectoryNames.compression.name());
        Luban.with(context)
                .load(paths)
                .setTargetDir(directory.getAbsolutePath())
                .setCompressListener(new CompressHandle(paths, resultAction, directory))
                .launch();
    }

    public void startCompressionByFiles(Context context, List<File> files, Action1<List<File>> resultAction) {
        List<String> paths = new ArrayList<>();
        if (!ObjectJudge.isNullOrEmpty(files)) {
            for (File file : files) {
                paths.add(file.getAbsolutePath());
            }
        }
        File directory = DirectoryUtils.getInstance().getDirectory(DirectoryNames.compression.name());
        Luban.with(context)
                .load(paths)
                .setTargetDir(directory.getAbsolutePath())
                .setCompressListener(new CompressHandle(paths, resultAction, directory))
                .launch();
    }

    private class CompressHandle implements OnCompressListener {

        private List<String> paths;
        private List<File> resultFiles = new LinkedList<>();
        private Action1<List<File>> resultAction;
        private File directory;

        public CompressHandle(List<String> paths, Action1<List<File>> resultAction, File directory) {
            this.paths = paths;
            this.resultAction = resultAction;
            this.directory = directory;
        }

        @Override
        public void onStart() {

        }

        @Override
        public void onSuccess(File file) {
            resultFiles.add(file);
            if (resultFiles.size() == paths.size()) {
                resultAction.call(resultFiles);
            }
        }

        @Override
        public void onError(Throwable e) {
            CompressFail(directory, paths, resultFiles, resultAction);
        }
    }

    private void CompressFail(File directory, List<String> paths, List<File> resultFiles, Action1<List<File>> resultAction) {
        if (directory != null) {
            StorageUtils.cleanDirectory(directory);
        }
        for (String path : paths) {
            File file = new File(path);
            if (!file.exists()) {
                continue;
            }
            resultFiles.add(file);
        }
        resultAction.call(resultFiles);
    }
}
