package com.basic.eyflutter_uikit.pickers.utils;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.text.TextUtils;

import androidx.fragment.app.FragmentActivity;

import com.basic.eyflutter_core.enums.DirectoryNames;
import com.basic.eyflutter_core.utils.UriFileUtils;
import com.basic.eyflutter_uikit.pickers.PickerService;
import com.basic.eyflutter_uikit.pickers.PickersConstants;
import com.basic.eyflutter_uikit.pickers.beans.ImageInfo;
import com.basic.eyflutter_uikit.pickers.beans.PickBean;
import com.basic.eyflutter_uikit.pickers.beans.PickersLangs;
import com.basic.eyflutter_uikit.pickers.enums.PickBehavior;
import com.cloud.eyutils.events.Action0;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.storage.files.DirectoryUtils;
import com.cloud.eyutils.storage.files.StorageUtils;
import com.cloud.eyutils.utils.GlobalUtils;
import com.cloud.eyutils.utils.ObjectJudge;
import com.yalantis.ucrop.UCrop;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class SelectHandler {

    private PickerService pickerService = new PickerService();
    private File destFile;
    private CompressionHandler compressionHandler = new CompressionHandler();

    public void onPickSendClick(FragmentActivity activity, List<ImageInfo> selectItems, Action1<File> targetAction, PickBean pickBean, PickersLangs langs, PickBehavior behavior) {
        if (ObjectJudge.isNullOrEmpty(selectItems)) {
            return;
        }
        try {
            int maxCount = pickBean.getMaxCount();
            if (maxCount > 1 || !pickBean.isClipPhoto()) {
                MultiResultCall(activity, selectItems, pickBean, behavior);
                return;
            }
            ImageInfo imageInfo = selectItems.get(0);
            if (!TextUtils.equals(imageInfo.getImageType(), PickersConstants.jpgType) && !TextUtils.equals(imageInfo.getImageType(), PickersConstants.pngType)) {
                MultiResultCall(activity, selectItems, pickBean, behavior);
                return;
            }
            String originalPath = imageInfo.getOriginUrl();
            File file = new File(originalPath);
            File directory = DirectoryUtils.getInstance().getDirectory(DirectoryNames.images.name());
            String fileName = String.format("%s.png", GlobalUtils.getGuidNoConnect());
            destFile = StorageUtils.getFile(directory, fileName);
            PickUtils.getInstance().startCrop(activity, file, destFile, langs.getCropTitle(), pickBean);
            if (targetAction != null) {
                targetAction.call(destFile);
            }
        } catch (Exception e) {
            if (destFile != null) {
                StorageUtils.forceDelete(destFile);
            }
        }
    }

    private void MultiResultCall(Context context, List<ImageInfo> selectItems, PickBean pickBean, PickBehavior behavior) {
        List<File> files = new LinkedList<>();
        List<ImageInfo> imageList = new LinkedList<>();
        for (ImageInfo imageInfo : selectItems) {
            String originalPath = imageInfo.getOriginUrl();
            File file = new File(originalPath);
            if (!file.exists()) {
                continue;
            }
            if (pickBean.isStartCompression()) {
                if (!TextUtils.equals(imageInfo.getImageType(), PickersConstants.jpgType) && !TextUtils.equals(imageInfo.getImageType(), PickersConstants.pngType)) {
                    files.add(file);
                } else {
                    imageList.add(imageInfo);
                }
            } else {
                files.add(file);
            }
        }
        if (pickBean.isStartCompression()) {
            pickerService.onPickResultCall(files, behavior);
            compressionHandler.startCompression(context, imageList, (resultFiles) -> {
                files.addAll(resultFiles);
                pickerService.onPickResultCall(files, behavior);
            });
        } else {
            pickerService.onPickResultCall(files, behavior);
        }
    }

    public void cropResultHandle(Context context, int requestCode, int resultCode, Intent data, File cropDestFile, PickBehavior behavior, Action0 cropSuccessAction) {
        if (resultCode == Activity.RESULT_OK && requestCode == UCrop.REQUEST_CROP) {
            final Uri resultUri = UCrop.getOutput(data);
            File file = UriFileUtils.getInstance().getFile(context, resultUri);
            if (!file.exists()) {
                if (cropDestFile != null) {
                    StorageUtils.forceDelete(cropDestFile);
                }
                return;
            }
            List<File> files = new LinkedList<>();
            files.add(file);
            PickBean pickBean = PickUtils.getInstance().getPickBean();
            if (pickBean.isStartCompression()) {
                List<String> paths = new ArrayList<>();
                paths.add(file.getAbsolutePath());
                compressionHandler.startCompressionByPaths(context, paths, (resultFiles) -> {
                    pickerService.onPickResultCall(resultFiles, behavior);
                });
            } else {
                pickerService.onPickResultCall(files, behavior);
            }
            if (cropSuccessAction != null) {
                cropSuccessAction.call();
            }
        } else if (resultCode == UCrop.RESULT_ERROR) {
            if (cropDestFile != null) {
                StorageUtils.forceDelete(cropDestFile);
            }
        }
    }
}
