import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';

class ExportSudokuDialog {
  factory ExportSudokuDialog() => _getInstance();

  static ExportSudokuDialog get instance => _getInstance();
  static ExportSudokuDialog? _instance;

  ExportSudokuDialog._internal();

  static ExportSudokuDialog _getInstance() {
    return _instance ??= ExportSudokuDialog._internal();
  }

  YYDialog? contentDialog;

  void show({required BuildContext context}) {
    var model = _ExportSudokuModel();
    contentDialog = Dialogs.instance.showDialog(context,
        gravity: Gravity.bottom,
        margin: EdgeInsets.zero,
        padding: EdgeInsets.zero,
        borderRadius: const BorderRadius.only(bottomLeft: Radius.zero, bottomRight: Radius.zero),
        contentView: _ExportSudokuPanel(
          model: model,
          closeCall: () {
            contentDialog?.dismiss();
            _instance = null;
          },
        ));
  }
}

class _ExportSudokuModel extends CusChangeNotifier {}

class _ExportSudokuPanel extends StatelessWidget {
  final _ExportSudokuModel model;
  final Function closeCall;

  const _ExportSudokuPanel({Key? key, required this.model, required this.closeCall}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<_ExportSudokuModel>(model, (context, model) {
      return Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Text(
            "数独类型：",
            style: TextStyle(color: Color(0xcc333333), fontSize: 14),
          ),
          RadioGroup(
            items: [
              RadioEntry(text: "四宫数独", tag: "4"),
              RadioEntry(text: "六宫数独", tag: "6"),
              RadioEntry(text: "九宫数独", tag: "9"),
              RadioEntry(text: "十二宫数独", tag: "12"),
              RadioEntry(text: "十数独", tag: "12"),
            ],
            selectIconColor: const Color(0xffCEC6A1),
            unSelectIconColor: const Color(0xffCEC6A1),
            textPadding: const EdgeInsets.only(left: 6, top: 10, right: 4, bottom: 10),
            orientation: RadioArrange.vertical,
            tag: "4",
            allowReverseSelection: false,
            isTile: true,
            changeCall: (int position, String tag) {},
          )
        ],
      );
    });
  }
}
