//
//  EyInfoCameraController.m
//  cameraPicker
//
//  Created by eyinfo on 2021/6/9.
//

#import "EyInfoCameraController.h"
#import <AVFoundation/AVFoundation.h>
#import <AssetsLibrary/AssetsLibrary.h>
#import "EyInfoCameraManager.h"
#import "UserAuthorization.h"
#import "UIView+GBLayout.h"
#import "EyInfoCameraVideoPlayer.h"
#import "TZImagePickerController.h"
#import "GCDTimer.h"
#import "Tools.h"


@interface EyInfoCameraController ()<AVCaptureFileOutputRecordingDelegate>

@property (nonatomic,strong)EyInfoCameraManager *cameraManager;
@property (nonatomic,strong)AVCaptureVideoPreviewLayer *previewLayer;

//UI
@property (nonatomic,strong)UIView *topView;
@property (nonatomic,strong)UIButton *flashBtn;
@property (nonatomic,strong)UILabel *timeLable;

@property (nonatomic,strong)UIView *bottomView;
@property (nonatomic,strong)UIButton *cancleBtn;
@property (nonatomic,strong)UIButton *switchBtn;
@property (nonatomic,strong)UIButton *takeBtn;


@property (nonatomic,strong)UIView *switchTakeView;
@property (nonatomic,strong)UIButton *photoBtn;
@property (nonatomic,strong)UIButton *videoBtn;

@property (nonatomic,strong)UIView *useBottomView;
@property (nonatomic,strong)UIButton *useBtn;
@property (nonatomic,strong)UIButton *reTakeBtn;

@property (nonatomic,strong)UIImageView *lookView;

@property (nonatomic, strong) UISwipeGestureRecognizer *leftSwipeGestureRecognizer;
@property (nonatomic, strong) UISwipeGestureRecognizer *rightSwipeGestureRecognizer;

@end

@implementation EyInfoCameraController
{

    EyInfoCameraVideoPlayer *_videoPlayer;
    NSString *_timeId;
    int _time;
}
- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self iniUI];
    [self chechAuth];
}

-(void)dealloc{
    NSLog(@"%s",__func__);
}

-(void)chechAuth{
    if (!self.permissModel) {
        return;
    }
    NSDictionary *dict = @{
        @"prompt":self.permissModel.camera.prompt?self.permissModel.camera.prompt:@"",
        @"cancle":self.langArgumentModel.cancle?self.langArgumentModel.cancle:@"",
        @"setting":self.langArgumentModel.setting?self.langArgumentModel.setting:@"",
        @"permissions":self.permissModel.camera.permissions?self.permissModel.camera.permissions:@[]
    };
    [UserAuthorization authorizationsWithParams:dict callBack:^(NSString *callkey, BOOL granted) {
        if (granted) {
            [self.cameraManager sessionStartRunning];
        }
    }];
}

//闪光灯
-(void)flashBtnAction{
    self.flashBtn.selected = !self.flashBtn.isSelected;
    [self.cameraManager switchFlashwith:self.flashBtn.selected];
}

-(void)cancleBtnAction{
    if (self.cameraManager.isRecording) {
        return;
    }
    [self.cameraManager sessionStopRunning];
    [self.cameraManager cleanMovieFile];
    //关闭闪光灯
    [self.cameraManager switchFlashwith:NO];
    [GCDTimer cancelTask:_timeId];
    [self dismissViewControllerAnimated:YES completion:^{
        if (self.didFinishRecordingAndSaveToPhotoAlbum) {
            self.didFinishRecordingAndSaveToPhotoAlbum(nil, nil);
        }
        if (self.didFinishRecordingToOutputFileAtURL) {
            self.didFinishRecordingToOutputFileAtURL(nil, nil);
        }
        if (self.didFinishTakePictureAndSaveToPhotoAlbum) {
            self.didFinishTakePictureAndSaveToPhotoAlbum(nil, nil);
        }
    }];
}

-(void)takeBtnAction{
    
    if (!self.cameraManager.session.isRunning) {
        return;
    }
    
    __weak typeof(self) weakself = self;
    if (self.photoBtn.selected) {
        [self.cameraManager takePhotoWith:^(UIImage * _Nonnull imgage, NSError * _Nonnull error) {
            NSLog(@"%@",imgage);
            [weakself showLookViewWith:imgage];
        }];
    }

    if (self.videoBtn.selected) {
        if (self.cameraManager.isRecording) {
            [self.cameraManager stopRecord];
            //关闭闪光灯
            [self.cameraManager switchFlashwith:NO];
            self.timeLable.text = @"00:00:00";
            _time = 0;
            [UIView animateWithDuration:0.25 animations:^{
                self.timeLable.backgroundColor = [UIColor clearColor];
            }];
            [GCDTimer cancelTask:_timeId];
        }else{
            self.switchBtn.hidden = YES;
            self.cancleBtn.hidden = YES;
            self.switchTakeView.hidden = YES;
            
            [UIView animateWithDuration:0.25 animations:^{
                self.timeLable.backgroundColor = [UIColor redColor];
            }];
            _timeId = [GCDTimer execTask:self selector:@selector(recordTime) start:1 interval:1 repeats:YES async:NO];
           
            
            [self.cameraManager startRecordWith:^(NSURL * _Nonnull fileUrl, NSError * _Nonnull error) {
  
                [weakself showLookMovieViewWith:[fileUrl copy]];
               
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    weakself.switchTakeView.hidden = NO;
                    weakself.switchBtn.hidden = NO;
                    weakself.cancleBtn.hidden = NO;
                });
                
            }];
        }
       
    }
   
}

-(void)switchBtnAction{
    self.switchBtn.selected = !self.switchBtn.isSelected;
    
    CATransition *animation = [CATransition animation];
    animation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    animation.duration = 0.5;
    animation.type = @"oglFlip";
    
    AVCaptureDevicePosition position = [[self.cameraManager.input device] position];
 
    if (position == AVCaptureDevicePositionFront) {
        animation.subtype = kCATransitionFromLeft;
 
    }else if (position == AVCaptureDevicePositionBack){
        animation.subtype = kCATransitionFromRight;
    }
    //为摄像头的转换加转场动画
    [self.previewLayer addAnimation:animation forKey:nil];
    
    [self.cameraManager changeWebcam];
}

-(void)photoBtnAction{
    self.photoBtn.selected = YES;
    self.videoBtn.selected = NO;
    [self switchTimeLable];
    
    if (self.takeRecording) {
        [UIView animateWithDuration:0.6 animations:^{
            self.photoBtn.GB_centerX = self.bottomView.GB_centerX;
            self.videoBtn.GB_left = self.bottomView.GB_width/2.0 - self.videoBtn.GB_width - 15 - self.photoBtn.GB_width/2.0;
        }];
    }
    
}
-(void)videoBtnAction{

    self.videoBtn.selected = YES;
    self.photoBtn.selected = NO;
    [self switchTimeLable];
    
    if (self.takeRecording) {
        [UIView animateWithDuration:0.6 animations:^{
            self.videoBtn.GB_centerX = self.bottomView.GB_centerX;
            self.photoBtn.GB_left = self.bottomView.GB_width/2.0 + 15 + self.videoBtn.GB_width/2.0;
        }];
    }
}

#pragma mark useBottomViewAction
-(void)reTakeBtnAction{
    [self.useBottomView removeFromSuperview];
    [self.lookView removeFromSuperview];
    self.lookView.image = nil;
    
    if (self.videoBtn.selected) {
        [self.cameraManager cleanFile:_videoPlayer.movieURL];
    }
    
    [_videoPlayer removeFromSuperview];
    _videoPlayer = nil;
    
}

-(void)useBtnAction{
    if (self.videoBtn.selected) {
        if (self.videoSaveToPhotoAlbum) {
            [EyInfoCameraManager saveVideoWithUrl:[_videoPlayer.movieURL copy] location:nil completion:^(PHAsset * _Nonnull asset, NSError * _Nonnull error) {
                if (self.didFinishRecordingAndSaveToPhotoAlbum) {
                    self.didFinishRecordingAndSaveToPhotoAlbum(asset, error);
                }
            }];
        }else{
            if (self.didFinishRecordingToOutputFileAtURL) {
                self.didFinishRecordingToOutputFileAtURL([_videoPlayer.movieURL copy], nil);
            }
        }
        
    }else{
        [EyInfoCameraManager savePhotoWithImage:[self.lookView.image copy] completion:self.didFinishTakePictureAndSaveToPhotoAlbum];
    }
    [self.cameraManager sessionStopRunning];
    [self dismissViewControllerAnimated:YES completion:nil];
}


-(void)showLookViewWith:(UIImage *)image{
    self.lookView.image = image;
    self.lookView.frame = CGRectMake(0 , 0, screenw, self.useBottomView.GB_top);
    [self.view addSubview:self.lookView];
    [self.view addSubview:self.useBottomView];
    [self.view bringSubviewToFront:self.useBottomView];
}

-(void)showLookMovieViewWith:(NSURL *)movieUrl{
    EyInfoCameraVideoPlayer *movieView = [EyInfoCameraVideoPlayer videoPlayerWithMovieUrl:movieUrl];
    movieView.frame = CGRectMake(0 , 0, screenw, self.useBottomView.GB_top);
    [self.view addSubview:self.useBottomView];
    [self.view addSubview:movieView];
    [self.view bringSubviewToFront:self.useBottomView];
    
    _videoPlayer = movieView;
}

#pragma mark 时间lable 计时
- (void)recordTime{
    _time++;
    int seconds = _time % 60;
    int minutes = (_time / 60) % 60;
    int hours = _time / 3600;

    self.timeLable.text = [NSString stringWithFormat:@"%02d:%02d:%02d",hours, minutes, seconds];
    if (self.maxDuration>0 && _time>=self.maxDuration) {
        [self takeBtnAction];
    }
}

#pragma mark 时间lable 显示或者隐藏
-(void)switchTimeLable{
    self.timeLable.hidden = !self.videoBtn.selected;
}

#pragma mark lazy
- (void)handleSwipes:(UISwipeGestureRecognizer *)sender
{
    if (self.switchTakeView.subviews.count==1) {
        ((UIButton *)self.switchTakeView.subviews.firstObject).selected = YES;
        [self switchTimeLable];
        return;
    }
    
    if (sender.direction == UISwipeGestureRecognizerDirectionLeft) {
        [self photoBtnAction];
    }
    if (sender.direction == UISwipeGestureRecognizerDirectionRight) {
        [self videoBtnAction];
    }
}

#pragma Setter
-(void)setLangArgumentModel:(pickerLangArgumentModel *)langArgumentModel{
    _langArgumentModel = langArgumentModel;

    if (![Tools isBlankString:langArgumentModel.video]) {
        self.videoBtnTitle = langArgumentModel.video;
    }
    if (![Tools isBlankString:langArgumentModel.photo]) {
        self.photoBtnTitle = langArgumentModel.photo;
    }
    if (![Tools isBlankString:langArgumentModel.send]) {
        self.useBtnTitle = langArgumentModel.send;
    }
    if (![Tools isBlankString:langArgumentModel.reTake]) {
        self.reTakeBtnTitle = langArgumentModel.reTake;
    }
    
}

-(void)setUseBtnTitle:(NSString *)useBtnTitle{
    _useBtnTitle = useBtnTitle;
    [self.useBtn setTitle:useBtnTitle forState:UIControlStateNormal];
    [self.useBtn sizeToFit];
    
}
-(void)updateUseBtn{
    _useBtn.GB_centerY = self.useBottomView.GB_height/2.0;
    _useBtn.GB_size = CGSizeMake(_useBtn.GB_width + 28, 35);
    _useBtn.GB_right = self.useBottomView.GB_width - 15;

//    [Tools roundAngleWithView:_useBtn Radius:_useBtn.GB_right/2.0 color:nil lineWidth:0];
    [Tools cornerByRoundingWith:_useBtn Corners:UIRectCornerAllCorners cornerRadius:_useBtn.GB_height/2.0];
    [Tools changeAlphaWithView:_useBtn Colors:@[[Tools colorWithHex:0xFFFF4949],[Tools colorWithHex:0xFFFF8123]] start:CGPointMake(0, 0.5) endPoint:CGPointMake(1, 0.5)];
  
}
-(void)setPhotoBtnTitle:(NSString *)photoBtnTitle{
    _photoBtnTitle = photoBtnTitle;
    [self.photoBtn setTitle:photoBtnTitle forState:UIControlStateNormal];
    [self.photoBtn sizeToFit];
}

-(void)setVideoBtnTitle:(NSString *)videoBtnTitle{
    _videoBtnTitle = videoBtnTitle;
    [self.videoBtn setTitle:videoBtnTitle forState:UIControlStateNormal];
    [self.videoBtn sizeToFit];
}

-(void)setReTakeBtnTitle:(NSString *)reTakeBtnTitle{
    _reTakeBtnTitle = reTakeBtnTitle;
    [self.reTakeBtn setTitle:reTakeBtnTitle forState:UIControlStateNormal];
    [self.reTakeBtn sizeToFit];
}
#pragma mark  INITUI
-(void)iniUI{
    [self.view.layer addSublayer:self.previewLayer];
    [self.view addSubview:self.topView];
    [self.view addSubview:self.bottomView];
    
    self.topView.frame = CGRectMake(0, 0, screenw, statusBarh + 40);
    self.bottomView.frame = CGRectMake(0, screenh - 140, screenw, 140);
    self.previewLayer.frame = CGRectMake(0, 0, screenw, screenh);
    
//    [self.topView addSubview:self.flashBtn];
    [self.topView addSubview:self.timeLable];
    self.timeLable.frame = CGRectMake((self.topView.GB_width - self.timeLable.GB_width - 10)/2.0, (self.topView.GB_height - self.timeLable.GB_height - 4)/2.0 + statusBarh/2.0, self.timeLable.GB_width + 10, self.timeLable.GB_height +4);
    [self switchTimeLable];
    
    [self.bottomView addSubview:self.switchTakeView];
    [self.bottomView addSubview:self.cancleBtn];
    [self.bottomView addSubview:self.switchBtn];
    [self.bottomView addSubview:self.takeBtn];
    
    self.flashBtn.frame = CGRectMake(20, statusBarh, 40, 40);
   
    
    self.switchTakeView.frame = CGRectMake(0, 0, self.bottomView.GB_width, 30);
    
    self.cancleBtn.frame = CGRectMake(20, (self.bottomView.GB_height - self.cancleBtn.GB_height)/2.0 + self.switchTakeView.GB_height/2.0, self.cancleBtn.GB_width, self.cancleBtn.GB_height);
    
    self.takeBtn.frame = CGRectMake((self.bottomView.GB_width - self.takeBtn.GB_width)/2.0, (self.bottomView.GB_height -  self.takeBtn.GB_height)/2.0 + self.switchTakeView.GB_height/2.0,  self.takeBtn.GB_width,  self.takeBtn.GB_height);
    
    self.switchBtn.frame = CGRectMake(self.bottomView.GB_width - 20 - self.switchBtn.GB_width, (self.bottomView.GB_height - self.switchBtn.GB_height)/2.0 + self.switchTakeView.GB_height/2.0, self.switchBtn.GB_width, self.switchBtn.GB_height);
    
    
    if (!self.takeImage&&!self.takeRecording) {
        [self.switchTakeView addSubview:self.photoBtn];
        self.photoBtn.center = CGPointMake(self.switchTakeView.GB_centerX, self.switchTakeView.GB_height/2.0);
    }
    
    if (self.takeImage) {
        [self.switchTakeView addSubview:self.photoBtn];
        self.photoBtn.center = CGPointMake(self.switchTakeView.GB_centerX, self.switchTakeView.GB_height/2.0);
    }
    
    if (self.takeRecording) {
        [self.switchTakeView addSubview:self.videoBtn];
        if (self.takeImage) {
            self.videoBtn.GB_origin = CGPointMake(self.photoBtn.GB_left - self.videoBtn.GB_width - 15, self.photoBtn.GB_top);
        }else{
            self.videoBtn.center = CGPointMake(self.switchTakeView.GB_centerX, self.switchTakeView.GB_height/2.0);
            [self videoBtnAction];
        }
    }
    
    self.useBottomView.frame = CGRectMake(self.bottomView.GB_left, screenh - 55 - kBottomSafeHeight, self.bottomView.GB_width, 55 + kBottomSafeHeight);
    [self.useBottomView addSubview:self.reTakeBtn];
    [self.useBottomView addSubview:self.useBtn];
    
    self.reTakeBtn.frame = CGRectMake(15, (self.useBottomView.GB_height - self.reTakeBtn.GB_height)/2.0, self.reTakeBtn.GB_width, self.reTakeBtn.GB_height);
    [self updateUseBtn];
}



#pragma mark lazy
-(EyInfoCameraManager *)cameraManager{
    if (!_cameraManager) {
        _cameraManager = [EyInfoCameraManager defaultManager];
    }
    return _cameraManager;
}


#pragma mark previewLayer
-(AVCaptureVideoPreviewLayer *)previewLayer{
    if (_previewLayer == nil) {
        _previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:self.cameraManager.session];
        _previewLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
        _previewLayer.frame = CGRectMake(0,0, screenw, screenh);
        
        UIInterfaceOrientation statusBarOrientation = [UIApplication sharedApplication].statusBarOrientation;
        AVCaptureVideoOrientation initialVideoOrientation = AVCaptureVideoOrientationPortrait;
        if (statusBarOrientation != UIInterfaceOrientationUnknown) {
            initialVideoOrientation = (AVCaptureVideoOrientation)statusBarOrientation;
        }
        // videoPreviewLayer 也有 connection
        _previewLayer.connection.videoOrientation = initialVideoOrientation;
    }
    
    return _previewLayer;
}

#pragma mark UI
-(UIView *)topView{
    if (!_topView) {
        _topView = [[UIView alloc] init];
        _topView.backgroundColor = [UIColor clearColor];
    }
    return _topView;
}
-(UIButton *)flashBtn{
    if (!_flashBtn) {
        _flashBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 40, 40)];
        [_flashBtn setTitle:@"闪光" forState:UIControlStateNormal];
        [_flashBtn setTitle:@"没光" forState:UIControlStateSelected];
        [_flashBtn addTarget:self action:@selector(flashBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _flashBtn;
}

-(UILabel *)timeLable{
    if (!_timeLable) {
        _timeLable = [[UILabel alloc] init];
        _timeLable.font = [UIFont boldSystemFontOfSize:14];
        _timeLable.textColor = [UIColor whiteColor];
        _timeLable.textAlignment = NSTextAlignmentCenter;
        _timeLable.text = @"00:00:00";
        [_timeLable sizeToFit];
        _timeLable.clipsToBounds = YES;
        _timeLable.layer.cornerRadius = 3;
    }
    return _timeLable;;
}


-(UIView *)bottomView{
    if (!_bottomView) {
        _bottomView = [[UIView alloc] init];
        _bottomView.backgroundColor = [UIColor clearColor];
    }
    return _bottomView;
}
-(UIButton *)cancleBtn{
    if (!_cancleBtn) {
        _cancleBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 40, 40)];
        [_cancleBtn setImage:[UIImage tz_imageNamedFromMyBundle:@"phto_cancle"] forState:UIControlStateNormal];
        [_cancleBtn addTarget:self action:@selector(cancleBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _cancleBtn;
}

-(UIButton *)switchBtn{
    if (!_switchBtn) {
        _switchBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 40, 40)];
        [_switchBtn setImage:[UIImage tz_imageNamedFromMyBundle:@"lens_switch"] forState:UIControlStateNormal];
        [_switchBtn addTarget:self action:@selector(switchBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _switchBtn;
}

-(UIButton *)takeBtn{
    if (!_takeBtn) {
        _takeBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 64, 64)];
        _takeBtn.backgroundColor = [Tools colorWithHex:0xFFF4F4F4];
        [Tools roundAngleWithView:_takeBtn Radius:64/2.0 color:[Tools colorWithHex:0xffffffff] lineWidth:5];
        [_takeBtn addTarget:self action:@selector(takeBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _takeBtn;
}

-(UIView *)switchTakeView{
    if (!_switchTakeView) {
        _switchTakeView = [[UIView alloc] init];
        _switchTakeView.backgroundColor = [UIColor clearColor];
        
        [_switchTakeView addGestureRecognizer:self.leftSwipeGestureRecognizer];
        [_switchTakeView addGestureRecognizer:self.rightSwipeGestureRecognizer];
    }
    return _switchTakeView;
}

-(UIButton *)videoBtn{
    if (!_videoBtn) {
        _videoBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 40, 40)];
        [_videoBtn setTitle:@"视频" forState:UIControlStateNormal];
        [_videoBtn.titleLabel setFont:[UIFont systemFontOfSize:13]];
        [_videoBtn setTitleColor:[UIColor colorWithRed:1 green:1 blue:1 alpha:0.7] forState:UIControlStateNormal];
        [_videoBtn setTitleColor:[UIColor colorWithRed:1 green:1 blue:1 alpha:1] forState:UIControlStateSelected];
        [_videoBtn sizeToFit];
        [_videoBtn addTarget:self action:@selector(videoBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _videoBtn;
}

-(UIButton *)photoBtn{
    if (!_photoBtn) {
        _photoBtn = [[UIButton alloc] initWithFrame:CGRectMake(20, statusBarh, 40, 40)];
        [_photoBtn setTitle:@"照片" forState:UIControlStateNormal];
        [_photoBtn.titleLabel setFont:[UIFont systemFontOfSize:13]];
        _photoBtn.selected = YES;
        [_photoBtn setTitleColor:[UIColor colorWithRed:1 green:1 blue:1 alpha:0.7] forState:UIControlStateNormal];
        [_photoBtn setTitleColor:[UIColor colorWithRed:1 green:1 blue:1 alpha:1] forState:UIControlStateSelected];
        [_photoBtn sizeToFit];
        [_photoBtn addTarget:self action:@selector(photoBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _photoBtn;
}

-(UISwipeGestureRecognizer *)leftSwipeGestureRecognizer{
    if (!_leftSwipeGestureRecognizer) {
        _leftSwipeGestureRecognizer = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipes:)];
        _leftSwipeGestureRecognizer.direction = UISwipeGestureRecognizerDirectionLeft;
    }
    return _leftSwipeGestureRecognizer;;
}

-(UISwipeGestureRecognizer *)rightSwipeGestureRecognizer{
    if (!_rightSwipeGestureRecognizer) {
        _rightSwipeGestureRecognizer =  [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipes:)];
        _rightSwipeGestureRecognizer.direction = UISwipeGestureRecognizerDirectionRight;
    }
    return _rightSwipeGestureRecognizer;
}

-(UIImageView *)lookView{
    if (!_lookView) {
        _lookView = [[UIImageView alloc]init];
        _lookView.backgroundColor = [UIColor clearColor];
    }
    return _lookView;
}

-(UIView *)useBottomView{
    if (!_useBottomView) {
        _useBottomView = [[UIView alloc] init];
        _useBottomView.backgroundColor = [Tools colorWithHex:0xffffffff];
    }
    return _useBottomView;
}
-(UIButton *)reTakeBtn{
    if (!_reTakeBtn) {
        _reTakeBtn = [[UIButton alloc] init];
        [_reTakeBtn setTitle:@"Retake" forState:UIControlStateNormal];
        [_reTakeBtn setTitleColor:[Tools colorWithHex:0xFF518AFF] forState:UIControlStateNormal];
        [_reTakeBtn sizeToFit];
        [_reTakeBtn addTarget:self action:@selector(reTakeBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _reTakeBtn;
}

-(UIButton *)useBtn{
    if (!_useBtn) {
        _useBtn = [[UIButton alloc] init];
        [_useBtn setTitle:@"Send" forState:UIControlStateNormal];
        [_useBtn setTitleColor:[Tools colorWithHex:0xFFffffFF] forState:UIControlStateNormal];
        [_useBtn sizeToFit];
        [_useBtn addTarget:self action:@selector(useBtnAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _useBtn;
}
@end
