//
//  EyInfoCameraManager.m
//  cameraPicker
//
//  Created by eyinfo on 2021/6/10.
//

#import "EyInfoCameraManager.h"
#import <UIKit/UIKit.h>
@interface EyInfoCameraManager ()<AVCaptureFileOutputRecordingDelegate>
@property (nonatomic,strong)AVCaptureDevice *device;
@property (nonatomic,strong)AVCaptureDevice *audioDevice;
@property (nonatomic,strong)AVCaptureDeviceInput *input;
@property (nonatomic,strong)AVCaptureDeviceInput *audioInput;

// 出流对象
@property (nonatomic,strong) AVCaptureMovieFileOutput *videoFileOutput;
@property (nonatomic,strong) AVCaptureStillImageOutput *photoOutput;

// 队列
@property (nonatomic,strong) dispatch_queue_t queue;

@property (nonatomic,copy) NSURL *movieURL;

@end

@implementation EyInfoCameraManager
+(instancetype)defaultManager{
    return  [[EyInfoCameraManager alloc] init];
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        [self configSeccion];
    }
    return self;
}

-(void)dealloc{
    NSLog(@"%s",__func__);
    [self stopRecord];
    [_session stopRunning];
    [_session removeInput:_input];
    [_session removeInput:_audioInput];
    [_session removeOutput:_photoOutput];
    [_session removeOutput:_videoFileOutput];
}

-(void)sessionStartRunning{
    [self.session startRunning];
}

-(void)sessionStopRunning{
    [self stopRecord];
    [self.session stopRunning];
}

-(void)configSeccion{
    [self.session beginConfiguration];
    
    if ([self.session canAddInput:self.audioInput]){
        [self.session addInput:self.audioInput];
    }
    
    if ([self.session canAddInput:self.input]){
        [self.session addInput:self.input];
    }
    
    if ([self.session canAddOutput:self.videoFileOutput]) {
        [self.session addOutput:self.videoFileOutput];
    }
    
    if ([self.session canAddOutput:self.photoOutput]) {
        [self.session addOutput:self.photoOutput];
    }
    
    [self.session commitConfiguration];
}


#pragma mark ---摄像---
-(void)startRecordWith:(void (^)(NSURL * _Nonnull, NSError * _Nonnull))complete{
    self.didFinishRecordingToOutputFileAtURL = complete;
    if (self.videoFileOutput.isRecording) {
        return;
    }
    self.movieURL = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@%@%@", NSTemporaryDirectory(),[NSDate date], @"movie.mov"]];
    
    //设置视频方向。会闪一下
//    AVCaptureConnection *videoConnection = [self.videoFileOutput connectionWithMediaType:AVMediaTypeVideo];
//    if (videoConnection.supportsVideoStabilization) {
//        videoConnection.preferredVideoStabilizationMode = AVCaptureVideoStabilizationModeAuto;
//    }
//    if (videoConnection.supportsVideoOrientation) {
//        videoConnection.videoOrientation = AVCaptureVideoOrientationPortrait;
//    }
    
    [self.videoFileOutput startRecordingToOutputFileURL:self.movieURL recordingDelegate:self];
}


-(void)stopRecord{
    [self switchFlashwith:NO];
    if (self.videoFileOutput.isRecording) {
        [self.videoFileOutput stopRecording];
    }
}

- (void)captureOutput:(nonnull AVCaptureFileOutput *)output didFinishRecordingToOutputFileAtURL:(nonnull NSURL *)outputFileURL fromConnections:(nonnull NSArray<AVCaptureConnection *> *)connections error:(nullable NSError *)error {
    NSLog(@"%@",outputFileURL);

    if (self.didFinishRecordingToOutputFileAtURL) {
        self.didFinishRecordingToOutputFileAtURL(self.movieURL, error);
    }
}

#pragma mark ---拍照---
-(void)takePhotoWith:(void (^)(UIImage *image, NSError *error))completion{
    self.didFinishTakePhoto = completion;
    
    AVCaptureConnection* stillImageConnection = [self.photoOutput connectionWithMediaType:AVMediaTypeVideo];
    // 2. 设置 stillImageConnection
    if (stillImageConnection.supportsVideoStabilization) {
        stillImageConnection.preferredVideoStabilizationMode = AVCaptureVideoStabilizationModeAuto;
    }
    if (stillImageConnection.supportsVideoOrientation) {
        stillImageConnection.videoOrientation = AVCaptureVideoOrientationPortrait;
    }
    
    
    [self.photoOutput captureStillImageAsynchronouslyFromConnection:stillImageConnection completionHandler:^(CMSampleBufferRef  _Nullable imageDataSampleBuffer, NSError * _Nullable error) {
        if (!imageDataSampleBuffer || error) {
            // TODO: - 处理错误
            NSLog(@"拍照出错");
            if (self.didFinishTakePhoto) {
                self.didFinishTakePhoto(nil, error);
            }
            return;
        }
        // 1. 获取 originImage
        NSData *imageData = [AVCaptureStillImageOutput jpegStillImageNSDataRepresentation:imageDataSampleBuffer];
        UIImage *originImage = [[UIImage alloc] initWithData:imageData];
        
        if (self.didFinishTakePhoto) {
            self.didFinishTakePhoto(originImage, error);
        }
        
    }];
}
#pragma mark ---保存视频到相册---
+ (void)saveVideoWithUrl:(NSURL *)url location:(CLLocation *)location completion:(void (^)(PHAsset *asset, NSError *error))completion {
    __block NSString *localIdentifier = nil;
    [[PHPhotoLibrary sharedPhotoLibrary] performChanges:^{
        PHAssetChangeRequest *request = [PHAssetChangeRequest creationRequestForAssetFromVideoAtFileURL:url];
        localIdentifier = request.placeholderForCreatedAsset.localIdentifier;
        if (location) {
            request.location = location;
        }
        
        request.creationDate = [NSDate date];
    } completionHandler:^(BOOL success, NSError *error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (success && completion && localIdentifier) {
                [self fetchAssetByIocalIdentifier:localIdentifier retryCount:10 completion:completion];
            } else {
                if (error) {
                    NSLog(@"保存视频出错:%@",error.localizedDescription);
                }
                if (completion) {
                    completion(nil, error);
                }
            }
        });
    }];
}

#pragma mark ---保存图片---
+ (void)savePhotoWithImage:(UIImage *)image completion:(void (^)(PHAsset *asset, NSError *error))completion {
    __block NSString *localIdentifier = nil;
    [[PHPhotoLibrary sharedPhotoLibrary] performChanges:^{
        PHAssetChangeRequest *request = [PHAssetChangeRequest creationRequestForAssetFromImage:image];
        localIdentifier = request.placeholderForCreatedAsset.localIdentifier;
        request.creationDate = [NSDate date];
    } completionHandler:^(BOOL success, NSError *error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (success && completion && localIdentifier) {
                [self fetchAssetByIocalIdentifier:localIdentifier retryCount:10 completion:completion];
            } else {
                if (error) {
                    NSLog(@"保存照片出错:%@",error.localizedDescription);
                }
                if (completion) {
                    completion(nil, error);
                }
            }
        });
    }];
}

+ (void)fetchAssetByIocalIdentifier:(NSString *)localIdentifier retryCount:(NSInteger)retryCount completion:(void (^)(PHAsset *asset, NSError *error))completion {
    PHAsset *asset = [[PHAsset fetchAssetsWithLocalIdentifiers:@[localIdentifier] options:nil] firstObject];
    if (asset || retryCount <= 0) {
        if (completion) {
            completion(asset, nil);
        }
        return;
    }
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self fetchAssetByIocalIdentifier:localIdentifier retryCount:retryCount - 1 completion:completion];
    });
}
//删除这个工具创建的视频文件
- (void)cleanMovieFile{
    [self cleanFile:self.movieURL];
}
// 根据路径删除文件
- (void)cleanFile:(NSURL *)filrUrl;
{
    // 利用NSFileManager实现对文件的管理
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if ([fileManager fileExistsAtPath:[filrUrl path]]) {
        [fileManager removeItemAtPath:[filrUrl path] error:nil];
    }
}


#pragma mark ---闪光灯切换---
-(void)switchFlashwith:(BOOL)isOpen{
    
    if (isOpen) { //打开闪光灯
        NSError *error = nil;
        if ([self.device hasTorch]) {
            BOOL locked = [self.device lockForConfiguration:&error];
            if (locked) {
                self.device.torchMode = AVCaptureTorchModeOn;
                [self.device unlockForConfiguration];
            }
        }
    }else{//关闭闪光灯
        if ([self.device hasTorch]) {
            [self.device lockForConfiguration:nil];
            [self.device setTorchMode: AVCaptureTorchModeOff];
            [self.device unlockForConfiguration];
        }
    }
}

#pragma mark ---切换摄像头---
/**
 切换摄像头按钮的点击方法的实现（切换摄像头时可以添加转场动画）
 */
-(void)changeWebcam{
    //获取摄像头的数量（该方法会返回当前能够输入视频的全部设备，包括前后摄像头和外接设备）
    NSInteger cameraCount = [[AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo] count];
    //摄像头的数量小于等于1的时候直接返回
    if (cameraCount <= 1) {
        return;
    }
    AVCaptureDevice *newCamera = nil;
    AVCaptureDeviceInput *newInput = nil;
    //获取当前相机的方向（前/后）
    AVCaptureDevicePosition position = [[self.input device] position];
 
 
    if (position == AVCaptureDevicePositionFront) {
        newCamera = [self cameraWithPosition:AVCaptureDevicePositionBack];
    }else if (position == AVCaptureDevicePositionBack){
        newCamera = [self cameraWithPosition:AVCaptureDevicePositionFront];
    }
   
    //输入流
    newInput = [AVCaptureDeviceInput deviceInputWithDevice:newCamera error:nil];
    if (newInput != nil) {
       
        [self.session stopRunning];
        [self.session beginConfiguration];
        //先移除原来的input
        [self.session removeInput:self.input];
        if ([self.session canAddInput:newInput]) {
            [self.session addInput:newInput];
            self.input = newInput;
        }else{
            //如果不能加现在的input，就加原来的input
            [self.session addInput:self.input];
        }
        NSLog(@"");
        [self.session commitConfiguration];
        [self.session startRunning];

        
    }
    
}


-(AVCaptureDevice *)cameraWithPosition:(AVCaptureDevicePosition)position{
    NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
    for (AVCaptureDevice *device in devices )
        if ( device.position == position )
            return device;
    return nil;
}

-(BOOL)isRecording{
    return self.videoFileOutput.isRecording;
}

#pragma mark ---lazy---
- (AVCaptureDevice *)audioDevice{
    if (!_audioDevice) {
        _audioDevice = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeAudio];
    }
    return _audioDevice;
}
-(AVCaptureDevice *)device {
    if (!_device) {
        _device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        
        NSError *error = nil;
        if ([_device lockForConfiguration:&error]) {
            if ([_device isSmoothAutoFocusSupported]) {// 平滑对焦
                _device.smoothAutoFocusEnabled = YES;
            }
            
            if ([_device isFocusModeSupported:AVCaptureFocusModeContinuousAutoFocus]) {// 自动持续对焦
                _device.focusMode = AVCaptureFocusModeContinuousAutoFocus;
            }
            
            if ([_device isExposureModeSupported:AVCaptureExposureModeContinuousAutoExposure ]) {// 自动持续曝光
                _device.exposureMode = AVCaptureExposureModeContinuousAutoExposure;
            }
            
            if ([_device isWhiteBalanceModeSupported:AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance]) {// 自动持续白平衡
                _device.whiteBalanceMode = AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance;
            }
            
        }
        
        [_device unlockForConfiguration];
        
    }
    
    return _device;
}

#pragma mark queue
-(dispatch_queue_t)queue {
    if (_queue == nil) {
        _queue = dispatch_get_main_queue();
    }
    
    return _queue;
}
#pragma mark metadataOutput
-(AVCaptureDeviceInput *)input{
    if (!_input) {
        NSError *error = nil;
        _input = [AVCaptureDeviceInput deviceInputWithDevice:self.device error:&error];
        // 2、设置输入：由于模拟器没有摄像头，因此最好做一个判断
        if (error) {
            NSLog(@"摄像头不可以");
        }
        
    }
    return _input;
}
- (AVCaptureDeviceInput *)audioInput{
    if (!_audioInput) {
        NSError *error = nil;
        _audioInput = [[AVCaptureDeviceInput alloc] initWithDevice:self.audioDevice error:&error];
        
    }
    return _audioInput;
}
#pragma mark session
-(AVCaptureSession *)session {
    if (_session == nil) {
        _session = [[AVCaptureSession alloc] init];
        _session.sessionPreset = AVCaptureSessionPresetHigh;
    }
    return _session;
}


-(AVCaptureStillImageOutput *)photoOutput{
    if (!_photoOutput) {
        _photoOutput =  [AVCaptureStillImageOutput new];
        _photoOutput.outputSettings = @{AVVideoCodecKey: AVVideoCodecJPEG};
    }
    return _photoOutput;
}

-(AVCaptureMovieFileOutput *)videoFileOutput{
    if (!_videoFileOutput) {
        _videoFileOutput = [AVCaptureMovieFileOutput new];
    }
    return _videoFileOutput;
}

@end
