//
//  TZPhtotGroupView.m
//  cloud_basic_pickers
//
//  Created by goboo on 2021/6/15.
//

#import "TZPhtotGroupView.h"
#import "TZImagePickerController.h"
#import "TZImageManager.h"
#import "Tools.h"
#import "UIView+TZLayout.h"
@interface TZPhtotGroupView ()<UITableViewDelegate,UITableViewDataSource>
@property (nonatomic,strong)UITableView *tableview;
@property (nonatomic,strong)NSMutableArray *sourceArray;
@end
@implementation TZPhtotGroupView

+(TZPhtotGroupView *)groupViewWithCurrentModel:(TZAlbumModel *)model click:(void (^)(TZAlbumModel *sourceModel))clickBlock{
    TZPhtotGroupView *moreview = [[TZPhtotGroupView alloc] init];
    moreview.clickBlock = clickBlock;
    moreview.currentModel = model;
    [moreview loadData];
    return moreview;
    
}

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self initUI];
    }
    return self;
}

#pragma mark delegate

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.sourceArray.count;
}
-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    TZPhtotGroupCell *cell = [tableView dequeueReusableCellWithIdentifier:@"TZPhtotGroupCell"];
    TZAlbumModel *model = self.sourceArray[indexPath.row];
    cell.titlaLable.text = model.name;
    [cell.titlaLable sizeToFit];
    if ([model.name isEqualToString:self.currentModel.name]) {
        cell.isSelectedCell = YES;
    }else{
        cell.isSelectedCell = NO;
    }
    return cell;;
}
-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    TZAlbumModel *model = self.sourceArray[indexPath.row];
    if (self.clickBlock) {
        self.clickBlock(model);
    }
    [self dismiss];
}


#pragma mark data
-(void)loadData{
    TZImagePickerController *imagePickerVc = (TZImagePickerController *)self.weakcontroller.navigationController;
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [[TZImageManager manager] getAllAlbumsWithFetchAssets:NO completion:^(NSArray<TZAlbumModel *> *models) {
            dispatch_async(dispatch_get_main_queue(), ^{
                self->_sourceArray = [NSMutableArray arrayWithArray:models];
                for (TZAlbumModel *albumModel in self->_sourceArray) {
                    albumModel.selectedModels = imagePickerVc.selectedModels;
                }
                [self->_tableview reloadData];
                
            });
        }];
    });
}

-(void)showFrom:(UIView *)view{
    [view addSubview:self];
    [self.tableview reloadData];
    self.frame = CGRectMake(0, 0, screenw, view.tz_height);
    CGFloat tableH = self.tableview.rowHeight * self.sourceArray.count;
    if (tableH>self.tz_height) {
        tableH = self.tz_height/2.0;
    }
    
    self.tableview.frame = CGRectMake(0,0, screenw, 0);
    [UIView animateWithDuration:0.2 animations:^(void){
        CGRect frame = self.tableview.frame;
        frame.size.height = tableH;
        self.tableview.frame = frame;
        self.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.3];
    }];
}

-(void)dismiss{
    [UIView animateWithDuration:0.2 animations:^(void){
        CGRect frame = self.tableview.frame;
        frame.size.height = 0;
        self.tableview.frame = frame;
        self.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.0];
    } completion:^(BOOL finished){
        [self removeFromSuperview];
    }];
}

#pragma mark UI
-(void)initUI{
    self.backgroundColor = [UIColor clearColor];
    [self addSubview:self.tableview];
    self.tableview.tz_height = 1;
}

#pragma mark lazy
-(UITableView *)tableview{
    if (!_tableview) {
        _tableview = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
        _tableview.backgroundColor = [Tools colorWithHex:0xffffffff];
        _tableview.separatorStyle = UITableViewCellSeparatorStyleNone;
        [_tableview registerClass:[TZPhtotGroupCell class] forCellReuseIdentifier:@"TZPhtotGroupCell"];
        _tableview.rowHeight = 50;
        _tableview.estimatedRowHeight = 50;
        _tableview.delegate = self;
        _tableview.dataSource = self;
    }
    return _tableview;
}

-(NSMutableArray *)sourceArray{
    if (!_sourceArray) {
        _sourceArray = [NSMutableArray array];
    }
    return _sourceArray;
}
@end





@interface TZPhtotGroupCell()
@property (nonatomic,strong)UIButton *selectedBtn;
@end

@implementation TZPhtotGroupCell

-(instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        self.selectionStyle = UITableViewCellSelectionStyleNone;
        [self initUI];
    }
    return self;
}
-(void)setIsSelectedCell:(BOOL *)isSelectedCell{
    _isSelectedCell = isSelectedCell;
    self.selectedBtn.selected = isSelectedCell;
    self.titlaLable.textColor = isSelectedCell?[Tools colorWithHex:0xFFFF6700]:[Tools colorWithHex:0xFF000000];
}


-(void)setFrame:(CGRect)frame{
    [super setFrame:frame];
    
    self.titlaLable.tz_centerY = frame.size.height/2.0;
    self.titlaLable.tz_left = 15;
    
    self.selectedBtn.tz_centerY = frame.size.height/2.0;
    self.selectedBtn.tz_left = frame.size.width - 12 - self.selectedBtn.tz_width;
}

#pragma mark initui
-(void)initUI{
    [self.contentView addSubview:self.titlaLable];
    [self.contentView addSubview:self.selectedBtn];
    self.selectedBtn.tz_size = CGSizeMake(40, 40);
}
#pragma mark lazy
-(UILabel *)titlaLable{
    if (!_titlaLable) {
        _titlaLable = [[UILabel alloc] init];
        _titlaLable.textAlignment = NSTextAlignmentCenter;
        _titlaLable.textColor = [Tools colorWithHex:0xFFFF6700];
    }
    return _titlaLable;
}

-(UIButton *)selectedBtn{
    if (!_selectedBtn) {
        _selectedBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_selectedBtn setImage:[UIImage tz_imageNamedFromMyBundle:@"group_is_selected"] forState:UIControlStateSelected];
        [_selectedBtn setImage:[UIImage new] forState:UIControlStateNormal];
        
    }
    return _selectedBtn;
}
@end
