//
//  TZThumbPhotoPreviewView.m
//  cloud_basic_pickers
//
//  Created by goboo on 2021/6/15.
//

#import "TZThumbPhotoPreviewView.h"
#import "UIView+TZLayout.h"
#import "Tools.h"
#import "TZImageManager.h"
#import "TZImagePickerController.h"
#import "TZImageCropManager.h"
#import <MediaPlayer/MediaPlayer.h>
@interface TZThumbPhotoPreviewView ()<UICollectionViewDelegate,UICollectionViewDataSource>

@end
@implementation TZThumbPhotoPreviewView
{
    UICollectionView *_thumbCollectionView;
    /// 最大选择图像数量
    NSInteger _maxPickerCount;
 
}


- (instancetype)initWithSource:(NSMutableArray *)album
               selectedAssets:(NSMutableArray *)selectedAssets
               maxPickerCount:(NSInteger)maxPickerCount
                    index:(NSInteger)index{
    
    self = [super init];
    if (self) {
        self.models = album;
        _maxPickerCount = maxPickerCount;
        
        self.currentIndex = (index < album.count) ? index : 0;
        [self initUIwithIndex:self.currentIndex];
        self.backgroundColor = [Tools colorWithHex:0xffffffff];
    }
    return self;
}

#pragma mark ---添加子控件---
-(void)initUIwithIndex:(NSInteger)index{
    
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
    layout.itemSize = CGSizeMake(70, 70);
    layout.minimumLineSpacing = 10;
    layout.minimumInteritemSpacing = 10;
    
    _thumbCollectionView = [[UICollectionView alloc] initWithFrame:CGRectMake(0, 0 ,  screenw, 90) collectionViewLayout:layout];
    _thumbCollectionView.delegate = self;
    _thumbCollectionView.dataSource = self;
    _thumbCollectionView.showsVerticalScrollIndicator = NO;
    _thumbCollectionView.showsHorizontalScrollIndicator = NO;
    _thumbCollectionView.backgroundColor = [Tools colorWithHex:0xffffffff];
    _thumbCollectionView.contentInset = UIEdgeInsetsMake(10, 10, 10,0);
    
    [_thumbCollectionView registerClass:[TZThumbPhotoPreviewCell class] forCellWithReuseIdentifier:@"TZThumbPhotoPreviewCell"];
    
    [self addSubview:_thumbCollectionView];
    [_thumbCollectionView reloadData];
    [_thumbCollectionView scrollToItemAtIndexPath:[NSIndexPath indexPathForItem:index inSection:0] atScrollPosition:UICollectionViewScrollPositionNone animated:NO];
    
    self.tz_height = _thumbCollectionView.tz_bottom;
}

#pragma mark ---UICollectionViewDataSource---
- (NSInteger)collectionView:(nonnull UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.models.count;
}
- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    TZThumbPhotoPreviewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"TZThumbPhotoPreviewCell" forIndexPath:indexPath];
   
    TZAssetModel *model = [self.models objectAtIndex:indexPath.item];
    
    cell.model = model;
    return cell;
}
#pragma mark ---UICollectionViewDelegate---
-(void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    
    NSLog(@"%s",__func__);
    TZAssetModel *model = [self.models objectAtIndex:indexPath.item];
    if (self.seleCtedBlock) {
        self.seleCtedBlock(indexPath.item, model);
    }
}


@end


@interface TZThumbPhotoPreviewCell ()
@property (nonatomic,strong)UIImageView *imageView;
@property (nonatomic,strong)UIButton *selectButton;
@property (nonatomic, assign) BOOL isRequestingGIF;
@property (nonatomic, assign) int32_t imageRequestID;
@end

@implementation TZThumbPhotoPreviewCell

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self initUI];
    }
    return self;
}

-(void)setModel:(TZAssetModel *)model{
    _model = model;
    
    self.isRequestingGIF = NO;
    if (model.type == TZAssetModelMediaTypePhotoGif) {
        // 先显示缩略图
        [[TZImageManager manager] getPhotoWithAsset:model.asset completion:^(UIImage *photo, NSDictionary *info, BOOL isDegraded) {
            if (photo) {
                self.imageView.image = photo;
            }
            if (self.isRequestingGIF) {
                return;
            }
            // 再显示gif动图
            self.isRequestingGIF = YES;
            [[TZImageManager manager] getOriginalPhotoDataWithAsset:model.asset progressHandler:^(double progress, NSError *error, BOOL *stop, NSDictionary *info) {
            
#ifdef DEBUG
                NSLog(@"[TZImagePickerController] getOriginalPhotoDataWithAsset:%f error:%@", progress, error);
#endif
            } completion:^(NSData *data, NSDictionary *info, BOOL isDegraded) {
                if (!isDegraded) {
                    self.isRequestingGIF = NO;
                    if ([TZImagePickerConfig sharedInstance].gifImagePlayBlock) {
                        [TZImagePickerConfig sharedInstance].gifImagePlayBlock(self, self.imageView, data, info);
                    } else {
                        self.imageView.image = [UIImage sd_tz_animatedGIFWithData:data];
                    }
                }
            }];
        } progressHandler:nil networkAccessAllowed:NO];
    } else {
        self.asset = model.asset;
    }
    
}


-(void)setAsset:(PHAsset *)asset{
    if (_asset && self.imageRequestID) {
        [[PHImageManager defaultManager] cancelImageRequest:self.imageRequestID];
    }
    _asset = asset;
    self.imageRequestID = [[TZImageManager manager] getPhotoWithAsset:asset completion:^(UIImage *photo, NSDictionary *info, BOOL isDegraded) {
        
        if (![asset isEqual:self->_asset]) return;
        if (photo) {
            self.imageView.image = photo;
        }

        if (!isDegraded) {
            self.imageRequestID = 0;
        }
    } progressHandler:^(double progress, NSError *error, BOOL *stop, NSDictionary *info) {
        if (![asset isEqual:self->_asset]) return;
      
      
        if (progress >= 1) {
            self.imageRequestID = 0;
        }
    } networkAccessAllowed:YES];
}

- (void)selectButtonAction{
    self.selectButton.selected = !self.selectButton.selected;
    
}

-(void)setFrame:(CGRect)frame{
    [super setFrame:frame];
    self.imageView.frame = CGRectMake(0, 0, self.tz_width, self.tz_height);
    [Tools cornerByRoundingWith:self.imageView Corners:UIRectCornerAllCorners cornerRadius:3];
}
#pragma mark ---添加子控件---
-(void)initUI{
    [self.contentView addSubview:self.imageView];
    [self.contentView addSubview:self.selectButton];
    
    
}
#pragma mark ---lazy---
-(UIImageView *)imageView{
    if (!_imageView) {
        _imageView = [[UIImageView alloc] init];
        _imageView.userInteractionEnabled = YES;
        _imageView.contentMode = UIViewContentModeScaleAspectFill;
        _imageView.clipsToBounds = YES;
        _imageView.backgroundColor = [UIColor whiteColor];
    }
    return _imageView;
}

-(UIButton *)selectButton{
    if (!_selectButton) {
        _selectButton = [UIButton buttonWithType:UIButtonTypeCustom];
//        [_selectButton setImage:nil forState:UIControlStateNormal];
        [_selectButton addTarget:self action:@selector(selectButtonAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _selectButton;
}

@end
