//
//  Tools.m
//  cloud_basic_pickers
//
//  Created by goboo on 2021/6/9.
//

#import "Tools.h"
#import <objc/runtime.h>

@implementation Tools

+(NSDictionary *)jsonStrToDictionary:(NSString *)jsonStr{

    if ([self isBlankString:jsonStr]) {
            return nil;
        }

        NSData *jsonData = [jsonStr dataUsingEncoding:NSUTF8StringEncoding];
        NSError *err;
        NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                            options:NSJSONReadingMutableContainers
                                                              error:&err];
        if(err)
        {
            NSLog(@"json解析失败：%@",err);
            return nil;
        }
        return dic;
}

#pragma mark ——————string相关——————
+  (BOOL) isBlankString:(id )string {
    if (string == nil || string == NULL) {
        return YES;
    }
    
    if ([string isKindOfClass:[NSNull class]]) {
        return YES;
    }
    if (![string isKindOfClass:[NSString class]]) {
        return NO;
    }
    NSAssert([string isKindOfClass:[NSString class]], @"当前的string不是字符串或者不是字符串的子类");
    
    if ([string isEqualToString:@""] || [string isEqual:@"<null>"] || [string isEqual:@"(null)"]) {
        return YES;
    }
    
    if ([[string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] length]==0) {
        return YES;
    }
    return NO;
}

//获取类的成员变量
+ (NSArray *)loadPropertiesWith:(Class)cls {
    
    unsigned int count = 0;
    
    // 返回值是所有属性的数组
    objc_property_t *properties = class_copyPropertyList(cls, &count);
    
    NSMutableArray *arrayM = [NSMutableArray arrayWithCapacity:count];
    
    for (int i = 0; i < count; ++i) {
        // 1. 从数组中获得属性
        objc_property_t pty = properties[i];
        
        // 2. 拿到属性名称
        const char *cname = property_getName(pty);
        [arrayM addObject:[NSString stringWithUTF8String:cname]];
    }
    
    // 释放属性数组
    free(properties);
    
    return arrayM;
}


+ (UIColor *)colorWithHex:(UInt32)hex{
    return [UIColor colorWithRed:((hex >> 16) & 0xFF)/255.0
                           green:((hex >> 8) & 0xFF)/255.0
                            blue:(hex & 0xFF)/255.0
                           alpha:((hex >> 24) & 0xFF)/255.0];
}


/**
 设置渐变背景
 
 @param colors 渐变颜色数组
 @param startPoint 开始点
 @param endPoint 结束点
 */
+(void)changeAlphaWithView:(UIView *)view Colors:(NSArray <UIColor *>*)colors start:(CGPoint)startPoint endPoint:(CGPoint)endPoint{
    
    if ([view.layer.sublayers.firstObject isKindOfClass:[CAGradientLayer class]]) {
        [view.layer.sublayers.firstObject removeFromSuperlayer];
    }
    
    CAGradientLayer *_gradLayer = [CAGradientLayer layer];
    NSMutableArray *CGColors = [NSMutableArray array];
    for (UIColor * color in colors) {
        [CGColors addObject:(id)color.CGColor];
    }
    
    [_gradLayer setColors:CGColors];
    //渐变起止点，point表示向量
    [_gradLayer setStartPoint:startPoint];
    [_gradLayer setEndPoint:endPoint];
    
    [_gradLayer setFrame:view.bounds];
    
    [view.layer insertSublayer:_gradLayer atIndex:0];
    
}


/**
 绘制圆角
 
 @param radius 半径
 @param color 颜色
 @param width 圆角路径宽度
 */
+(void)roundAngleWithView:(UIView *)view
                   Radius:(CGFloat)radius
                      color:(UIColor *)color
                  lineWidth:(CGFloat)width
{
    
    view.layer.masksToBounds       = YES;
    view.layer.cornerRadius        = radius;
    view.layer.borderColor         = color?color.CGColor:[UIColor clearColor].CGColor;
    view.layer.borderWidth         = width;
}

/**
 圆角
 */
+ (void)cornerByRoundingWith:(UIView *)view Corners:(UIRectCorner)corners cornerRadius:(CGFloat)cornerRadius{
    
    UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:view.bounds byRoundingCorners:corners cornerRadii:CGSizeMake(cornerRadius, cornerRadius)];
    CAShapeLayer *maskLayer = [[CAShapeLayer alloc] init];
    maskLayer.frame = view.bounds;
    maskLayer.path = maskPath.CGPath;
    view.layer.mask = maskLayer;
    view.layer.contentsScale = [[UIScreen mainScreen] scale];
}

@end
