import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:flutter/material.dart';

mixin ProgressBarController {
  void notifyProgress(double progress);
}

typedef OnBuildProgressBarCall = void Function(ProgressBarController controller);

class ProgressBar extends StatefulWidget {
  /// 进度背景颜色
  final Color backgroundColor;

  /// 进度颜色
  final Color progressColor;

  /// 显示进度文本
  final bool showProgressText;

  /// 文本字体颜色
  final Color progressTextColor;

  /// 文本字体大小
  final double progressTextSize;

  /// 进度最大值
  final double maxValue;

  /// 当前进度值
  final double progress;

  /// 组件宽度、高度
  final double width, height;

  /// 组件圆角大小
  final double radius;

  /// 组件构建完成后回调
  final OnBuildProgressBarCall? buildCall;

  const ProgressBar({
    Key? key,
    this.backgroundColor = const Color(0xeecccccc),
    this.progressColor = const Color(0xffADDE98),
    this.showProgressText = false,
    this.progressTextColor = const Color(0xff333333),
    this.progressTextSize = 11,
    this.maxValue = 100,
    this.progress = 0,
    this.width = 100,
    this.height = 16,
    this.radius = 20,
    this.buildCall,
  }) : super(key: key);

  @override
  _ProgressBarState createState() => _ProgressBarState();
}

class _ProgressBarState extends State<ProgressBar> with ProgressBarController {
  double _progress = 0;
  String _progressText = "";

  @override
  void initState() {
    super.initState();
    if (widget.buildCall != null) {
      widget.buildCall!(this);
    }
    _progress = widget.progress;
    _progressText = "${(widget.progress * 100) ~/ widget.maxValue}%";
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> lst = [];
    lst.add(buildBackground());
    lst.add(buildProgress());
    if (widget.showProgressText) {
      lst.add(buildText());
    }
    return Stack(
      children: lst,
    );
  }

  Widget buildBackground() {
    return Container(
      width: widget.width,
      height: widget.height,
      decoration: BoxDecoration(
        color: widget.backgroundColor,
        borderRadius: BorderRadius.all(Radius.circular(widget.radius)),
      ),
      // child: buildProgress(),
    );
  }

  Widget buildText() {
    return Positioned(
      child: Center(
        child: Text(
          _progressText,
          style: TextStyle(color: widget.progressTextColor, fontSize: widget.progressTextSize),
          textAlign: TextAlign.center,
        ),
      ),
      top: 0,
      left: 0,
      right: 0,
      bottom: 0,
    );
  }

  Widget buildProgress() {
    return Container(
      width: widget.width * (_progress / widget.maxValue),
      height: widget.height,
      decoration: BoxDecoration(
        color: widget.progressColor,
        borderRadius: BorderRadius.all(Radius.circular(widget.radius)),
      ),
    );
  }

  @override
  void notifyProgress(double progress) {
    setState(() {
      _progress = progress;
      _progressText = "${(progress * 100) ~/ widget.maxValue}%";
    });
  }
}
