import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';

mixin DropdownListController {
  void notifierLazy(String value);

  /// 设置组件启用状态
  void enable(bool enable);
}

/// DropdownList(
///   items: [
///     ListItem(value: "1", text: '选项1'),
///     ListItem(text: "选项2", value: "2"),
///   ],
///   width: 200,
///   title: "标题",
///   submitText: "确定",
///   isPreSelect: true,
///   isMultiSelect: true,
///   hintText: "请选择 ",
///   leastSelectHintText: "至少选择一项",
///   buttonBackgroundColors: const [Color(0xff1afa29), Color(0xff1afa29)],
///   buttonIconColor: Colors.white,
///   selectCall: (List<ListItem> selectItems) {},
/// )
class DropdownList extends StatefulWidget {
  /// 组件宽度
  final double? width;

  /// 组件高度
  final double height;

  /// 边框颜色
  final Color borderColor;

  /// 边框宽度
  final double borderWidth;

  /// 边框圆角
  final BorderRadius borderRadius;

  /// 弹窗面板圆角
  final BorderRadius dialogBorderRadius;

  /// 组件背景颜色
  final Color backgroundColor;

  /// 弹窗面板背景
  final Color dialogBackgroundColor;

  /// 按钮背景颜色
  final List<Color> buttonBackgroundColors;

  /// 按钮禁用状态背景颜色
  final List<Color> buttonDisableBackgroundColors;

  /// 按钮图标颜色
  final Color buttonIconColor;

  /// 菜单项
  final List<ListItem> items;

  /// 选择组件显示标题
  final String? title;

  /// 标题颜色
  final Color? titleColor;

  /// 标题大小
  final double? titleSize;

  /// 提交按钮
  final Widget? submit;

  /// 提交按钮文本与submit互斥
  final String? submitText;

  /// 提交按钮文本颜色
  final Color? submitTextColor;

  /// 提交按钮文本大小
  final double? submitTextSize;

  /// 提交按钮边框圆角
  final BorderRadiusGeometry submitBorderRadius;

  /// 提交按钮背景颜色
  final Color? submitBackgroundColor;

  /// true-选择后需要由确认完成;false-选择直接完成;
  final bool isPreSelect;

  /// 仅在isPreSelect=true时有效
  /// true-多选;false-单选;
  final bool isMultiSelect;

  /// 选择回调
  /// [selectItems] 选择项
  final Function(List<ListItem> selectItems) selectCall;

  /// 提示文本
  final String hintText;

  /// 至少选择提示文本
  final String leastSelectHintText;

  /// 默认值
  final String defaultValue;

  /// 构建完成后回传控制器
  final Function(DropdownListController controller)? buildCall;

  /// 是否启用
  final bool enable;

  /// 显示文本内边距
  final EdgeInsetsGeometry padding;

  /// 列表分割线颜色
  final Color? listSplitLineColor;

  /// 列表项高度
  final double itemHeight;

  /// 列表文本颜色
  final Color? itemTextColor;

  /// 列表文本大小
  final double? itemTextSize;

  /// 多选选中图标
  final Widget? multiSelectIcon;

  /// 多选未选中图标
  final Widget? multiUnSelectIcon;

  /// 单选选中图标
  final Widget? singleSelectIcon;

  /// 单选未选中图标
  final Widget? singleUnSelectIcon;

  const DropdownList({
    Key? key,
    this.width,
    this.height = 38,
    this.borderColor = const Color(0x60e4e4e4),
    this.borderWidth = 0,
    this.borderRadius = const BorderRadius.all(Radius.circular(4)),
    this.dialogBorderRadius = const BorderRadius.only(topLeft: Radius.circular(4), topRight: Radius.circular(4)),
    this.backgroundColor = Colors.white,
    this.buttonBackgroundColors = const [Color(0xff1afa29), Color(0xff1afa29)],
    this.buttonDisableBackgroundColors = const [Color(0x661afa29), Color(0x661afa29)],
    this.buttonIconColor = const Color(0x99333333),
    this.dialogBackgroundColor = Colors.white,
    required this.items,
    this.title,
    this.titleColor,
    this.titleSize,
    this.submit,
    this.submitText,
    this.submitTextColor,
    this.submitTextSize,
    this.submitBorderRadius = const BorderRadius.all(Radius.circular(5)),
    this.submitBackgroundColor,
    this.isPreSelect = true,
    this.isMultiSelect = false,
    required this.selectCall,
    this.hintText = "",
    this.leastSelectHintText = "",
    this.defaultValue = "",
    this.buildCall,
    this.enable = true,
    this.padding = const EdgeInsets.only(left: 6, right: 4),
    this.listSplitLineColor,
    this.itemHeight = 42,
    this.itemTextSize,
    this.itemTextColor,
    this.multiSelectIcon,
    this.multiUnSelectIcon,
    this.singleSelectIcon,
    this.singleUnSelectIcon,
  }) : super(key: key);

  @override
  _DropdownListState createState() => _DropdownListState();
}

class _DropdownListModel extends CusChangeNotifier {
  String displayText = "";
}

class _DropdownListState extends State<DropdownList> with DropdownListController {
  _DropdownListModel model = _DropdownListModel();
  YYDialog? listDialog;
  List<ListItem> _items = [];
  bool _enable = true;

  @override
  void initState() {
    super.initState();
    if (widget.buildCall != null) {
      widget.buildCall!(this);
    }
    _items = widget.items;
    _enable = widget.enable;
    defaultSelectItems();
  }

  void defaultSelectItems() {
    List<ListItem> selectItems = [];
    _items.forEach((element) {
      if (element.value == widget.defaultValue) {
        element.check = true;
        selectItems.add(element);
      } else if (element.check ?? false) {
        selectItems.add(element);
      }
    });
    notifyDisplayText(selectItems, false);
  }

  @override
  Widget build(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<_DropdownListModel>(model, (context, model) {
      return Container(
        width: widget.width,
        height: widget.height,
        decoration: BoxDecoration(
            color: widget.backgroundColor,
            borderRadius: BorderRadius.only(
              topLeft: widget.borderRadius.topLeft,
              bottomLeft: widget.borderRadius.bottomLeft,
              topRight: widget.borderRadius.topRight,
              bottomRight: widget.borderRadius.bottomRight,
            ),
            border: Border.all(color: widget.borderColor, width: widget.borderWidth)),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            Expanded(
              child: Padding(
                padding: widget.padding,
                child: Text(
                  model.displayText.isEmptyString ? widget.hintText : model.displayText,
                  style: TextStyle(color: Color(0xbe222222), fontSize: 13),
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                ),
              ),
            ),
            buildButton(),
          ],
        ),
      );
    });
  }

  Widget buildButton() {
    return Container(
      child: TextButton(
        child: Icon(
          (listDialog?.isShowing ?? false) ? Icons.keyboard_arrow_up : Icons.keyboard_arrow_down,
          color: widget.buttonIconColor,
        ),
        style: ButtonStyle(
          padding: MaterialStateProperty.all(EdgeInsets.zero),
          shape: MaterialStateProperty.all(RoundedRectangleBorder(borderRadius: BorderRadius.zero)),
          backgroundColor: MaterialStateProperty.all(Colors.transparent),
          visualDensity: VisualDensity.compact,
          minimumSize: MaterialStateProperty.all(Size(50, widget.height + 10)),
        ),
        onPressed: _enable ? showModel : null,
      ),
      width: buttonWidth,
      height: widget.height,
      decoration: BoxDecoration(
        borderRadius:
            BorderRadius.only(topRight: widget.borderRadius.topRight, bottomRight: widget.borderRadius.bottomRight),
        gradient: LinearGradient(
          colors: _enable ? widget.buttonBackgroundColors : widget.buttonDisableBackgroundColors,
          begin: Alignment.topCenter,
          end: Alignment.bottomCenter,
        ),
      ),
    );
  }

  double get buttonWidth {
    var width = widget.width ?? 0;
    if (width / 5 > 50) {
      return 50;
    } else if (width / 5 < 24) {
      return 24;
    } else {
      return width / 5;
    }
  }

  void showModel() {
    listDialog = Dialogs.instance.showListDialog(
      context,
      items: _items,
      gravity: Gravity.bottom,
      title: widget.title,
      titleColor: widget.titleColor,
      titleSize: widget.titleSize,
      submit: widget.submit,
      isClickOutsideDismiss: true,
      submitText: widget.submitText,
      submitTextColor: widget.submitTextColor,
      submitTextSize: widget.submitTextSize,
      submitBorderRadius: widget.submitBorderRadius,
      submitBackgroundColor: widget.submitBackgroundColor,
      listSplitLineColor: widget.listSplitLineColor,
      isPreSelect: widget.isPreSelect,
      isMultiSelect: widget.isMultiSelect,
      leastSelectHintText: widget.leastSelectHintText,
      backgroundColor: widget.dialogBackgroundColor,
      borderRadius: widget.dialogBorderRadius,
      itemHeight: widget.itemHeight,
      itemTextSize: widget.itemTextSize,
      itemTextColor: widget.itemTextColor,
      multiSelectIcon: widget.multiSelectIcon,
      multiUnSelectIcon: widget.multiUnSelectIcon,
      singleSelectIcon: widget.singleSelectIcon,
      singleUnSelectIcon: widget.singleUnSelectIcon,
      itemPressed: (List<ListItem> selectItems) {
        notifyDisplayText(selectItems, true);
        widget.selectCall(selectItems);
      },
      buildCall: (YYDialog dialog) {
        model.notifierLazy();
      },
      dismissCall: () {
        Future.delayed(Duration.zero, () {
          model.notifierLazy();
        });
      },
    );
  }

  void notifyDisplayText(List<ListItem> selectItems, bool isNotify) {
    model.displayText = "";
    var count = selectItems.length;
    for (int i = 0; i < count; i++) {
      var item = selectItems[i];
      model.displayText += item.text;
      if ((i + 1) < count) {
        model.displayText += ";";
      }
    }
    model.notifierLazy();
  }

  @override
  notifierLazy(String value) {
    if (widget.isMultiSelect) {
      _items.forEach((element) {
        if (element.value == value) {
          element.check = true;
        }
      });
    } else {
      _items.forEach((element) {
        if (element.value == value) {
          element.check = true;
        } else {
          element.check = false;
        }
      });
    }
  }

  @override
  void enable(bool enable) {
    _enable = enable;
    model.notifierLazy();
  }
}
