import 'package:eyflutter_uikit/uikit/options/item_option.dart';
import 'package:eyflutter_uikit/uikit/options/option_panel_widget.dart';
import 'package:flutter/material.dart';

mixin OptionColumnPanelController {
  void notifyCheck(ItemOption option);
}

typedef OptionColumnPanelBuildCall = void Function(OptionColumnPanelController controller);

class OptionColumnPanelWidget extends StatefulWidget {
  /// 背景颜色
  final Color? backgroundColor;

  /// item禁用是背景色
  final Color itemDisableColor;

  /// 选项集合
  final List<ItemOption> items;

  /// 选项高度
  final double itemWidth;

  /// 选项高度
  final double itemHeight;

  /// 分割线高度
  final double dividerHeight;

  /// 分割线颜色
  final Color dividerColor;

  /// 文本样式
  final TextStyle textStyle;

  /// 内边距
  final EdgeInsetsGeometry padding;

  /// 选项单击事件
  final OnItemOptionClickCall? optionClickCall;

  final OptionColumnPanelBuildCall? buildCall;

  const OptionColumnPanelWidget(
      {Key? key,
      required this.items,
      required this.itemWidth,
      required this.itemHeight,
      this.backgroundColor,
      this.itemDisableColor = Colors.black12,
      this.dividerHeight = 0,
      this.dividerColor = Colors.white,
      this.textStyle = const TextStyle(color: Colors.white, fontSize: 12),
      this.padding = EdgeInsets.zero,
      this.optionClickCall,
      this.buildCall})
      : super(key: key);

  @override
  _OptionColumnPanelWidgetState createState() => _OptionColumnPanelWidgetState();
}

class _OptionColumnPanelWidgetState extends State<OptionColumnPanelWidget> with OptionColumnPanelController {
  @override
  void initState() {
    if (widget.buildCall != null) {
      widget.buildCall!(this);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return OptionPanelWidget(
      backgroundColor: widget.backgroundColor ?? Colors.white,
      child: IntrinsicWidth(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: _buildItems(),
        ),
      ),
    );
  }

  List<Widget> _buildItems() {
    List<Widget> lst = [];
    int i = 0;
    int len = widget.items.length;
    widget.items.forEach((element) {
      lst.add(_buildItem(element));
      if ((i + 1) < len) {
        lst.add(Container(
          width: double.infinity,
          height: widget.dividerHeight,
          color: widget.dividerColor,
        ));
      }
      i++;
    });
    return lst;
  }

  Widget _buildItem(ItemOption item) {
    return Container(
      width: widget.itemWidth,
      height: widget.itemHeight,
      color: item.enable ? Color(0x00ffffff) : widget.itemDisableColor,
      child: TextButton(
          onPressed: item.enable
              ? () {
                  if (widget.optionClickCall != null) {
                    widget.optionClickCall!(item);
                  }
                }
              : null,
          child: Padding(
            padding: widget.padding,
            child: Row(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                item.icon ?? Container(),
                Expanded(
                  child: Center(
                    child: Text(
                      item.title ?? "",
                      style: item.textStyle,
                    ),
                  ),
                ),
                Offstage(
                  offstage: !item.isCheck,
                  child: Container(
                    padding: EdgeInsets.only(right: 10),
                    child: Icon(
                      Icons.check,
                      color: Color(0xffEB3800),
                      size: 16,
                    ),
                  ),
                )
              ],
            ),
          )),
    );
  }

  @override
  void notifyCheck(ItemOption option) {
    setState(() {
      widget.items.forEach((element) {
        if (element.value == option.value) {
          element.isCheck = true;
        } else {
          element.isCheck = false;
        }
      });
    });
  }
}
