import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/options/item_option.dart';
import 'package:eyflutter_uikit/uikit/options/option_panel_widget.dart';
import 'package:flutter/material.dart';

class OptionGridPanelWidget extends StatefulWidget {
  /// item宽度
  final double itemWidth;

  /// item高度
  final double itemHeight;

  /// 背景颜色
  final Color? backgroundColor;

  /// 内边距
  final EdgeInsetsGeometry? padding;

  /// 副轴数量
  final int? crossAxisCount;

  /// 负轴间隔
  final double? crossAxisSpacing;

  /// 主轴间隔
  final double? mainAxisSpacing;

  /// 选项集合
  final List<ItemOption> items;

  /// 文本样式
  final TextStyle textStyle;

  /// 文本内边距
  final EdgeInsetsGeometry textPadding;

  /// 选项单击事件
  final OnItemOptionClickCall? optionClickCall;

  const OptionGridPanelWidget(
      {Key? key,
      required this.items,
      required this.itemWidth,
      required this.itemHeight,
      this.backgroundColor,
      this.padding,
      this.crossAxisCount,
      this.crossAxisSpacing,
      this.mainAxisSpacing,
      this.textStyle = const TextStyle(color: Colors.white, fontSize: 12),
      this.textPadding = EdgeInsets.zero,
      this.optionClickCall})
      : super(key: key);

  @override
  _OptionGridPanelWidgetState createState() => _OptionGridPanelWidgetState();
}

class _OptionGridPanelWidgetState extends State<OptionGridPanelWidget> {
  @override
  Widget build(BuildContext context) {
    double width = _gridWidth();
    double height = _gridHeight();
    return OptionPanelWidget(
        width: width,
        height: height,
        backgroundColor: widget.backgroundColor ?? Colors.white,
        child: GridView.count(
          padding: widget.padding ?? EdgeInsets.zero,
          crossAxisCount: _crossAxisCount(),
          crossAxisSpacing: widget.crossAxisSpacing ?? 0,
          mainAxisSpacing: widget.mainAxisSpacing ?? 0,
          shrinkWrap: true,
          physics: NeverScrollableScrollPhysics(),
          children: widget.items.map((item) => _buildItem(item)).toList(),
        ));
  }

  Widget _buildItem(ItemOption item) {
    return TextButton(
        onPressed: () {
          if (widget.optionClickCall != null) {
            widget.optionClickCall!(item);
          }
        },
        child: Center(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: _buildGridItem(item),
          ),
        ));
  }

  List<Widget> _buildGridItem(ItemOption item) {
    List<Widget> lst = [];
    if (item.icon != null) {
      lst.add(item.icon!);
    }
    if (item.title.isNotEmptyString) {
      lst.add(Container(
        padding: widget.textPadding,
        child: Text(
          item.title ?? "",
          style: widget.textStyle,
        ),
      ));
    }
    return lst;
  }

  double _gridHeight() {
    int crossCount = _crossAxisCount();
    var len = widget.items.length;
    int rowCount = (len % crossCount == 0) ? (len ~/ crossCount) : (len ~/ crossCount + 1);
    double height = rowCount * widget.itemHeight + (rowCount - 1) * (widget.mainAxisSpacing ?? 0);
    if (widget.padding != null) {
      height += widget.padding?.vertical ?? 0;
    }
    return height;
  }

  double _gridWidth() {
    int crossCount = _crossAxisCount();
    double width = crossCount * widget.itemWidth + (crossCount - 1) * (widget.crossAxisSpacing ?? 0);
    if (widget.padding != null) {
      width += widget.padding?.horizontal ?? 0;
    }
    return width;
  }

  int _crossAxisCount() {
    var max = widget.crossAxisCount ?? 1;
    var len = widget.items.length;
    return max > len ? len : max;
  }
}
