import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/state/lifecycle_state.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:flutter/material.dart';

abstract class BaseState<T extends StatefulWidget> extends LifecycleState<T> with OnLangChangeState {
  Locale? _locale;

  @override
  void onCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _locale = Localizations.localeOf(context);
    LangManager.instance.addChangeState(this.hashCode, this);
    var stateListener = StateUtils.instance.getStateListener();
    stateListener?.onBuildCall(context: context, viewName: viewName);
  }

  @override
  void dispatchLangState(Locale locale) {
    setState(() {
      _locale = locale;
    });
  }

  @override
  void onResume(String routeName, Map<String, dynamic> arguments) {
    var stateListener = StateUtils.instance.getStateListener();
    stateListener?.onResume(viewName, routeName, arguments);
  }

  @override
  void onPause(String routeName, Map<String, dynamic> arguments) {
    var stateListener = StateUtils.instance.getStateListener();
    stateListener?.onPause(viewName, routeName, arguments);
  }

  @override
  void onDestroy(String routeName, Map<String, dynamic> arguments) {
    LangManager.instance.removeChangeState(this.hashCode);
  }

  /// 是否预留顶部安全区域
  bool get topSafeArea => false;

  /// 是否预留底部安全区域
  bool get bottomSafeArea => false;

  ///在@override#build中调用
  Widget overrideLocalizations(
      {required BuildContext context, required Widget child, Color backgroundColor = const Color(0xffe8e8e8)}) {
    return SafeArea(
      child: Scaffold(
        backgroundColor: backgroundColor,
        body: Localizations.override(
          context: context,
          locale: _locale,
          child: GestureDetector(
            child: child,
            onTap: () => SoftUtils.hideSoft(context),
          ),
        ),
      ),
      top: topSafeArea,
      bottom: bottomSafeArea,
    );
  }
}
