import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/state/base_form_state.dart';
import 'package:eyflutter_uikit/uikit/state/lifecycle_state.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// 状态工具
class StateUtils {
  factory StateUtils() => _getInstance();

  static StateUtils get instance => _getInstance();
  static StateUtils? _instance;

  static StateUtils _getInstance() {
    return _instance ??= StateUtils._internal();
  }

  StateUtils._internal();

  String _stateKey = "23641c7917af37bc";

  /// 注册构建监听器
  void registerConfig<T extends OnStateListener>(T listener) {
    ConfigManager.instance.addConfig(_stateKey, listener);
  }

  OnStateListener? getStateListener() {
    var config = ConfigManager.instance.getConfig(_stateKey);
    if (config is! OnStateListener) {
      return null;
    }
    return config;
  }

  /// 消费对象组件转换
  /// [context] 视图上下文
  /// [builder] 对象消费者构建器
  /// [parent] 父对象model
  Widget convertToWidget(
      {required BuildContext context, required ConsumerBuilder builder, required ProviderModel parent}) {
    builder.setParentModel(parent);
    return builder.builder(context);
  }

  Widget subscribeNotifierWidget<M extends ChangeNotifier>(M model, ConsumerWidgetBuilder<M> childBuilder) {
    return ChangeNotifierProvider<M>(
      create: (_) => model,
      child: Consumer<M>(
        builder: (context, m, _) {
          return childBuilder(context, m);
        },
      ),
    );
  }

  /// 是否首页数据
  bool isFirstPage(Map<String, dynamic> response) {
    if (response.isEmptyMap()) {
      return true;
    }
    var fieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response[fieldName];
    if (isFirstPage is! bool) {
      return true;
    }
    return isFirstPage;
  }
}
