import 'dart:collection';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/webkit/script_handler_callback.dart';
import 'package:eyflutter_webview/eyflutter_webview.dart';

class _JavascriptCallReceive {
  String methodName;
  JavascriptMessage message;
  JavascriptMessageHandler messageHandler;

  _JavascriptCallReceive(this.methodName, this.message, this.messageHandler);

  void call() {
    messageHandler(message);
  }
}

class JavascriptHandler<Extras> {
  final Extras? extras;
  final HashMap<String, JavascriptChannel> jsChannels = HashMap<String, JavascriptChannel>();
  ScriptHandlerCallback? _handlerCallback;

  JavascriptHandler({this.extras});

  void registerBridges() {
    //step1
  }

  void setCallHandler(ScriptHandlerCallback callback) {
    this._handlerCallback = callback;
  }

  void evalJavascript(String methodName, List<String> params) {
    this._handlerCallback?.evalJavascript(methodName, params);
  }

  ///添加脚本注册方法
  void addHandler(String methodName, JavascriptMessageHandler onMessageReceived) {
    if (methodName.isEmpty) {
      return;
    }
    JavascriptChannel channel = new JavascriptChannel(
        name: methodName,
        onMessageReceived: (JavascriptMessage message) {
          var callReceive = _JavascriptCallReceive(methodName, message, onMessageReceived);
          callReceive.call();
        });
    if (jsChannels.containsKey(methodName)) {
      jsChannels[methodName] = channel;
    } else {
      jsChannels.putIfAbsent(methodName, () => channel);
    }
  }

  /// 生成加入js方法
  /// [method]js方法带参数不带返回类型
  /// [returnValue]js方法返回值
  String genJoinJsMethod(List<String> bridgeNames, {required String method, dynamic returnValue}) {
    var hasMatch = new RegExp("[(](.*)[)]").hasMatch(method);
    if (!hasMatch) {
      return "";
    }
    dynamic value;
    var type = returnValue.runtimeType.toString().toLowerCase();
    if (type == "bool" || type == "int" || type == "double") {
      value = returnValue;
    } else {
      value = '"$returnValue"';
    }
    var filter = new RegExp("[(](.*)[)]").firstMatch(method);
    var group = filter?.group(0);
    if (group.isEmptyString) {
      return "";
    }
    var _method = method.replaceAll(group!, "");
    var buffer = new StringBuffer('joinJsMethod9794([');
    bridgeNames.forEach((String bridge) {});
    for (int i = 0; i < bridgeNames.length; i++) {
      buffer.write("'${bridgeNames[i]}'");
      if ((i + 1) < bridgeNames.length) {
        buffer.write(",");
      }
    }
    buffer.write("], '$_method', function $group { return $value; });");
    return buffer.toString();
  }
}
