import 'package:eyflutter_uikit/uikit/widgets/icon_widget.dart';
import 'package:flutter/material.dart';

/// 选择状态事件
typedef OnCheckedCall = void Function(bool check, dynamic tag, int position);

typedef OnBuildCall = void Function(CheckController controller);

mixin CheckController {
  void notify(bool check);
}

class CheckButton extends StatefulWidget {
  /// 选择项事件
  final OnCheckedCall? checkedCall;

  /// 文本
  final String text;

  /// 文本大小
  final double textSize;

  /// 文本颜色
  final Color textColor;

  /// 选择图标颜色
  final Color selectIconColor;

  /// 未选择图标颜色
  final Color unSelectIconColor;

  /// 图标大小
  final double iconSize;

  /// 选中图标视图
  final IconData checkIcon;

  /// 未选中图标视图
  final IconData uncheckIcon;

  /// 文本内边距
  final EdgeInsetsGeometry textPadding;

  /// 标签数据
  final dynamic tag;

  /// 默认是否被选中
  final bool check;

  /// 组件在父容器中的索引(可选)
  final int? position;

  /// 组件构建回调
  final OnBuildCall? buildCall;

  /// 是否允许反选
  final bool allowReverseSelection;

  /// 是否填充容器
  final bool isTile;

  CheckButton({
    Key? key,
    required this.text,
    this.textSize = 12,
    this.textColor = const Color(0xff333333),
    this.selectIconColor = const Color(0xffEEEBD7),
    this.unSelectIconColor = const Color(0xffEEEBD7),
    this.iconSize = 20,
    this.checkedCall,
    this.textPadding = const EdgeInsets.only(left: 6),
    this.checkIcon = Icons.check_box_outlined,
    this.uncheckIcon = Icons.check_box_outline_blank,
    this.tag,
    this.check = false,
    this.position,
    this.buildCall,
    this.allowReverseSelection = true,
    this.isTile = false,
  }) : super(key: key);

  @override
  CheckButtonState createState() => CheckButtonState();
}

class CheckButtonState extends State<CheckButton> with CheckController {
  bool isCheck = false;
  late IconData _iconData;
  late Color _iconColor;

  @override
  void initState() {
    if (widget.check) {
      _iconData = widget.checkIcon;
      _iconColor = widget.selectIconColor;
    } else {
      _iconData = widget.uncheckIcon;
      _iconColor = widget.unSelectIconColor;
    }
    if (widget.buildCall != null) {
      widget.buildCall!(this);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [buildCheck(), widget.isTile ? Expanded(child: buildCheckText()) : buildCheckText()],
    );
  }

  Widget buildCheckText() {
    return GestureDetector(
      child: Container(
        padding: widget.textPadding,
        child: Text(
          widget.text,
          style: TextStyle(fontSize: widget.textSize, color: widget.textColor),
          maxLines: 1,
        ),
      ),
      onTap: checkEvent,
    );
  }

  Widget buildCheck() {
    return GestureDetector(
      child: IconWidget(
        _iconData,
        color: _iconColor,
        size: widget.iconSize,
      ),
      onTap: checkEvent,
    );
  }

  void checkEvent() {
    if (!widget.allowReverseSelection && isCheck) {
      return;
    }
    isCheck = !isCheck;
    setState(() {
      if (isCheck) {
        _iconData = widget.checkIcon;
        _iconColor = widget.selectIconColor;
      } else {
        _iconData = widget.uncheckIcon;
        _iconColor = widget.unSelectIconColor;
      }
    });
    if (widget.checkedCall != null) {
      widget.checkedCall!(isCheck, widget.tag, widget.position ?? 0);
    }
  }

  void updateCheck(bool check) {
    isCheck = check;
    setState(() {
      if (check) {
        _iconData = widget.checkIcon;
      } else {
        _iconData = widget.uncheckIcon;
      }
    });
  }

  @override
  void notify(bool check) {
    isCheck = check;
    setState(() {
      if (isCheck) {
        _iconData = widget.checkIcon;
      } else {
        _iconData = widget.uncheckIcon;
      }
    });
  }
}
