import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:flutter/material.dart';

class IconWidget extends Icon {
  IconWidget(
    IconData icon, {
    Key? key,
    double? size,
    Color? color,
    String? semanticLabel,
    TextDirection? textDirection,
    this.isVisible = true,
    this.padding = EdgeInsets.zero,
    this.widthFactor,
    this.heightFactor,
    this.borderRadius,
  }) : super(icon, key: key, size: size, color: color, semanticLabel: semanticLabel, textDirection: textDirection);

  /// 是否显示组件，默认显示
  final bool isVisible;

  /// icon padding
  final EdgeInsetsGeometry padding;

  /// 横向裁剪比例
  final double? widthFactor;

  /// 纵向裁剪比例
  final double? heightFactor;

  /// 圆角属性
  final BorderRadius? borderRadius;

  @override
  Widget build(BuildContext context) {
    return Offstage(
      offstage: !isVisible,
      child: Padding(
        padding: padding,
        child: (widthFactor.isNullOrZero && heightFactor.isNullOrZero)
            ? super.build(context)
            : ClipRRect(
                borderRadius: borderRadius,
                child: ClipRect(
                  child: Align(
                    alignment: Alignment.center,
                    widthFactor: widthFactor,
                    heightFactor: heightFactor,
                    child: super.build(context),
                  ),
                ),
              ),
      ),
    );
  }
}
