import 'dart:async';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:flutter/material.dart';

class TimeCountModel extends ProviderModel {
  int _time = 0;
  Timer? _timer;
  String timeText = "00:00:00";

  /// 获取统计时间
  int get countTime => _time;

  void _timing() {
    _timer = Timer.periodic(Duration(seconds: 1), (timer) {
      _time++;
      int hour = _time ~/ 3600;
      int minutes = (_time - hour * 3600) ~/ 60;
      int seconds = _time - hour * 3600 - minutes * 60;
      timeText = "${hour >= 10 ? hour : "0$hour"}:${minutes >= 10 ? minutes : "0$minutes"}:${seconds >= 10
          ? seconds
          : "0$seconds"}";
      notifierLazy();
    });
  }

  /// 开始计时
  void start() {
    cancel();
    _timing();
    notifierLazy();
  }

  /// 重置计时
  void reset() {
    _time = 0;
    start();
  }

  /// 取消计时统计
  void cancel() {
    if (_timer?.isActive ?? false) {
      _timer?.cancel();
    }
  }
}

/// 计时统计
class TimeCount extends StatefulWidget {
  /// 计时统计模型
  final TimeCountModel countModel;

  const TimeCount({Key? key, required this.countModel}) : super(key: key);

  @override
  _TimeState createState() => _TimeState();
}

class _TimeState extends State<TimeCount> {
  @override
  Widget build(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<TimeCountModel>(widget.countModel, (context, model) {
      return Text(
        model.timeText,
        style: TextStyle(color: Color(0xff333333), fontSize: 14),
      );
    });
  }

  @override
  void dispose() {
    widget.countModel.cancel();
    super.dispose();
  }
}
